Package: DEXSeq
Version: 1.58.0
Title: Inference of differential exon usage in RNA-Seq
Authors@R: 
  c(
    person("Simon", "Anders", email = "sanders@fs.tum.de", role = "aut"),
    person("Alejandro", "Reyes", email = "alejandro.reyes.ds@gmail.com", role = c("aut", "ccp"), 
           comment = "Maintainer until 2026."),
    person("Hugo", "Gruson", email = "hugo.gruson@embl.de", role = "cre")
  )
Imports: BiocGenerics, biomaRt, hwriter, methods, stringr, Rsamtools,
        statmod, geneplotter, genefilter
Depends: BiocParallel, Biobase, SummarizedExperiment, IRanges (>=
        2.5.17), GenomicRanges (>= 1.23.7), DESeq2 (>= 1.39.6),
        AnnotationDbi, S4Vectors (>= 0.23.18)
Suggests: GenomeInfoDb, GenomicFeatures, txdbmaker, pasilla (>=
        0.2.22), BiocStyle, knitr, rmarkdown, testthat,
        pasillaBamSubset, GenomicAlignments, roxygen2, glmGamPoi
Description: The package is focused on finding differential exon usage
        using RNA-seq exon counts between samples with different
        experimental designs. It provides functions that allows the
        user to make the necessary statistical tests based on a model
        that uses the negative binomial distribution to estimate the
        variance between biological replicates and generalized linear
        models for testing. The package also provides functions for the
        visualization and exploration of the results.
License: GPL (>= 3)
URL:
biocViews: ImmunoOncology, Sequencing, RNASeq, DifferentialExpression,
        AlternativeSplicing, DifferentialSplicing, GeneExpression,
        Visualization
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/pak/sysreqs: libbz2-dev libicu-dev liblzma-dev libpng-dev
        libxml2-dev libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:35:31 UTC
RemoteUrl: https://github.com/bioc/DEXSeq
RemoteRef: RELEASE_3_23
RemoteSha: 847ca9b8fec10d109175501051311f6cca1453d7
NeedsCompilation: no
Packaged: 2026-04-29 15:03:47 UTC; root
Author: Simon Anders [aut],
  Alejandro Reyes [aut, ccp] (Maintainer until 2026.),
  Hugo Gruson [cre]
Maintainer: Hugo Gruson <hugo.gruson@embl.de>
Built: R 4.6.0; ; 2026-04-29 15:21:54 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DEXSeq)
export(DEXSeqDataSet)
export(DEXSeqDataSetFromHTSeq)
export(DEXSeqDataSetFromSE)
export(DEXSeqHTML)
export(DEXSeqResults)
export(`exonIDs<-`)
export(`featureIDs<-`)
export(`geneIDs<-`)
export(`groupIDs<-`)
export(estimateExonFoldChanges)
export(exonIDs)
export(featureCounts)
export(featureIDs)
export(geneIDs)
export(groupIDs)
export(perGeneQValue)
export(plotDEXSeq)
export(sampleAnnotation)
export(testForDEU)
exportClasses(DEXSeqDataSet)
exportClasses(DEXSeqResults)
exportMethods(`$<-`)
exportMethods(`[`)
exportMethods(`colData<-`)
exportMethods(counts)
exportMethods(estimateDispersions)
exportMethods(estimateSizeFactors)
exportMethods(findOverlaps)
exportMethods(plotMA)
exportMethods(subsetByOverlaps)
import(Biobase)
import(BiocGenerics)
import(BiocParallel)
import(DESeq2)
import(GenomicRanges)
import(IRanges)
import(Rsamtools)
import(S4Vectors)
import(SummarizedExperiment)
import(biomaRt)
import(genefilter)
import(geneplotter)
import(hwriter)
import(methods)
import(statmod)
import(stringr)
importFrom(AnnotationDbi,select)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,palette.colors)
importFrom(grDevices,rgb)
importFrom(grDevices,svg)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,model.matrix)
importFrom(stats,relevel)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,weighted.mean)
importFrom(utils,read.delim)
importFrom(utils,read.table)

## ----echo=FALSE, include=FALSE------------------------------------------------
knitr::opts_chunk$set(tidy = FALSE,
                      cache = TRUE,
                      dev = "png",
                      message = FALSE,
                      error = FALSE,
                      warning = TRUE)

## ----warning=FALSE------------------------------------------------------------

library(GenomicFeatures)  # for the exonicParts() function
library(txdbmaker)        # for the makeTxDbFromGFF() function
download.file(
    "https://ftp.ensembl.org/pub/release-62/gtf/drosophila_melanogaster/Drosophila_melanogaster.BDGP5.25.62.gtf.gz",
    destfile="Drosophila_melanogaster.BDGP5.25.62.gtf.gz")
txdb = makeTxDbFromGFF("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

file.remove("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

flattenedAnnotation = exonicParts( txdb, linked.to.single.gene.only=TRUE )
names(flattenedAnnotation) =
    sprintf("%s:E%0.3d", flattenedAnnotation$gene_id, flattenedAnnotation$exonic_part)


## -----------------------------------------------------------------------------

library(pasillaBamSubset)
library(Rsamtools)
library(GenomicAlignments)
library(GenomeInfoDb)
bamFiles = c( untreated1_chr4(), untreated3_chr4() )
bamFiles = BamFileList( bamFiles )
seqlevelsStyle(flattenedAnnotation) = "UCSC"
se = summarizeOverlaps(
    flattenedAnnotation, BamFileList(bamFiles), singleEnd=FALSE,
    fragments=TRUE, ignore.strand=TRUE )


## ----buildExonCountSet--------------------------------------------------------
library(DEXSeq)
colData(se)$condition = factor(c("untreated1", "untreated3"))
colData(se)$libType = factor(c("single-end", "paired-end"))
dxd = DEXSeqDataSetFromSE( se, design= ~ sample + exon + condition:exon )

## ----startVignette------------------------------------------------------------
data(pasillaDEXSeqDataSet, package="pasilla")

## -----------------------------------------------------------------------------
colData(dxd)

## -----------------------------------------------------------------------------
head( counts(dxd), 5 )

## -----------------------------------------------------------------------------
split( seq_len(ncol(dxd)), colData(dxd)$exon )

## -----------------------------------------------------------------------------
head( featureCounts(dxd), 5 )

## -----------------------------------------------------------------------------
head( rowRanges(dxd), 3 )

## -----------------------------------------------------------------------------
sampleAnnotation( dxd )

## -----------------------------------------------------------------------------
dxd = estimateSizeFactors( dxd )

## -----------------------------------------------------------------------------
dxd = estimateDispersions( dxd )

## ----fitdiagnostics, fig.small=TRUE, fig.cap="Fit Diagnostics. The initial per-exon dispersion estimates (shown by black points), the fitted mean-dispersion values function (red line), and the shrinked values in blue"----
plotDispEsts( dxd )

## ----testForDEU1--------------------------------------------------------------
dxd = testForDEU( dxd )

## ----estFC--------------------------------------------------------------------
dxd = estimateExonFoldChanges( dxd, fitExpToVar="condition")

## ----results1-----------------------------------------------------------------
dxr1 = DEXSeqResults( dxd )
dxr1

## ----results2-----------------------------------------------------------------
mcols(dxr1)$description

## ----tallyExons---------------------------------------------------------------
table ( dxr1$padj < 0.1 )

## ----tallyGenes---------------------------------------------------------------
table ( tapply( dxr1$padj < 0.1, dxr1$groupID, any ) )

## ----MvsA, fig.small=TRUE, fig.cap="MA plot. Mean expression versus $log{2}$ fold change plot. Significant hits at an $FDR=0.1$ are coloured in red."----
plotMA( dxr1, cex=0.8 )

## ----design-------------------------------------------------------------------
sampleAnnotation(dxd)

## ----formulas2----------------------------------------------------------------
formulaFullModel    =  ~ sample + exon + type:exon + condition:exon
formulaReducedModel =  ~ sample + exon + type:exon 

## ----estDisps_again-----------------------------------------------------------
dxd = estimateDispersions( dxd, formula = formulaFullModel )

## ----test_again---------------------------------------------------------------
dxd = testForDEU( dxd, 
	reducedModel = formulaReducedModel, 
        fullModel = formulaFullModel )

## ----res_again----------------------------------------------------------------
dxr2 = DEXSeqResults( dxd )

## ----table2-------------------------------------------------------------------
table( dxr2$padj < 0.1 )

## ----table3-------------------------------------------------------------------
table( before = dxr1$padj < 0.1, now = dxr2$padj < 0.1 )

## ----plot1, fig.height=8, fig.width=12, fig.cap="Fitted expression. The plot represents the expression estimates from a call to `testForDEU()`. Shown in red is the exon that showed significant differential exon usage."----
plotDEXSeq( dxr2, "FBgn0010909", legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----checkClaim---------------------------------------------------------------
wh = (dxr2$groupID=="FBgn0010909")
stopifnot(sum(dxr2$padj[wh] < formals(plotDEXSeq)$FDR)==1)

## ----plot2, fig.height=8, fig.width=12, fig.cap="Transcripts. As in the previous plots, but including the annotated transcript models"----
plotDEXSeq( dxr2, "FBgn0010909", displayTranscripts=TRUE, legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----plot3, fig.height=8, fig.width=12, fig.cap="Normalized counts. As in the previous plots, with normalized count values of each exon in each of the samples."----
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, norCounts=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----plot4, fig.height=8, fig.width=12, fig.cap="Fitted splicing. As in the previous plots, but after subtraction of overall changes in gene expression."----
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, splicing=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----DEXSeqHTML, cache=TRUE, eval=FALSE---------------------------------------
# DEXSeqHTML( dxr2, FDR=0.1, color=c("#FF000080", "#0000FF80") )

## ----para1,cache=TRUE,results='hide', eval=FALSE------------------------------
# BPPARAM = BiocParallel::MulticoreParam(4)
# dxd = estimateSizeFactors( dxd )
# dxd = estimateDispersions( dxd, BPPARAM=BPPARAM)
# dxd = testForDEU( dxd, BPPARAM=BPPARAM)
# dxd = estimateExonFoldChanges(dxd, BPPARAM=BPPARAM)

## ----alldeu-------------------------------------------------------------------
dxr = DEXSeq(dxd)
class(dxr)

## ----pergeneqval--------------------------------------------------------------
numbOfGenes = sum( perGeneQValue(dxr) < 0.1)
numbOfGenes

## ----systemFile---------------------------------------------------------------
pythonScriptsDir = system.file( "python_scripts", package="DEXSeq" )
list.files(pythonScriptsDir)
system.file( "python_scripts", package="DEXSeq", mustWork=TRUE )

## ----loadDEXSeq---------------------------------------------------------------
inDir = system.file("extdata", package="pasilla")
countFiles = list.files(inDir, pattern="fb.txt$", full.names=TRUE)
basename(countFiles)
flattenedFile = list.files(inDir, pattern="gff$", full.names=TRUE)
basename(flattenedFile)

## ----sampleTable--------------------------------------------------------------
sampleTable = data.frame(
   row.names = c( "treated1", "treated2", "treated3", 
      "untreated1", "untreated2", "untreated3", "untreated4" ),
   condition = c("knockdown", "knockdown", "knockdown",  
      "control", "control", "control", "control" ),
   libType = c( "single-end", "paired-end", "paired-end", 
      "single-end", "single-end", "paired-end", "paired-end" ) )

## ----check--------------------------------------------------------------------
sampleTable

## ----message=FALSE------------------------------------------------------------
library( "DEXSeq" )

dxd = DEXSeqDataSetFromHTSeq(
   countFiles,
   sampleData=sampleTable,
   design= ~ sample + exon + condition:exon,
   flattenedfile=flattenedFile )

## ----acc----------------------------------------------------------------------
head( geneIDs(dxd) )
head( exonIDs(dxd) )

## ----grmethods----------------------------------------------------------------
interestingRegion = GRanges( "chr2L", 
   IRanges(start=3872658, end=3875302) )
subsetByOverlaps( x=dxr, ranges=interestingRegion )
findOverlaps( query=dxr, subject=interestingRegion )

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

## ----echo=FALSE, include=FALSE------------------------------------------------
knitr::opts_chunk$set(tidy = FALSE,
                      cache = TRUE,
                      dev = "png",
                      message = FALSE,
                      error = FALSE,
                      warning = TRUE)

## ----warning=FALSE------------------------------------------------------------

library(GenomicFeatures)  # for the exonicParts() function
library(txdbmaker)        # for the makeTxDbFromGFF() function
download.file(
    "https://ftp.ensembl.org/pub/release-62/gtf/drosophila_melanogaster/Drosophila_melanogaster.BDGP5.25.62.gtf.gz",
    destfile="Drosophila_melanogaster.BDGP5.25.62.gtf.gz")
txdb = makeTxDbFromGFF("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

file.remove("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

flattenedAnnotation = exonicParts( txdb, linked.to.single.gene.only=TRUE )
names(flattenedAnnotation) =
    sprintf("%s:E%0.3d", flattenedAnnotation$gene_id, flattenedAnnotation$exonic_part)


## -----------------------------------------------------------------------------

library(pasillaBamSubset)
library(Rsamtools)
library(GenomicAlignments)
bamFiles = c( untreated1_chr4(), untreated3_chr4() )
bamFiles = BamFileList( bamFiles )
seqlevelsStyle(flattenedAnnotation) = "UCSC"
se = summarizeOverlaps(
    flattenedAnnotation, BamFileList(bamFiles), singleEnd=FALSE,
    fragments=TRUE, ignore.strand=TRUE )


## ----buildExonCountSet, eval=FALSE--------------------------------------------
#  library(DEXSeq)
#  colData(se)$condition = factor(c("untreated1", "untreated3"))
#  colData(se)$libType = factor(c("single-end", "paired-end"))
#  dxd = DEXSeqDataSetFromSE( se, design= ~ sample + exon + condition:exon )

## ----startVignette------------------------------------------------------------
data(pasillaDEXSeqDataSet, package="pasilla")

## -----------------------------------------------------------------------------
colData(dxd)

## -----------------------------------------------------------------------------
head( counts(dxd), 5 )

## -----------------------------------------------------------------------------
split( seq_len(ncol(dxd)), colData(dxd)$exon )

## -----------------------------------------------------------------------------
head( featureCounts(dxd), 5 )

## -----------------------------------------------------------------------------
head( rowRanges(dxd), 3 )

## -----------------------------------------------------------------------------
sampleAnnotation( dxd )

## -----------------------------------------------------------------------------
dxd = estimateSizeFactors( dxd )

## -----------------------------------------------------------------------------
dxd = estimateDispersions( dxd )

## ----fitdiagnostics, fig.small=TRUE, fig.cap="Fit Diagnostics. The initial per-exon dispersion estimates (shown by black points), the fitted mean-dispersion values function (red line), and the shrinked values in blue"----
plotDispEsts( dxd )

## ----testForDEU1--------------------------------------------------------------
dxd = testForDEU( dxd )

## ----estFC--------------------------------------------------------------------
dxd = estimateExonFoldChanges( dxd, fitExpToVar="condition")

## ----results1-----------------------------------------------------------------
dxr1 = DEXSeqResults( dxd )
dxr1

## ----results2-----------------------------------------------------------------
mcols(dxr1)$description

## ----tallyExons---------------------------------------------------------------
table ( dxr1$padj < 0.1 )

## ----tallyGenes---------------------------------------------------------------
table ( tapply( dxr1$padj < 0.1, dxr1$groupID, any ) )

## ----MvsA, fig.small=TRUE, fig.cap="MA plot. Mean expression versus $log{2}$ fold change plot. Significant hits at an $FDR=0.1$ are coloured in red."----
plotMA( dxr1, cex=0.8 )

## ----design-------------------------------------------------------------------
sampleAnnotation(dxd)

## ----formulas2----------------------------------------------------------------
formulaFullModel    =  ~ sample + exon + type:exon + condition:exon
formulaReducedModel =  ~ sample + exon + type:exon 

## ----estDisps_again-----------------------------------------------------------
dxd = estimateDispersions( dxd, formula = formulaFullModel )

## ----test_again---------------------------------------------------------------
dxd = testForDEU( dxd, 
	reducedModel = formulaReducedModel, 
        fullModel = formulaFullModel )

## ----res_again----------------------------------------------------------------
dxr2 = DEXSeqResults( dxd )

## ----table2-------------------------------------------------------------------
table( dxr2$padj < 0.1 )

## ----table3-------------------------------------------------------------------
table( before = dxr1$padj < 0.1, now = dxr2$padj < 0.1 )

## ----plot1, fig.height=8, fig.width=12, fig.cap="Fitted expression. The plot represents the expression estimates from a call to `testForDEU()`. Shown in red is the exon that showed significant differential exon usage."----
plotDEXSeq( dxr2, "FBgn0010909", legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----checkClaim---------------------------------------------------------------
wh = (dxr2$groupID=="FBgn0010909")
stopifnot(sum(dxr2$padj[wh] < formals(plotDEXSeq)$FDR)==1)

## ----plot2, fig.height=8, fig.width=12, fig.cap="Transcripts. As in the previous plots, but including the annotated transcript models"----
plotDEXSeq( dxr2, "FBgn0010909", displayTranscripts=TRUE, legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----plot3, fig.height=8, fig.width=12, fig.cap="Normalized counts. As in the previous plots, with normalized count values of each exon in each of the samples."----
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, norCounts=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----plot4, fig.height=8, fig.width=12, fig.cap="Fitted splicing. As in the previous plots, but after subtraction of overall changes in gene expression."----
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, splicing=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )

## ----DEXSeqHTML, cache=TRUE, eval=FALSE---------------------------------------
#  DEXSeqHTML( dxr2, FDR=0.1, color=c("#FF000080", "#0000FF80") )

## ----para1,cache=TRUE,results='hide', eval=FALSE------------------------------
#  BPPARAM = MultiCoreParam(4)
#  dxd = estimateSizeFactors( dxd )
#  dxd = estimateDispersions( dxd, BPPARAM=BPPARAM)
#  dxd = testForDEU( dxd, BPPARAM=BPPARAM)
#  dxd = estimateExonFoldChanges(dxd, BPPARAM=BPPARAM)

## ----alldeu-------------------------------------------------------------------
dxr = DEXSeq(dxd)
class(dxr)

## ----pergeneqval--------------------------------------------------------------
numbOfGenes = sum( perGeneQValue(dxr) < 0.1)
numbOfGenes

## ----systemFile---------------------------------------------------------------
pythonScriptsDir = system.file( "python_scripts", package="DEXSeq" )
list.files(pythonScriptsDir)
system.file( "python_scripts", package="DEXSeq", mustWork=TRUE )

## ----loadDEXSeq---------------------------------------------------------------
inDir = system.file("extdata", package="pasilla")
countFiles = list.files(inDir, pattern="fb.txt$", full.names=TRUE)
basename(countFiles)
flattenedFile = list.files(inDir, pattern="gff$", full.names=TRUE)
basename(flattenedFile)

## ----sampleTable--------------------------------------------------------------
sampleTable = data.frame(
   row.names = c( "treated1", "treated2", "treated3", 
      "untreated1", "untreated2", "untreated3", "untreated4" ),
   condition = c("knockdown", "knockdown", "knockdown",  
      "control", "control", "control", "control" ),
   libType = c( "single-end", "paired-end", "paired-end", 
      "single-end", "single-end", "paired-end", "paired-end" ) )

## ----check--------------------------------------------------------------------
sampleTable

## ----message=FALSE------------------------------------------------------------
library( "DEXSeq" )

dxd = DEXSeqDataSetFromHTSeq(
   countFiles,
   sampleData=sampleTable,
   design= ~ sample + exon + condition:exon,
   flattenedfile=flattenedFile )

## ----acc----------------------------------------------------------------------
head( geneIDs(dxd) )
head( exonIDs(dxd) )

## ----grmethods----------------------------------------------------------------
interestingRegion = GRanges( "chr2L", 
   IRanges(start=3872658, end=3875302) )
subsetByOverlaps( x=dxr, ranges=interestingRegion )
findOverlaps( query=dxr, subject=interestingRegion )

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

arrangeCoefs <- function( frm, mf, mm = model.matrix( frm, mf ), fit = NULL, insertValues = TRUE ) {

   if( any( attr( mm, "contrasts" ) != "contr.treatment" ) )
      stop( "Can only deal with standard 'treatment' contrasts." )   # Do I need this?
   if( is.null(fit) & insertValues )
      stop( "If fit==NULL, returnCoefValues must be FALSE" )
   if( !is.null(fit) )
      stopifnot( all( colnames(mm) == names(coefficients(fit)) ) )

   fctTbl <- attr( terms(frm), "factors" )

   coefIndicesList <- 
   lapply( seq_len(ncol(fctTbl)), function( fctTblCol ) {
      termName <- colnames(fctTbl)[ fctTblCol ]
      varsInTerm <- stringr::str_split( termName, stringr::fixed(":") )[[1]] 
      stopifnot( all( fctTbl[ varsInTerm, fctTblCol ] == 1 ) )
      stopifnot( sum( fctTbl[ , fctTblCol ] ) == length( varsInTerm ) )
      coefNames <- colnames(mm)[ attr( mm, "assign" ) == fctTblCol ]
      lvlTbl <- stringr::str_match( coefNames, 
         stringr::str_c( "^", stringr::str_c( varsInTerm, "([^:]*)", collapse=":" ), "$" ) )[ , -1, drop=FALSE ]
      stopifnot( ncol(lvlTbl) == length( varsInTerm ) )
      stopifnot( nrow(lvlTbl) == length( coefNames ) )
      if( !all( sapply( varsInTerm, function(v) is.factor(mf[[v]]) | is.character(mf[[v]]) ) ) )
         stop( "Non-factor in model frame" )

      varLevels <- lapply( varsInTerm, function(v) levels( factor( mf[[v]] ) ) ) 
      coefIndices <- array( NA_character_, dim = sapply( varLevels, length ), dimnames = varLevels )
      names( dimnames( coefIndices ) ) <- varsInTerm

      for( i in seq_len( nrow(lvlTbl) ) )
         coefIndices <- do.call( `[[<-`, c( quote(coefIndices), as.list( lvlTbl[ i, ] ), coefNames[i] ) )

      coefIndices
   } )
   names( coefIndicesList ) <- colnames( fctTbl )

   if( attr( terms(frm), "intercept" ) ) {
      a <- array( c( `(Intercept)` = "(Intercept)" ) )
      dimnames(a) <- list( `(Intercept)` = c( "(Intercept)" ) )
      coefIndicesList <- c( list( `(Intercept)` = a ), coefIndicesList )
   }

   if( !insertValues )
      ans <- coefIndicesList
   else
      ans <- lapply( coefIndicesList, function(coefIndices) {
         a <- ifelse( is.na(coefIndices), 0, coefficients(fit)[ coefIndices ] )
         attr( a, "variables" ) <- attr( coefIndices, "variables" )
         a } )
      
   lapply( ans, function(x) 
      if( is.array(x) ) 
         x 
      else { 
         y <- array( x, dim=length(x) )
         attr( y, "variables" ) <- attr( x, "variables" )
         dimnames(y) <- list( names(x) )
         y } )
}

apply2 <- function( X, MARGIN, FUN, ... ) {
   if( length(MARGIN) > 0 ) 
      apply( X, MARGIN, FUN, ... ) 
   else 
      FUN( X, ... ) }

balanceExons <- function( coefs, dispersions ) {
   stopifnot( any( sapply( coefs, function(x) 
      identical( names(dimnames(x)), "(Intercept)" ) ) ) )
   termsWithExon <- sapply( coefs, function(x) "exon" %in% names(dimnames(x)) )
   meanMainEffect <- sum( sapply( coefs[!termsWithExon], mean, na.rm=TRUE ) )
   meanExonEffects <- rowSums( sapply( coefs[termsWithExon], function(x) 
      apply2( x, "exon", mean, na.rm=TRUE ) ) )

   meanExonFittedValue <- exp( meanMainEffect + meanExonEffects )

   exonWeights <-  1 / ( dispersions + 1 / meanExonFittedValue )

   shifts <- lapply( coefs[termsWithExon], function(x) { 
      nonExonDims <- which(  names(dimnames(x)) != "exon" )
      list(
         vars = names(dimnames(x))[ nonExonDims ],
         wmeans = apply2( x, nonExonDims, weighted.mean, exonWeights) ) } )

   lapply( coefs, function(x) {
      nonExonVars <- names(dimnames(x))[ names(dimnames(x)) != "exon" ]
      if( identical( nonExonVars, "(Intercept)" ) )
         whichShift <- which( sapply( shifts, function(xx) length( xx$vars ) == 0 ) )
      else
         whichShift <- which( sapply( shifts, function(xx) identical( xx$vars, nonExonVars ) ) )
      if( length( whichShift ) == 0 )
         return( x )
      if( length( whichShift ) > 1 )
         stop( "Confused about selecting shift." )
      if( "exon" %in% names(dimnames(x)) )
         x - shifts[[ whichShift ]]$wmeans
      else
         x + shifts[[ whichShift ]]$wmeans
    } )
}         


fitAndArrangeCoefs <- function( frm = count ~ condition * exon, balanceExons = TRUE, mf, fitExpToVar, geneID)
{
   if( length(levels(mf$exon)) <= 1 )
      return( NULL )
   mm <- model.matrix( frm, mf )
   fit <- tryCatch(
       {
           glmnb.fit(mm, mf$count, dispersion=mf$dispersion,
                     offset=log(mf$sizeFactor), tol=0.01)
       },
       error=function(cond){
           message( sprintf("Fit for gene/exon %s failed, coefficients for this gene won't show up.", geneID ) )
           return(NULL)
       },
       warning=function(cond){
           message( sprintf("Fit for gene/exon %s threw the next warning(s): %s", geneID, unique(cond$message) ) )
       } )
   
   if( is.null( fit ) ){
       return( NULL )
   }

   if( is( mf[[fitExpToVar]], "numeric" ) ){
       coefs <- fit$coefficients
       attributes(coefs)$fitType <- "numeric"
   }else{
       coefs <- arrangeCoefs( frm, mf, mm, fit )
       if( balanceExons ) {
           coefs <- balanceExons( coefs, tapply( mf$dispersion, mf$exon, `[`, 1 ) )
       }
       attributes(coefs)$fitType <- "factor"
   }
   coefs
}


getEffectsForPlotting <- function( coefs, groupingVar = "condition", averageOutExpression=FALSE, frm, mf )
{
    if( attributes(coefs)$fitType == "factor" ){
        groupingExonInteraction <- which( sapply( coefs, function(x) 
            all( c( groupingVar, "exon") %in% names(dimnames(x)) ) & length(dim(x)) == 2 ) ) 
        fittedValues <- coefs[[ groupingExonInteraction ]]
        if( names(dimnames(fittedValues))[1] == "exon" )
            fittedValues <- t( fittedValues )
        stopifnot( identical( names(dimnames(fittedValues)), c( groupingVar, "exon" ) ) )
        for( x in coefs[ -groupingExonInteraction ] ) {
            if( all( c( groupingVar, "exon") %in% names(dimnames(x)) ) )
                stop( "Cannot yet deal with third-order terms." )
            if( !any( c( groupingVar, "exon") %in% names(dimnames(x)) ) ) {
                fittedValues <- fittedValues + mean( x )
            } else if( averageOutExpression & identical( names(dimnames(x)), groupingVar ) ) {
                fittedValues <- fittedValues + mean( x )
            } else if( groupingVar %in% names(dimnames(x)) ) {
                groupMeans <- apply2( x, groupingVar, mean )
                stopifnot( identical( names(groupMeans), dimnames(fittedValues)[[1]] ) )
                fittedValues <- fittedValues + groupMeans
            } else if( "exon" %in% names(dimnames(x)) ) {
                exonMeans <- apply2( x, "exon", mean )
                fittedValues <- t( t(fittedValues) + exonMeans )
            } else {
                print( x )
                stop( "Unexpected term encountered." )
            }
        }
        return( fittedValues )
    }else{
        stopifnot( "(Intercept)" %in% names(coefs) )
        stopifnot( "exonthis" %in% names(coefs) )
        allVars <- all.vars(frm)
        continuousVar <- allVars[!allVars %in% c("count", "exon")]
        interactionCoefName <- paste0( continuousVar, ":exonthis" )
        stopifnot( interactionCoefName %in% names(coefs) )
        mf[[continuousVar]]
        predictors <- unique( mf[[continuousVar]] )
        if( averageOutExpression ){
            fittedValues <- coefs["exonthis"] + coefs[interactionCoefName]*predictors
        }else{
            fittedValues <- coefs["(Intercept)"] + coefs["exonthis"] +
                (coefs[continuousVar] + coefs[interactionCoefName])*predictors
        }
        fittedValues <- matrix(fittedValues, ncol=1)
        colnames(fittedValues) <- "this"
#        rownames(fittedValues) <- sprintf("%s=%s", continuousVar, predictors)
        rownames(fittedValues) <- as.character(predictors)
        return( fittedValues )
    }
}


modelFrameSM <- function(object)
{
    mfSmall <- as.data.frame( colData(object) )
    mfSmall$exon <- factor( mfSmall$exon, levels=c("others", "this") )
    ##mfSmall$exon <- relevel( mfSmall$exon, "others" )
    mfSmall$dispersion <- NA
    mfSmall$count <- NA
    mfSmall
}

getEffectsForGeneBM <- function(geneID, groups, notNAs, countsAll,
                                disps, features, mf, frm, numsamples,
                                fitExpToVar, averageOutExpression=TRUE)
{
    rt <- groups %in% geneID & notNAs
    if( sum(rt) < 2 ){ return(NULL) }
    countsThis <- countsAll[rt,]
    rownames(countsThis) <- gsub("\\S+:", "", rownames(countsThis))
    dispsThis <- disps[rt]
    names(dispsThis) <- features[rt]
    numexons <- sum(rt)
    newMf <- mf[as.vector( sapply( split( seq_len(nrow(mf)), mf$sample ), "[", seq_len( numexons ) ) ),]
    newMf$exon <- factor( rep( features[rt], numsamples ) )
    for (i in seq_len(nrow(newMf))) {
       newMf[i, "dispersion"] <- dispsThis[as.character(newMf[i, "exon"])]
       newMf[i, "count"] <- countsThis[as.character(newMf[i, "exon"]), as.character(newMf[i, "sample"])]
    }
    newMf <- droplevels(newMf)
    coefficients <- fitAndArrangeCoefs( frm, balanceExons = TRUE, mf=newMf, fitExpToVar=fitExpToVar, geneID=geneID)
    if (is.null(coefficients)){
       return(coefficients)
    }
    ret <- t( getEffectsForPlotting(coefficients, averageOutExpression = averageOutExpression, 
        groupingVar = fitExpToVar, frm=frm, mf=newMf))
    rownames(ret) <- paste(geneID, rownames(ret), sep = ":")
    return(ret)
}

getEffectsForExonsSM <- function(index, frm, countsAll, disps,
                                 mfSmall, averageOutExpression=TRUE,
                                 fitExpToVar)
{
    mfSmall$count <- countsAll[index,]
    mfSmall$dispersion <- disps[index]
    coefs <- fitAndArrangeCoefs( frm, mf=mfSmall, balanceExons=FALSE, fitExpToVar=fitExpToVar, rownames(countsAll)[index])
    if( is.null(coefs) ){
        return(NULL)
    }
    getEffectsForPlotting(coefs,
            averageOutExpression=averageOutExpression,
            groupingVar=fitExpToVar, frm, mfSmall)[,"this"]
}

getEffectsForGene <- function( geneID, object, maxRowsMF, fitExpToVar)
{
    rt <- object$groupID %in% geneID
    sampleData <- object@sampleData
    if( is(object@sampleData[[fitExpToVar]], "numeric") ){
        maxRowsMF <- 0
    }
    numsamples <- nrow(object@sampleData)
    numexons <- sum(rt)
    featuresInGene <- object$featureID[rt]
    dispersions <- object$dispersion[rt]
    dispersions[is.na(dispersions)] <- 1e-08
    frm <- as.formula(paste("count ~", fitExpToVar, "* exon"))
    bigFlag <- numsamples*numexons < maxRowsMF
    if( bigFlag ){
        mf <- object@modelFrameBM
        mf <- mf[as.vector(sapply(split(seq_len(nrow(mf)), mf$sample), 
            "[", seq_len(numexons))), ]
        mf$exon <- factor(rep(featuresInGene, nrow(sampleData)))
        counts <- object$countData[rt,]
        rownames(counts) <- gsub("\\S+:", "", rownames(counts))
        names(dispersions) <- object$featureID[rt]
        for (i in seq_len(nrow(mf))) {
            mf[i, "dispersion"] <-
                dispersions[as.character(mf[i, "exon"])]
            mf[i, "count"] <-
                counts[as.character(mf[i, "exon"]), as.character(mf[i, "sample"])]
        }
        mf <- droplevels(mf)
        coefs <- fitAndArrangeCoefs(frm, balanceExons=TRUE, mf=mf, fitExpToVar=fitExpToVar, geneID)
        if( is.null(coefs ) ){
            return(NULL)
        }
        splicing <- t(getEffectsForPlotting( coefs, groupingVar=fitExpToVar, averageOutExpression=TRUE, frm=frm, mf=mf))
        expression <- t(getEffectsForPlotting( coefs, groupingVar=fitExpToVar, averageOutExpression=FALSE, frm=frm, mf=mf))
        rownames(splicing) <- sprintf("%s:%s", geneID, rownames(splicing))
        rownames(expression) <- rownames(splicing)
        list( expression=expression, splicing=splicing )
    }else{
        mf <- object@sampleData
        mf <- rbind( data.frame(mf, exon="this"), data.frame(mf, exon="others"))
        mf$exon <- factor( mf$exon, levels=c("others", "this") )
        ##mf$exon <- relevel( mf$exon, "others" )
        countsThis <- object$countData[rt,]
        countsOthers <- sapply( rownames( countsThis ),
                               function(x){
                                   colSums(countsThis[!rownames(countsThis) %in% x,,drop=FALSE])
                               })
        countsOthers <- t(countsOthers)
        stopifnot(all(rownames(countsThis) ==  rownames(countsOthers)))
        effects <- lapply( seq_len(numexons), function(x){
                   mf$count <- c( countsThis[x,], countsOthers[x,])
                   mf$dispersion <- dispersions[x]
                   coefs <- fitAndArrangeCoefs(frm, balanceExons=FALSE, mf=mf, fitExpToVar=fitExpToVar, geneID)
                   if( is.null(coefs) ){
                       return(NULL)
                   }
                   splicing <- getEffectsForPlotting( coefs, groupingVar=fitExpToVar, averageOutExpression=TRUE, frm=frm, mf=mf)[,"this"]
                   expression <- getEffectsForPlotting( coefs, groupingVar=fitExpToVar, averageOutExpression=FALSE, frm=frm, mf=mf)[,"this"]
                   list(splicing=splicing, expression=expression)
               })
        names(effects) <- rownames(object)[rt]
        splicing <- t(sapply(effects, "[[", "splicing"))
        expression <- t( sapply(effects, "[[", "expression" ))
        list( expression=expression, splicing=splicing )
    }
}

DEXSeqHTML <- function(object, genes=NULL, path="DEXSeqReport", file="testForDEU.html", fitExpToVar="condition", FDR=0.1, color=NULL, color.samples=NULL, mart="", filter="", attributes="", extraCols=NULL, BPPARAM=MulticoreParam(workers=1) )
{
   stopifnot( is( object, "DEXSeqResults" ) )
   if(!fitExpToVar %in% colnames( object@modelFrameBM )){
       stop(sprintf("The value of the parameter fitExpToVar, '%s', is not a variable from the annotation of the samples. Please specify a column name of the colData slot of the DEXSeqDataSet object.", fitExpToVar))
   }
   ######## GET THE RESULT TABLE READY ##########
   genomicData <- as.data.frame( object$genomicData )
   results <- data.frame( object[, c( "groupID", "featureID", "exonBaseMean", "dispersion", "pvalue", "padj" )], stringsAsFactors=TRUE)
   results <- cbind( results, genomicData )

   dexseqR <- mcols( object )$type == "DEXSeq results"

   if(sum(dexseqR, na.rm=TRUE) > 0){
      results <-
          cbind(
              results,
              round(
                  as.data.frame( object[,which(dexseqR)] ), 3
                  )
              )
   }

   rownames(results) <- NULL
   sampleData <- object@sampleData
   
   numcond <- length(unique(sampleData[[fitExpToVar]]))
   if(is.null(color)){
      color<-rgb(colorRamp(c("#D7191C", "#FFFFBF", "#2B83BA"))(seq(0, 1, length.out=numcond)), maxColorValue=255, alpha=175)
   }
   names(color) <- sort(levels(sampleData[[fitExpToVar]]))
   dir.create(file.path(path, "files"), recursive=TRUE)

   if(is.null(genes)){
      gns <- as.character(unique(results$groupID[which(results$padj < FDR)]))
   }else{
      gns <- genes
   }
   
   if(!all(gns %in% object$groupID)){
      stop("The geneIDs provided are not in the ecs object")}
   if(length(gns)==0){ 
      stop("There are no significant results in the test... nothing to report")}

   p<-openPage(file.path(path, file))
   hwrite('DEXSeq differential exon usage test', p, heading=1)
   hwrite('Experimental design', p, heading=2)
   cond<-as.matrix( as.data.frame( sampleData[,!colnames(sampleData) %in% "sizeFactor"] ) )
   rownames(cond) <- NULL
   condcolor <- matrix(rep("white", nrow(cond)*ncol(cond)), nrow(cond))
   condcolor[,which(colnames(cond) %in% fitExpToVar)] <- color[as.character(sampleData[[fitExpToVar]])]
   if(!is.null(color.samples)){
      condcolor[,1] <- color.samples}
   hwrite(cond, bgcolor=condcolor, p)

   formulas <- mcols(object)[colnames(object) == "pvalue","description"]
   formulas <- sapply( strsplit(formulas, "vs|p-value:" ), "[", c(2, 3))
   formulas <- as.vector( gsub("'", "", formulas) )
   
   hwrite(paste("\n\nformulaDispersion = ", formulas[1], sep=""), p, heading=3)
   hwrite(paste("\nformula0 = ", formulas[2], sep=""), p, heading=3)
   hwrite(paste("\nformula1 = ", formulas[1], sep=""), p, heading=3)
   hwrite('testForDEU result table', p, heading=2)
   ptowrite <- paste0(path, "/files/")
   ######### prepare colors for table results
	
   m2col <- colorRampPalette(c("#FF706B", "#FEE08B", "white"), space="rgb")(5)
   matcol <- matrix(rep(m2col[5], ncol(results)*nrow(results)), nrow(results))
   ######### COLOR LEGEND = I BET THERE IS A SMARTER WAY OF DOING THIS:
   j <- 4
   for(i in c(0.25, 0.1, 0.05, 0.01)){
      matcol[which(results$padj <= i),] <- m2col[j]
      j <- j-1
   }
   legend <- hwrite(c("<= 0.01", "<= 0.05", "<= 0.1", "<= 0.25", "> 0.25"), bgcolor=m2col)

  makePagesForGene <- function(gene){
#      print( gene )
      back <- hwrite("back", link=file.path("..", file))
      nameforlinks <- sapply(strsplit(gene, "\\+"), "[[", 1)
      otherlinks <- hwrite(c("counts", "expression", "splicing", "transcripts", "results"), link=c(paste(nameforlinks, "counts.html", sep=""), paste(nameforlinks, "expression.html", sep=""), paste(nameforlinks, "splicing.html", sep=""), paste(nameforlinks, "transcripts.html", sep=""), paste(nameforlinks, "results.html", sep="")), table=FALSE)
      loc <- as.character(results$groupID) %in% as.character(gene)
      ### this makes the page where to explore the pvalues ###
      subres <- results[loc,]
      subres[c("dispersion", "pvalue", "padj")] <- round(subres[c("dispersion", "pvalue", "padj")], 3)
      submatcol <- matcol[loc,]
      rownames(subres) <- NULL
      genpage <- openPage(paste(ptowrite, nameforlinks, "results.html", sep=""))
      hwrite(c(back, otherlinks), table=TRUE, border=0, genpage)
      hwrite(legend, page=genpage)
      hwrite(as.matrix(subres), bgcolor=submatcol, table.class="sortable", style='margin:16px; border:0px solid black; border-width:0px; width:200px', table=TRUE, page=genpage)
      close(genpage, splash=TRUE)
      ### MAKE THE PLOT HTML PAGES FOR expression, counts and splicing
      makePlotPage( object, ptowrite=ptowrite, gene=gene, whichtag="expression", links=c(back, otherlinks), color=color, color.samples=color.samples, FDR=FDR, fitExpToVar=fitExpToVar, width=1200, height=700, h=7)
      makePlotPage( object, ptowrite=ptowrite, gene=gene, whichtag="counts", links=c(back, otherlinks), color=color, color.samples=color.samples, FDR=FDR, fitExpToVar=fitExpToVar, width=1200, height=700, h=7)
      makePlotPage( object, ptowrite=ptowrite, gene=gene, whichtag="splicing", links=c(back, otherlinks), color=color, color.samples=color.samples, FDR=FDR, fitExpToVar=fitExpToVar, width=1200, height=700, h=7)

      transcripts <- object$transcripts[loc]
      if(length( unlist( transcripts ) ) > 0 ) {
         trans <- Reduce(union, transcripts)
         h <- ifelse(length(trans) > 10, 7+(length(trans)*0.3), 7)
      if(sum(loc) > 30){   ############# if there are more than 30 exons, increase the size of the plotting region
         h <- h + (sum(loc)*0.1)
      }
      makePlotPage( object, ptowrite=ptowrite, gene=gene, whichtag=c("expression", "transcripts"), links=c(back, otherlinks), color=color, color.samples=color.samples, FDR=FDR, fitExpToVar=fitExpToVar, width=1200, height=h*100, h=h)
      }
      return()
   }


   bplapply( gns, makePagesForGene, BPPARAM=BPPARAM )
   
   results <- results[as.character(results$groupID) %in% gns,]

   splitCols <- split( seq_len(nrow( results ) ), results$groupID )
   
   genetable <- lapply( splitCols, function(x){
       data.frame(
           chr=unique( results$seqnames[x] ),
           start=min( results$start[x] ),
           end=max( results$end[x] ),
           total_exons = length(x),
           exon_changes = sum( results$padj[x] < FDR, na.rm=TRUE) )
   })
   genetable <- do.call(rbind, genetable)
   genetable <- cbind( geneID=rownames(genetable), genetable )

   if(class(mart) == "Mart"){
      if(attributes(mart)$dataset != ""){
      forvalues <- strsplit(as.character(genetable$geneID), "\\+")
      names(forvalues) <- genetable$geneID
      if(length(filter) > 1){
         warning("length(filter) > 2, only first element will be taken")
         filter <- filter[1]
      }
      extra <- getBM(attributes=c(filter, attributes), filters=filter, values=unlist(forvalues), mart=mart)
      fromart <- lapply(genetable$geneID, function(x){
         sep <- do.call(c, strsplit(as.character(x), "\\+"))
         extra[which(extra[,filter] %in% sep),]
      })

      extra <- sapply(attributes, 
         function(r){
           unlist(
              lapply(fromart, 
                 function(x){
                    paste(x[,r], collapse=" ")
                  }
              )
           )
         }
      )
      genetable <- cbind(geneID=genetable$geneID, extra, genetable[,2:length(genetable)])
      }else{
         warning("No dataset in biomart specified")
      }
   }else if( mart != ""){
      warning("Please provide a Mart class object for parameter mart")
   }  

   if( !is.null( extraCols )){
      genetable <- cbind(extraCols[match(genetable$geneID, rownames(extraCols)),], genetable)
   }
   
	
   genetable$geneID <- sapply(as.character(genetable$geneID), function(m){w <- strsplit(m, "\\+");ns <- sapply(w, "[[", 1);hwrite(paste(unlist(w), collapse=" "), link=paste("files/", ns, "expression.html", sep=""))})
   rownames(genetable) <- NULL
   hwrite(genetable, page=p, table=TRUE, table.class="table-layout:fixed", style='margin:16px; border:0px solid black; border-width:1px; width:20%')
   close(p, splash=TRUE)
}


makePlotPage <- function(object, ptowrite, gene, whichtag, links, color, color.samples, FDR, fitExpToVar, width, height, h){
   allopts <- c("expression", "splicing", "counts", "transcripts")
   opts <- allopts %in% whichtag
   onlytag <- allopts[max(which(opts))]
   pagename <- sapply(strsplit(as.character(gene), "\\+"), "[[", 1)
   genpage <- openPage(paste(ptowrite, pagename, onlytag, ".html", sep=""))
   hwrite(links, table=TRUE, border=0, genpage)
   svg(paste(ptowrite, pagename, onlytag, ".svg", sep=""), height=h, width=12, pointsize=14)
   plotDEXSeq(object, geneID=gene, FDR=FDR, lwd=2, expression=opts[1], splicing=opts[2], norCounts=opts[3], displayTranscripts=opts[4], fitExpToVar=fitExpToVar, legend=TRUE, color=color, color.samples=color.samples, cex.axis=1.5)
   dev.off()
   hwrite(hmakeTag("iframe", src=paste(pagename, onlytag, ".svg", sep=""), width=width, height=height, border=0), page=genpage )
   close(genpage, splash=TRUE)
}
     

##--------------------------------------------------
## get q-value for each gene
##--------------------------------------------------

perGeneQValue = function(object, p = "pvalue", method = perGeneQValueExact) {
  stopifnot( is(object, "DEXSeqResults"))
  wTest <- which( !is.na( object$padj ) )
  ## use only those exons that were testable
  pvals     = object[[p]][wTest]
  ## 'factor' removes ununsed levels
  geneID    = factor(object[["groupID"]][wTest])
  geneSplit = split(seq(along=geneID), geneID)

  ## summarise p-values of exons for one gene: take the minimum
  pGene = sapply(geneSplit, function(i) min(pvals[i]))
  stopifnot(all(is.finite(pGene)))

  ## Determine the thetas to be used
  theta = unique(sort(pGene))

  ## compute q-values associated with each theta
  q = method(pGene, theta, geneSplit)

  ## return a named vector of q-values per gene
  res        = rep(NA_real_, length(pGene))
  res        = q[match(pGene, theta)]
  res = pmin(1, res)
  names(res) = names(geneSplit)
  stopifnot(!any(is.na(res)))
  return(res)
}

##----------------------------------------------------------------------
## For each value of theta, determine how many minima of random per-exon
## p-values are smaller (using the number of exons per gene)
##----------------------------------------------------------------------
perGeneQValueBySimulation = function(pGene, theta, geneSplit, nperm = 24) {
  nr = sum(listLen(geneSplit))
  pRand = apply(matrix(runif(nr*nperm), nrow=nr, ncol=nperm), 2,
    function(p) sapply(geneSplit, function(i) min(p[i])))

  ## check that the apply/sapply stuff worked as intended
  stopifnot(nrow(pRand)==length(pGene), ncol(pRand)==nperm)

  hTest   = hist(pGene, breaks=c(theta,+Inf), plot=FALSE)
  hRand   = hist(pRand, breaks=c(theta,+Inf), plot=FALSE)
  stopifnot(sum(hTest$counts)==length(pGene),
            sum(hRand$counts)==length(pRand))

  numPos       = cumsum(hTest$counts)
  numFalsePos  = cumsum(hRand$counts)/nperm

  return(numFalsePos/numPos)
}

##--------------------------------------------------
## Exact computation - see methods part of the paper
##---------------------------------------------------
perGeneQValueExact = function(pGene, theta, geneSplit) {
  stopifnot(length(pGene)==length(geneSplit))

  ## Compute the numerator \sum_{i=1}^M 1-(1-theta)^{n_i}
  ## Below we first identify the summands which are the same
  ## (because they have the same n_i), then do the sum via the
  ## mapply
  numExons     = listLen(geneSplit)
  tab          = tabulate(numExons)
  notZero      = (tab>0)
  numerator    = mapply(function(m, n) m * (1 - (1-theta)^n),
                            m = tab[notZero],
                            n = which(notZero))
  numerator    = rowSums(numerator)

  ## Compute the denominator: for each value of theta, the number
  ## of genes with pGene <= theta[i].
  ## Note that in cut(..., right=TRUE), the intervals are
  ## right-closed (left open) intervals.
  bins   = cut(pGene, breaks=c(-Inf, as.vector(theta)), right = TRUE, include.lowest = TRUE)
  counts = tabulate(bins, nbins = nlevels(bins))
  denom  = cumsum(counts)
  stopifnot(denom[length(denom)]==length(pGene))

  return(numerator/denom)
}

plotDEXSeq <- function( object, geneID, FDR=0.1, fitExpToVar="condition",
                       norCounts=FALSE, expression=TRUE, splicing=FALSE,
                       displayTranscripts=FALSE, names=FALSE, legend=FALSE,
                       color=NULL, color.samples=NULL, transcriptDb=NULL,
                       additionalAnnotation=NULL, maxRowsMF=2400, ...)
{
   stopifnot(is( object, "DEXSeqResults") | is( object, "DEXSeqDataSet"))
   if ( !fitExpToVar %in% colnames( object@modelFrameBM ) ) {
       stop(sprintf("The value of the parameter fitExpToVar,'%s', is not a column name of the 'colData' DataFrame from the DEXSeqDataSet object.", fitExpToVar))
   }
   op <- sum(c(expression, splicing, norCounts))
   if(op == 0){
       stop("Please indicate what would you like to plot\n")}

   if(!is.null(transcriptDb)){
       stopifnot( is( transcriptDb, "TxDb" ) )
   }

   if(!is.null(additionalAnnotation)){
      stopifnot( is( additionalAnnotation, "GRangesList" ) )
   }
   
   if( is(object, "DEXSeqResults")){
       sampleData <- object@sampleData
       genomicData <- object$genomicData
       rt<-which(object$groupID==geneID)
       count <- t( t(object$countData[rt,])/sampleData$sizeFactor )
       transcripts <- object$transcripts[rt]
       each <- object$padj[rt]
  }else{
       sampleData <- sampleAnnotation( object )
       genomicData <- rowRanges( object )
       rt <- which( mcols( object )$groupID == geneID )
       transcripts <- genomicData$transcripts[rt]
       mcols(genomicData) <- NULL
       count <- featureCounts( object, normalized=TRUE )[rt,]
       each <- rep(1, length.out=length(rt))
   }
   
   if(sum(count) == 0){
      warning("No read counts falling in this gene, there is nothing to plot.")
      return()}
   if(FDR>1|FDR<0){
      stop("FDR has to be a numeric value between 0 - 1")}
#   if( is(sampleData[[fitExpToVar]], "numeric" ) & expression ){
#       warning("'Expression' fit not supported for numeric predictors")
#       return()
#   }

   rango <- seq(along=rt)
   intervals<-(0:nrow(count))/nrow(count)
   numcond<-length(unique(sampleData[[fitExpToVar]]))
   numexons<-nrow(count)
   #exoncol<-ifelse(each<=FDR, "#8B0000", "dark green")
   #exoncol[is.na(exoncol)]<-"black"
   #colorlines <- ifelse(each<=FDR, "#FF000060", "lightgrey")
   exoncol<-ifelse(each<=FDR, "#F219ED", "#CCCCCC")
   exoncol[is.na(exoncol)]<-"white"
   colorlines <- ifelse(each<=FDR, "#F219ED60", "#B3B3B360")   # vertical dashed lines
   colorlines[is.na(colorlines)] <- "#B3B3B360"
   colorlinesB <- ifelse(each<=FDR, "#9E109B", "#666666")  # slanted solid lines
   colorlinesB[is.na(colorlinesB)] <- "#666666"

   ################## DETERMINE THE LAYOUT OF THE PLOT DEPENDING OF THE OPTIONS THE USER PROVIDES ###########
   if( length( unlist(start(genomicData))) > 0 ){
       
      sub <- data.frame(
         start=start(genomicData[rt]),
         end=end(genomicData[rt]),
         chr=as.character( seqnames( genomicData[rt] ) ),
         strand=as.character( strand( genomicData[rt] )  ) )
      rownames(sub) <- rownames(object)[rt]

      if( !is.null( additionalAnnotation ) ){
          additionalHits <- findOverlaps(additionalAnnotation, range( genomicData[rt]) )
          additionalAnnotation <- additionalAnnotation[queryHits( additionalHits )]
          if( length(additionalAnnotation) == 0 ){
              additionalAnnotation <- NULL
          }
      }
     
      rel<-(data.frame(sub$start, sub$end))-min(sub$start)
      rel<-rel/max(rel[,2])
      trans <- unique(unlist(transcripts))
      trans <- trans[!is.na(trans)]
      numberOfTrans <- length(trans) + length(additionalAnnotation)
      
      if( (displayTranscripts & !is.null( unlist(transcripts) ) ) | !is.null(additionalAnnotation) ){
         if(numberOfTrans > 40){
            warning("This gene contains more than 40 transcripts annotated, only the first 40 will be plotted\n")
         }
         if( !displayTranscripts ){
             numberOfTrans <- numberOfTrans - length(trans)
         }
         mat <- seq_len(3+min(numberOfTrans, 40)) ## max support from transcripts is 40, which seems to be the max for the layout supported by graphics
         hei<-c(8, 1, 1.5, rep(1.5, min(numberOfTrans, 40)))
      }else{
         mat<-1:3
         hei<-c(5, 1, 1.5)
      }
      if(op > 1){
         hei <- c(rep(hei[1], op-1), hei)
         mat <- c(mat, length(mat)+seq(along=op))
      }
      hei <- c(hei, .2)
      mat <- c(mat, length(mat)+1)
      layout(matrix(mat), heights=hei)
      par(mar=c(2, 4, 4, 2))
   }else if(op > 1){
      par(mfrow=c(op,1))
   }
   
   ####### DETERMINE COLORS, IF THE USER DOES NOT PROVIDE ONE PER SAMPLE THE COUNT WILL OBTAIN THEM CORRESPONDING TO THEIR DESIGN ####
   ##### determine colors if not provided by user ######
   if(is.null(color)){
      if( numcond < 10 ){
         color <- suppressWarnings( brewer.pal(numcond, "Set1")[seq_len(numcond)] )
      }else{
      color<-
          rgb(
              colorRamp(brewer.pal(5, "Set1"))(seq(0, 1, length.out=numcond)),
              maxColorValue=255,
              alpha=175)
     }
   }

#   if( is(sampleData[[fitExpToVar]], "numeric" ) & expression ){
#       warning("'Expression' fit not supported for numeric predictors")
#       return()
#   }

   
   names(color) <- sort( unique( as.character( (sampleData[[fitExpToVar]]) ) ) )
   

   if( expression | splicing ){
       if( !is(object, "DEXSeqResults") ){
           stop("To visualize beta estimates, please provide a DEXSeqResults object")
       }
       effects <- getEffectsForGene( geneID, object, maxRowsMF, fitExpToVar)
       if(is.null(effects[["splicing"]])){
#           warning(sprintf("glm fit failed for gene %s", geneID))
           return()
       }
       if( !all( rownames(sub) %in% rownames(effects[["splicing"]]) ) ){
#           warning(sprintf("glm fit failed for gene %s", geneID))
           return()
       }       
   }

   if(expression){
      coeff <- effects[["expression"]]
      coeff <- coeff[rownames(sub),]
      coeff <- exp(coeff)
      ylimn <- c(0, max(coeff, na.rm=TRUE))
      coeff <- vst( coeff, object )
      drawPlot(matr=coeff,
               ylimn, object, intervals,
               rango, textAxis="Expression",
               rt=rt, color=rep( color[colnames(coeff)], each=numexons),
               colorlines=colorlines, ...)
   }

   if(splicing){
      coeff <- effects[["splicing"]]
      coeff <- coeff[rownames(sub),]
      coeff <- exp(coeff)
      ylimn <- c(0, max(coeff, na.rm=TRUE))
      coeff <- vst( coeff, object )
      drawPlot(matr=coeff, ylimn, object, intervals,
               rango, textAxis="Exon usage", rt=rt,
               color=rep( color[colnames(coeff)], each=numexons),
               colorlines=colorlines, ...)
       
   }

   if(norCounts){
      ylimn <- c(0, max(count, na.rm=TRUE))
      count <- vst( count, object )
      if(is.null(color.samples)){
         colorcounts <- rep( color[as.character(sampleData[[fitExpToVar]])], each=numexons)
      }else{
         colorcounts <- rep(color.samples, each=numexons)
      }
      drawPlot(matr=count, ylimn, object, intervals, rango, textAxis="Normalized counts", rt=rt, color=colorcounts, colorlines=colorlines, ...)
   }
	########### plot the gene model ########## just if transcript information available
   if( length( unlist(start(genomicData))) > 0 ){
      par(mar=c(0, 4, 0, 2))
      plot.new()
      segments(apply((rbind(rel[rango,2], rel[rango, 1])), 2, median), 0, apply(rbind(intervals[rango], intervals[rango+1]-((intervals[rango+1]-intervals[rango])*0.2)), 2, median), 1, col=colorlinesB)
      par(mar=c(1.5, 4, 0, 2))
      drawGene(min(sub$start), max(sub$end), tr=sub, exoncol=exoncol, names, trName="Gene model", cex=0.8)
      if( length( unlist( transcripts ) ) > 0 ){
          i <- 1
      ##### plot the transcripts #######
          if(displayTranscripts){
              for(i in seq_len(min(length(trans), 40))) {
                  logicexons <- sapply(transcripts, function(x){length(which(x==trans[i]))})
                  tr <- reduce(IRanges( sub$start[logicexons == 1], sub$end[logicexons==1] ))
                  if( is.null( transcriptDb ) ){
                      tr <- as.data.frame(tr)[,c("start", "end")]
                      drawGene(min(sub$start), max(sub$end), tr=tr, exoncol="black", names, trName=trans[i], cex=0.8)
                  }else{
                      codingRanges <- select( transcriptDb,
                                                keys=trans[i],
                                                columns=c("CDSSTART", "CDSEND"),
                                             keytype="TXNAME")
                      if( is.na( any( codingRanges$CDSSTART ) ) ){ ### in case is a non coding transcript
                          tr <- as.data.frame(tr)[,c("start", "end")]
                          drawGene(min(sub$start), max(sub$end), tr=tr, exoncol=NULL, names, trName=trans[i], cex=0.8, miny=.25, maxy=.75)
                      }else{
                          codingRanges <- IRanges(codingRanges$CDSSTART,
                                                  codingRanges$CDSEND)
                          utrRanges <- setdiff(tr, codingRanges)
                          drawGene(min(sub$start), max(sub$end),
                                   tr=as.data.frame(codingRanges)[,c("start", "end")],
                                   exoncol="black", names, trName=trans[i], cex=0.8,
                                   drawNames=FALSE, drawIntronLines=FALSE)
                          if( length( utrRanges ) > 0 ){
                              drawGene( min(sub$start), max(sub$end),
                                   tr=as.data.frame(utrRanges)[,c("start", "end")],
                                   exoncol=NULL, names, trName=trans[i], cex=0.8,
                                   drawNames=FALSE, drawIntronLines=FALSE, newPanel=FALSE,
                                   miny=.25, maxy=.75)
                          }
                          drawGene(min(sub$start), max(sub$end),
                                   tr=as.data.frame(tr)[,c("start", "end")],
                                   exoncol="black", names, trName=trans[i], cex=0.8,
                                   newPanel=FALSE, drawExons=FALSE)
                      }
                  }
              }
          }
          if(!is.null(additionalAnnotation)){
              for( j in seq_along(additionalAnnotation) ){
                  tr <- as.data.frame( additionalAnnotation[[j]] )[,c("start", "end")]
                  drawGene(min(sub$start), max(sub$end), tr=tr, exoncol="darkred", names, trName=names(additionalAnnotation)[j], cex=0.8, introncol="darkred")
                  i <- i + 1
                  if( i > 40 ) break
              }
          }
      }
      axis(1, at=round(seq(min(sub$start), max(sub$end), length.out=10)), labels=round(seq(min(sub$start), max(sub$end), length.out=10)), pos=0, lwd.ticks=0.2, padj=-0.7, ...)   ########## genome axis
   }
   if(legend){
      mtext(paste(geneID, unique(sub$strand)), side=3, adj=0.25, padj=1.5, line=0, outer=TRUE, cex=1.5)
      posforlegend <- seq(.7, .9, length.out=numcond)
      for(i in seq(along=color)) {
         mtext(names(color[i]), side=3, adj=posforlegend[i], padj=1.5, line=0, outer=TRUE, col=color[i], ...)
      }
   }else{
      mtext(paste(geneID, unique(sub$strand)), side=3, adj=0.5, padj=1.5, line=0, outer=TRUE, cex=1.5)
   }
}

####################
#FUNCTION TO MAKE THE AXIS OF THE VST VALUES
####################

makevstaxis <- function(min, ylimn, ecs, ...)
{
   minlog10 <- floor( log10( 1/nrow( sampleAnnotation(ecs) ) ) )
   maxlog10 <- ceiling( log10( ylimn[2] ) )
   ticks <- 10^seq( minlog10, maxlog10 )
   decade_lengths <- ( vst(ticks, ecs)[ 2 : length(ticks) ] - vst(ticks, ecs)[ 1 : (length(ticks)-1) ] ) /
      ( vst( ylimn[2], ecs) - vst( ylimn[1], ecs) )
#   ticks <- c( 0, ticks[ min( which( decade_lengths > .1 ) ) : length(ticks) ] )
   mlength <- which( decade_lengths > .1)
   if( length( mlength ) ){
      fromw <- pmax(mlength, 0, na.rm=TRUE)[1]
   }else{
      fromw <- 0
   }
   ticks <- ticks[ fromw : length(ticks) ]
   axis( 2, at=vst(c(0, ticks), ecs), labels=c("",ticks), las=2, pos=0, ...)

   for( i in minlog10 : (maxlog10-1) ) {
      decade_length <- ( vst( 10^(i+1), ecs) - vst( 10^i, ecs) ) / ( vst( ylimn[2], ecs) - vst( ylimn[1], ecs) )
      if( decade_length > .1 ) {
         axis( 2, at = vst( 1:9 * 10^i, ecs), labels = FALSE, las=2, tcl=-.25, pos=0, ...)
      }
      if( decade_length > .4 & decade_length <= .6) {
         axis( 2, at = vst( c(2,3,5) * 10^i, ecs ), labels = ( c(2,3,5) * 10^i ), las=2, tcl=-.25, pos=0, ...)
      } else if( decade_length > .6 ) {
         axis( 2, at = vst( c(1.5,2:9) * 10^i, ecs ), labels = ( c(1.5,2:9) * 10^i ), las=2, tcl=-.25, pos=0, ...)
      }
   }
#   axis( 2, at=vst(0, ecs), labels=FALSE, las=2, pos=0, ...)
}



#######################
#FUNCTION TO WRITE THE PLOTS:
#######################
drawPlot <- function(matr, ylimn, ecs, intervals, rango, fitExpToVar, numexons, textAxis, rt, color, colorlines, ...)
{
   plot.new()
   plot.window(xlim=c(0, 1), ylim=c(0, max(matr)))
   makevstaxis(1/ncol(matr), ylimn, ecs, ...)
   intervals<-(0:nrow(matr))/nrow(matr)
   middle <- apply(cbind(intervals[rango], (intervals[rango+1]-((intervals[rango+1])-intervals[rango])*0.2)), 1, median)
   matr<-rbind(matr, NA)
   j <- seq_len(ncol(matr))
   segments(intervals[rango], matr[rango,j], intervals[rango+1]-((intervals[rango+1]-intervals[rango])*0.2), matr[rango,j], col=color, ...)  #### line with the y level
   segments(intervals[rango+1]-((intervals[rango+1]-intervals[rango])*0.2), matr[rango,j], intervals[rango+1], matr[rango+1,j], col=color, lty="dotted", ...)  #### line joining the y levels
   abline(v=middle[rango], lty="dotted", col=colorlines)
   mtext(textAxis, side=2, adj=0.5, line=1.5, outer=FALSE, ...)
   axis(1, at=middle[seq(along=rt)], labels=featureIDs(ecs)[rt], ...)
}

#########################
#FUNCTION TO WRITE THE GENE MODELS:
#########################
drawGene <- function(minx, maxx, tr, exoncol=NULL, names, trName, newPanel=TRUE,
                     drawIntronLines=TRUE, drawNames=TRUE, drawExons=TRUE, miny=0, maxy=1, introncol="black", ...)
{
    if( newPanel ){
        plot.new()
        plot.window(xlim=c(minx, maxx), ylim=c(0, 1))
    }
    rango <- seq_len(nrow(tr))
    if( drawExons ){
        rect(tr[rango,"start"], miny, tr[rango,"end"], maxy, col=exoncol)
    }
    if( drawIntronLines ){
        zr <- apply(rbind(tr[rango, "end"], tr[rango+1, "start"]), 2, median)
        segments(tr[rango,"end"], 0.5, zr, 0.65, col=introncol)
        segments(zr, 0.65, tr[rango+1,"start"], 0.5, col=introncol)
    }
    if(names & drawNames){
        mtext(trName, side=2, adj=0.5, padj=1, line=1, outer=FALSE, las=2, ...)
    }
}

estimateSizeFactors.DEXSeqDataSet <- function(object, locfunc=median, geoMeans) {
  # Temporary hack for backward compatibility with "old" DEXSeqDataSet
  # objects. Remove once all serialized DEXSeqDataSet objects around have
  # been updated.
  if (!.hasSlot(object, "rowRanges"))
      object <- updateObject(object)
  normFactors <- estimateSizeFactorsForMatrix(featureCounts(object), locfunc, geoMeans=geoMeans)
  sizeFactors(object) <- rep(normFactors, 2)
  maxExons <- length( unique( object@modelFrameBM$exon ) )
  object@modelFrameBM$sizeFactor <- rep(normFactors, each=maxExons)
  object
}

setMethod("estimateSizeFactors", signature(object="DEXSeqDataSet"),
          estimateSizeFactors.DEXSeqDataSet)

estimateDispersions.DEXSeqDataSet <- function( object, fitType=c("parametric","local","mean", "glmGamPoi"),
      maxit=100, niter=10, quiet=FALSE, formula=design(object), BPPARAM=SerialParam()) {
      ## Temporary hack for backward compatibility with "old" DEXSeqDataSet
      ## objects. Remove once all serialized DEXSeqDataSet objects around have
      ## been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    if (is.null(sizeFactors(object)) & is.null(normalizationFactors(object))) {
        stop("first call estimateSizeFactors or provide a normalizationFactor matrix before estimateDispersions")
    }
    if (!is.null(dispersions(object))){
        if (!quiet) message("you had estimated dispersions, replacing these")
        mcols(object) <- mcols(object)[,!(mcols(mcols(object))$type %in% c("intermediate","results"))]
    }
    stopifnot(length(maxit)==1)
    fitType <- match.arg(fitType, choices=c("parametric","local","mean", "glmGamPoi"))
    
    allVars <- all.vars(formula)
    if( any(!allVars %in% colnames( colData(object) )) ){
        notPresent <- allVars[!allVars %in% colnames( colData(object) )]
        notPresent <- paste(notPresent, collapse=",")
        stop(sprintf("the variables '%s' of the parameter 'formula' are not specified in the columns of the colData", notPresent ))
    }

    if( fitType == "glmGamPoi" & !is(BPPARAM, "SerialParam") ){
        message("\nParallelization has not been implemented for estimation of dispersions using glmGamPoi, using a single core for this estimation.\n")
        BPPARAM <- SerialParam()
    }

    if( is( BPPARAM, "SerialParam" ) ){
        numParts <- 1L
    }else{
        numParts <- BPPARAM$workers
    }
    
    splitParts <- sort( rep( seq_len( numParts ), length.out=nrow(object) ) )
    splitObject <- split( object, splitParts )
    
    modelMatrix <- rmDepCols(
        model.matrix(formula, as.data.frame(colData(object))))
    
    glmType <- ifelse( fitType == "glmGamPoi", "glmGamPoi", "DESeq2" )
    
    splitObject <- bplapply( splitObject,
        function(x, ... ){
            estimateDispersionsGeneEst(x,
                maxit=maxit, quiet=quiet,
                modelMatrix = modelMatrix,
                niter = niter, type=glmType ) },
        maxit=maxit, quiet=quiet,
        modelMatrix=modelMatrix,
        glmType=glmType,
        niter=niter,
        BPPARAM=BPPARAM )

    mergeObject <- do.call( rbind, splitObject )
    matchedNames <- match( rownames(object), rownames(mergeObject))
    mcols(object) <- mcols( mergeObject )[matchedNames,]
    assays(object) <- assays(mergeObject[matchedNames,])
    
    mcols(object)$baseMean <- rowMeans( featureCounts(object, normalized=TRUE) )
    mcols(object)$baseVar <- mcols(object)$exonBaseVar
    mcols(object)$allZero <-
                    unname( rowSums( featureCounts(object)) == 0 |
                        rowSums(counts(object)[, colData(object)$exon == "others"]) ==0 )

    ## object <- estimateDispersionsFit(object, fitType=fitType, quiet=quiet)
    object <- estimateDispersionsFit(object, fitType=fitType, quiet=quiet)

    dispPriorVar <- estimateDispersionsPriorVar(object, modelMatrix=modelMatrix)

    splitObject <- split( object, splitParts )
    
    splitObject <- bplapply( splitObject,
        function(x, ... ){
            estimateDispersionsMAP(x,
                maxit=maxit,
                quiet=quiet,
                modelMatrix=modelMatrix,
                dispPriorVar=dispPriorVar,
                type=glmType)
        },
        maxit=maxit, quiet=quiet,
        modelMatrix=modelMatrix,
        dispPriorVar=dispPriorVar,
        glmType=glmType,
        BPPARAM=BPPARAM )

    if( fitType == "glmGamPoi" ){
        attr(object, "quasiLikelihood_df0") <- attr(splitObject[[1L]], "quasiLikelihood_df0")
    }
    
    mergeObject <- do.call( rbind, splitObject )
    matchedNames <- match( rownames(object), rownames(mergeObject) )
    mcols(object) <- mcols( mergeObject )[matchedNames,]
    mcols(object)$baseMean <- unname( rowMeans( counts(object, normalized=TRUE) ) )
    mcols(object)$baseVar <- unname( rowVars( counts(object, normalized=TRUE) ) )
    mcols(object)$dispersion <- pmin( mcols(object)$dispersion, ncol(object) )
    object
}

#' @docType methods
#' @name estimateDispersions
#' @aliases estimateDispersions,DEXSeqDataSet-method
#' @title Estimate the dispersions for a DEXSeqDataSet
#'
#' @param object A DEXSeqDataSet.
#' @param fitType Either "parametric", "local", "mean" or "glmGamPoi"
#' for the type of fitting of dispersions to the mean
#' intensity. See ?estimateDispersions,DESeqDataSet-method for details.
#' If "glmGamPoi" is selected, the GLM fitter from "glmGamPoi" is also
#' used for estimating the dispersion estimates.
#' @param  maxit Control parameter: maximum number of iterations to allow for convergence
#' @param niter Number of times to iterate between estimation of means and estimation of dispersion.
#' @param quiet Whether to print messages at each step.
#' @param formula Formula used to fit the dispersion estimates.
#' @param BPPARAM A "BiocParallelParam" instance. See \code{?bplapply} for details.
#'
#' @return A DEXSeqDataSet with the dispersion information filled in as metadata columns.
#' @description This function obtains dispersion estimates for negative binomial distributed data for the specific case for DEXSeq.
#'
#' @details See ?estimateDispersions,DESeqDataSet-method for details.
#'
#' @examples
#' data(pasillaDEXSeqDataSet, package="pasilla")
#' dxd <- estimateSizeFactors( dxd )
#' dxd <- estimateDispersions( dxd )
#'
#' @exportMethod estimateDispersions
#' 
setMethod( "estimateDispersions", signature(object="DEXSeqDataSet"),
          estimateDispersions.DEXSeqDataSet )

plotDispEsts.DEXSeqDataSet <- function(object, ymin,
       genecol = "black", fitcol = "red", finalcol = "dodgerblue",
       legend=TRUE, xlab, ylab, log = "xy", cex = 0.45, ...)
{
  stopifnot(is(object, "DEXSeqDataSet"))
  # Temporary hack for backward compatibility with "old" DEXSeqDataSet
  # objects. Remove once all serialized DEXSeqDataSet objects around have
  # been updated.
  if (!.hasSlot(object, "rowRanges"))
      object <- updateObject(object)
  mcols(object)$baseMean <-
    mcols(object)$exonBaseMean
  mcols(object)$baseVar <-
    mcols(object)$exonBaseVar
  plotDispEsts( as(object, "DESeqDataSet"), genecol=genecol, fitcol=fitcol,
    finalcol=finalcol, legend=legend, log=log, cex=cex, ...)
}

setMethod( "plotDispEsts", signature(object="DEXSeqDataSet"),
  plotDispEsts.DEXSeqDataSet )


plotMA.DEXSeqDataSet <- function( object, alpha=0.1, ylim=c(-2, 2), foldChangeColumn=NULL, ...){
  stopifnot( is(object, "DEXSeqDataSet") )
  # Temporary hack for backward compatibility with "old" DEXSeqDataSet
  # objects. Remove once all serialized DEXSeqDataSet objects around have
  # been updated.
  if (!.hasSlot(object, "rowRanges"))
      object <- updateObject(object)
  hasResults <- attr(object, "results")
  dexseqResults <- mcols(object)[,mcols( mcols( object ) )$type == "DEXSeq results"]
  if( ncol( dexseqResults ) == 0 ){
    stop("first call estimateExonFoldChanges")
  }
  if( is.null(foldChangeColumn)){
    y <- dexseqResults[,grep("log2fold", colnames(dexseqResults))[1]]
  }else{
    y <- dexseqResults[,foldChangeColumn]
  }
  x <- rowMeans( featureCounts(object, normalized=TRUE) )
  df <- data.frame( x, y, results(object)$padj < alpha )
#  ylim=c(-2, 2)
  plotMA(df, ylim, ... )
}

setMethod("plotMA", signature(object="DEXSeqDataSet"),
          plotMA.DEXSeqDataSet)

plotMA.DEXSeqResults <- function(object, alpha=0.1, ylim=c(-2,2), foldChangeColumn=NULL, ...){
  stopifnot( is(object, "DEXSeqResults") )
  # Temporary hack for backward compatibility with "old" DEXSeqDataSet
  # objects. Remove once all serialized DEXSeqDataSet objects around have
  # been updated.
  if (!.hasSlot(object, "rowRanges"))
      object <- updateObject(object)
  x <- rowMeans( counts(object, normalized=TRUE) )
  dexseqResults <- object[,which( mcols( object )$type == "DEXSeq results" )]
  if( is.null(foldChangeColumn)){
    y <- dexseqResults[,grep("log2fold", colnames(dexseqResults))[1]]
  }else{
    y <- dexseqResults[,foldChangeColumn]
  }
  df <- data.frame( x, y, object$padj < alpha )
#  ylim=c(-2, 2)
  plotMA(df, ylim, ... )
}

setMethod("plotMA", signature(object="DEXSeqResults"),
          plotMA.DEXSeqResults)

show.DEXSeqResults <- function(object){
  cat("\n")
  cat(mcols(object)[colnames(object) == "pvalue","description"])
  cat("\n\n")
  show(DataFrame(object))
}

setMethod("show", signature(object="DEXSeqResults"),
          show.DEXSeqResults )

counts.DEXSeqResults <- function(object, normalized=FALSE){
    if( normalized ){
        t(t( object$countData ) / object@sampleData$sizeFactor)
    }else{
       object$countData
   }
}

setMethod( "counts", signature(object="DEXSeqResults"),
          counts.DEXSeqResults )


subsetByOverlaps.DEXSeqResults <- function( x, ranges, maxgap = -1L, minoverlap = 0L,
         type = c("any", "start", "end", "within", "equal"),
         ignore.strand = FALSE ){
  stopifnot( is( x, "DEXSeqResults") )
  genomicData <- x$genomicData
  overlaps <- findOverlaps( query=genomicData, subject=ranges, maxgap=maxgap,
    minoverlap=minoverlap, type=type,
    ignore.strand=ignore.strand )
  x[queryHits( overlaps ),]
}

setMethod("subsetByOverlaps", signature(x="DEXSeqResults", ranges="GenomicRanges"),
          subsetByOverlaps.DEXSeqResults)

findOverlaps.DEXSeqResults <- function( query, subject, maxgap = -1L, minoverlap = 0L,
         type = c("any", "start", "end", "within", "equal"),
         ignore.strand = FALSE ){
  stopifnot( is( query, "DEXSeqResults") )
  genomicData <- query$genomicData
  overlaps <- findOverlaps( query=genomicData, subject=subject, maxgap=maxgap,
    minoverlap=minoverlap, type=type,
    ignore.strand=ignore.strand)
  overlaps
}

setMethod("findOverlaps", signature(query="DEXSeqResults", subject="GenomicRanges"),
          findOverlaps.DEXSeqResults)

setMethod("[", "DEXSeqDataSet", function(x, i, j, ..., drop = FALSE) {
    x <- callNextMethod()
    colData(x) <- droplevels(colData(x))
    x@modelFrameBM <- makeBigModelFrame(x)
    validObject(x)
    x
})

setReplaceMethod("colData", c("DEXSeqDataSet", "DataFrame"),
    function(x, ..., value)
{
    x <- callNextMethod()
    x@modelFrameBM <- makeBigModelFrame(x)
    validObject(x)
    x
})

setReplaceMethod("$", "DEXSeqDataSet",
    function(x, name, value)
{
    x <- callNextMethod()
    x@modelFrameBM <- makeBigModelFrame(x)
    validObject(x)
    x
})

#' @import Biobase BiocGenerics hwriter methods stringr statmod biomaRt GenomicRanges S4Vectors IRanges Rsamtools DESeq2 BiocParallel geneplotter genefilter SummarizedExperiment
#' @importFrom AnnotationDbi select
#' @importFrom RColorBrewer brewer.pal
#' @importFrom grDevices colorRamp colorRampPalette dev.off rgb svg
#' @importFrom graphics abline axis hist layout mtext par plot.new plot.window rect segments
#' @importFrom stats as.formula coefficients model.matrix runif terms weighted.mean
#' @importFrom utils read.delim read.table
#' @importFrom stats relevel
#' @exportClass DEXSeqDataSet DEXSeqResults
#' @exportMethod  plotMA counts estimateSizeFactors subsetByOverlaps findOverlaps `[` `$<-` `colData<-`
#' @export geneIDs `geneIDs<-` groupIDs `groupIDs<-` exonIDs `exonIDs<-` featureIDs `featureIDs<-` DEXSeqHTML plotDEXSeq featureCounts perGeneQValue DEXSeqDataSet DEXSeqDataSetFromSE DEXSeqDataSetFromHTSeq DEXSeqResults estimateExonFoldChanges DEXSeq sampleAnnotation

rmDepCols <- function(m){
    q <- qr(m)
    if (q$rank < ncol(m)) 
        m[, -q$pivot[(q$rank + 1):ncol(m)]]
    else m
}


#BPPARAM <- MulticoreParam(workers=1)
#fullModel <- design(object)
#reducedModel <- ~ sample + exon
vst <- function( x,  object ){
    if( is.null( attr(object@dispersionFunction, "fitType") ) ) {
        warning("Dispersion function not found, applying log2(x+ 1) instead of vst...\n")
        return( log10(x+1) )
    }else if ( attr(object@dispersionFunction, "fitType") != "parametric" ){
        warning("Dispersion function not parametric, applying log2(x+ 1) instead of vst...\n")
        return( log10(x+1) )
    }
  coefs <- attr(object@dispersionFunction, "coefficients")
    (2/(sqrt(coefs["asymptDisp"]))) * log(2 * coefs["asymptDisp"] * 
      sqrt(x) + 2 * sqrt(coefs["asymptDisp"] * (coefs["extraPois"] + 
        1 + coefs["asymptDisp"] * x))) - (2/(sqrt(coefs["asymptDisp"]))) * 
          log(2 * sqrt(coefs["asymptDisp"] * (coefs["extraPois"] + 1)))
}


#' @name testForDEU
#' @title Test for Differential Exon Usage
#' @description This will perform a likelihood ratio test for differential
#' exon usage. Internally, it calls the DESeq2 function \code{nbinomLRT}. 
#' @param object A DEXSeqDataSet object.
#' @param fullModel The full model formula.
#' @param reducedModel Null model formula.
#' @param BPPARAM A "BiocParallelParam" instance. See \code{?bplapply} for details.
#' @param fitType Specifies what internal engine to use for fitting the GLMs. Options are "DESeq2" or "glmGamPoi".
#' @details The information of the variables of the formulas
#' should be present in the \code{colData} of the
#' \code{DEXSeqDataSet} object.
#' @return A \code{DEXSeqDataSet} with slots filled with information about the test.
#'
#' @examples
#'
#' data(pasillaDEXSeqDataSet, package="pasilla")
#' dxd <- estimateSizeFactors( dxd )
#' dxd <- estimateDispersions( dxd )
#' dxd <- testForDEU( dxd )
#' @export 
testForDEU <-
  function(object,
           fullModel=design(object),
           reducedModel= ~ sample + exon,
           BPPARAM=SerialParam(), fitType=c("DESeq2", "glmGamPoi"))
{
  if (is.null(sizeFactors(object)) & is.null(normalizationFactors(object))) {
    stop("first call estimateSizeFactors or provide a normalizationFactor matrix")
  }
  if (is.null(dispersions(object))) {
    stop("first call estimateDispersions")
  }

  allVars <- all.vars(reducedModel)
  if( any(!allVars %in% colnames( colData(object) )) ){
     notPresent <- allVars[!allVars %in% colnames( colData(object) )]
     notPresent <- paste(notPresent, collapse=",")
     stop(sprintf("the variables '%s' of the parameter 'reducedModel' are not specified in the columns of colData", notPresent ) )
  }

  allVars <- all.vars(fullModel)
  if( any(!allVars %in% colnames( colData(object) )) ){
     notPresent <- allVars[!allVars %in% colnames( colData(object) )]
     notPresent <- paste(notPresent, collapse=",")
     stop(sprintf("the variables '%s' of the parameter 'fullModel' are not specified in the columns of colData", notPresent ) )
  }

  fullModelMatrix <- 
    rmDepCols( model.matrix( fullModel, as.data.frame(colData(object)) ) )

  reducedModelMatrix <- 
    rmDepCols( model.matrix( reducedModel, as.data.frame(colData(object)) ) )

  if( is( BPPARAM, "SerialParam" ) ){
    numParts <- 1L
  }else{
    numParts <- BPPARAM$workers
  }
  
  splitParts <- sort( rep( seq_len( numParts ), length.out=nrow(object) ) )
  splitObject <- split( object, splitParts )

  splitObject <- bplapply(
      splitObject,
      function(x, ... ){
          nbinomLRT( x, reduced = reducedModelMatrix, full=fullModelMatrix, type=fitType )
      },
      reducedModelMatrix=reducedModelMatrix, fullModelMatrix=fullModelMatrix,
      fitType=fitType,
      BPPARAM=BPPARAM )

  mergeObject <- do.call(rbind, splitObject)
  matchedNames <- match( rownames(object), rownames(mergeObject))  
  mcols(object) <- mcols( mergeObject )[matchedNames,]
  assays(object) <- assays(mergeObject[matchedNames,])

  extraAttributes <- setdiff( names( attributes(splitObject[[1]]) ),  names( attributes(object) ) )

  for( atr in extraAttributes ){
    attr( object, atr ) <- attr( splitObject[[1]], atr )
  }

  object
 
}

estimateExonFoldChanges <- function( object,
                                    fitExpToVar = "condition",
                                    denominator = "",
                                    BPPARAM=SerialParam(), 
                                    maxRowsMF=2400, independentFiltering=FALSE, filter)
{
    stopifnot(is(object, "DEXSeqDataSet"))
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    if (any(is.na(sizeFactors(object)))) {
      stop("Please estimate sizeFactors first\n")
    }
    if (!fitExpToVar %in% colnames(sampleAnnotation(object))) {
      stop(sprintf("The value of the parameter fitExpToVar,'%s', is not a column name of colData", fitExpToVar))
    }
    if ( is.null( dispersions(object) ) ){
      stop("please call estimateDispersions first")
    }
    if ( length( attr( object, "test" ) ) == 0 ){
      stop("please call testForDEU first")
    }
    frm <- as.formula(paste("count ~", fitExpToVar, "* exon"))
    if( independentFiltering ){
        if( missing(filter) ){
            filter=rowMeans(featureCounts(object, normalized = TRUE))
        }
        notNAs <- !is.na( results(object, filter=filter)$padj )
    }else{
        notNAs <- rep(TRUE, nrow(object))
    }
    notNAs <- notNAs & !mcols(object)$allZero
    groups <- groupIDs(object)
    disps <- dispersions(object)
    disps[is.na(disps)] <- 1e-6
    mf <- object@modelFrameBM
    if( is( mf[[fitExpToVar]], "numeric" ) ){
        maxRowsMF <- 0
    }
    numsamples <- nrow( sampleAnnotation(object) )
    features <- featureIDs(object)
    countsAll <- featureCounts(object)
    allExonIDs <- as.character( mf$exon )
    testablegenes <- groups[notNAs]

    ###### separate genes with few exons from large exons
    ###### and fit the different but equivalent models separately
    ######
    
    numOfExonsLimitBM <- round(maxRowsMF/numsamples)
    exonsPerGene <- table(testablegenes)
    testablegenesBM <- names(exonsPerGene[exonsPerGene < numOfExonsLimitBM])
    testablegenesSM <- names(exonsPerGene)[!names(exonsPerGene) %in% testablegenesBM]
    if( length(testablegenesBM) > 0 ){
       alleffectsBM <- bplapply( testablegenesBM,
                              getEffectsForGeneBM,
                              groups=groups, notNAs=notNAs,
                              countsAll=countsAll, disps=disps,
                              features=features, mf=mf, frm=frm,
                              numsamples=numsamples,
                              fitExpToVar=fitExpToVar,
                              BPPARAM=BPPARAM )
       alleffectsBM <- do.call(rbind, alleffectsBM)
    }else{
        alleffectsBM <- NULL
    }

    exonIndexes <- which( groups %in% testablegenesSM & notNAs )

    if( length(exonIndexes) > 0 ){
        countsAll <- counts( object )
        mfSmall <- modelFrameSM(object)
        alleffectsSM <- bplapply( exonIndexes, getEffectsForExonsSM,
                                 frm=frm, countsAll=countsAll,
                                 disps=disps, mfSmall=mfSmall,
                                 fitExpToVar=fitExpToVar, 
                                 BPPARAM=BPPARAM) 
        names(alleffectsSM) <- rownames(object)[exonIndexes]
#        alleffectsSM <- t(simplify2array(alleffectsSM))
        alleffectsSM <- do.call( rbind, alleffectsSM )
    }else{
        alleffectsSM <- NULL
    }

    alleffects <- rbind( alleffectsBM, alleffectsSM )
#    alleffects[
    alleffectsVst <- vst(exp(alleffects), object)
    toadd <- matrix(NA, nrow = nrow(object), ncol = ncol(alleffects))
    rownames(toadd) <- rownames(object)
    colnames(toadd) <- colnames(alleffects)
    toadd[rownames(alleffects), colnames(alleffects)] <- alleffectsVst
    toadd <- DataFrame(toadd)
    mcols(toadd) <- DataFrame(
        type=rep("DEXSeq results",ncol(toadd)),
        description=rep("exon usage coefficient",ncol(toadd) ) )
    toadd2 <- matrix(NA, nrow = nrow(object), ncol = ncol(alleffects) )
    if (denominator == "") {
#        denominator <- as.character(sampleAnnotation(object)[[fitExpToVar]][1])
        denominator=colnames(alleffects)[1]
    }
    colRemove <- colnames(alleffects) %in% denominator
    stopifnot(any(colRemove))
    denoCol <- which(colnames(alleffects) == denominator)
    alleffects <- alleffects / log(2)
    alleffects <- alleffects - alleffects[, denoCol]
    colnames(alleffects) <- sprintf("log2fold_%s_%s", colnames(alleffects), 
            denominator)
    colnames(toadd2) <- colnames(alleffects)
    rownames(toadd2) <- rownames(object)
    toadd2[rownames(alleffects), colnames(alleffects)] <- alleffects
    toadd2 <- toadd2[,-denoCol, drop=FALSE]
    toadd2 <- DataFrame(toadd2)
    mcols(toadd2) <- DataFrame(
        type=rep("DEXSeq results",ncol(toadd2)),
        description=rep("relative exon usage fold change",ncol(toadd2) ) )
    allAdd <- cbind(toadd, toadd2)
    mcols(object) <- cbind(mcols(object), allAdd)
    object
}


DEXSeqResults <- function( object, independentFiltering=TRUE, filter){
  stopifnot( is(object, "DEXSeqDataSet"))
  # Temporary hack for backward compatibility with "old" DEXSeqDataSet
  # objects. Remove once all serialized DEXSeqDataSet objects around have
  # been updated.
  if (!.hasSlot(object, "rowRanges"))
      object <- updateObject(object)
  if( missing( filter ) ){
      filter=rowMeans( featureCounts( object, normalized=TRUE ) )
  }
  LRTresults <- results(object, filter=filter, independentFiltering=independentFiltering )
  LRTresults$exonBaseMean <- rowMeans(featureCounts(object, normalized=TRUE))
  LRTresults$featureID <- mcols(object)$featureID
  LRTresults$groupID <- mcols(object)$groupID
  LRTresults$dispersion <- mcols(object)$dispersion
  
  LRTresults <- LRTresults[,c("groupID", "featureID", "exonBaseMean", "dispersion", "stat", "pvalue", "padj")]
  mcols( LRTresults )[colnames(LRTresults) %in% c("groupID", "featureID", "exonBaseMean"),"type"] <- "input"
  mcols( LRTresults )[colnames(LRTresults) %in% "groupID","description"] <- "group/gene identifier"
  mcols( LRTresults )[colnames(LRTresults) %in% "featureID","description"] <- "feature/exon identifier"
  mcols( LRTresults )[colnames(LRTresults) %in% "exonBaseMean","description"] <- "mean of the counts across samples in each feature/exon"
  mcols( LRTresults )[colnames(LRTresults) %in% "dispersion","description"] <- "exon dispersion estimate"
  toadd <- mcols(object)[,mcols( mcols(object ) )$type == "DEXSeq results", drop=FALSE]
  LRTresults <- cbind( LRTresults, toadd )
  genomicData <- rowRanges(object)
  mcols(genomicData) <- NULL
  LRTresults$genomicData <- genomicData
  LRTresults$countData <- featureCounts(object)
  LRTresults$transcripts <- mcols(object)$transcripts
  mcols( LRTresults )[colnames(LRTresults) %in% c("genomicData", "countData", "transcripts"),"type"] <- "input"
  mcols( LRTresults )[colnames(LRTresults) %in% "genomicData","description"] <- "GRanges object of the coordinates of the exon/feature"
  mcols( LRTresults )[colnames(LRTresults) %in% "countData","description"] <- "matrix of integer counts, of each column containing a sample"
  mcols( LRTresults )[colnames(LRTresults) %in% "transcripts","description"] <- "list of transcripts overlapping with the exon"
  dxr <-
      new("DEXSeqResults",
          LRTresults,
          modelFrameBM=object@modelFrameBM,
          sampleData = sampleAnnotation(object),
          dispersionFunction = object@dispersionFunction )

  dxr
}

DEXSeq <- function( object,
                   fullModel=design(object),
                   reducedModel = ~ sample + exon,
                   BPPARAM=MulticoreParam(workers=1), fitExpToVar="condition", quiet=TRUE ){
  stopifnot(is( object, "DEXSeqDataSet") )
  # Temporary hack for backward compatibility with "old" DEXSeqDataSet
  # objects. Remove once all serialized DEXSeqDataSet objects around have
  # been updated.
  if (!.hasSlot(object, "rowRanges"))
      object <- updateObject(object)
  object <- estimateSizeFactors( object )
  object <- estimateDispersions( object, formula=fullModel, BPPARAM=BPPARAM, quiet=TRUE)
  object <- testForDEU( object, reducedModel=reducedModel, fullModel=fullModel, BPPARAM=BPPARAM )
  object <- estimateExonFoldChanges( object, fitExpToVar=fitExpToVar )
  res <- DEXSeqResults( object )
  res
}

context("glmGamPoi")
test_that( "Funcion to create DEXSeqDataSet has the correct inputs", {

    data("pasillaDEXSeqDataSet", package="pasilla")

    dxd <- estimateSizeFactors( dxd )

    library(glmGamPoi)
    expect_message(
        dxd_glmgampoi <- estimateDispersions( dxd, fitType="glmGamPoi", BPPARAM=MulticoreParam(2), niter=2 ),
        "Parallelization has not been implemented for estimation" )
    
    dxd_glmgampoi <- testForDEU( dxd_glmgampoi, fitType="glmGamPoi" )
    expect_s4_class(dxd_glmgampoi, "DEXSeqDataSet")

    dxd_default <- estimateDispersions( dxd )
    dxd_default <- testForDEU( dxd_default )
    expect_s4_class(dxd_default, "DEXSeqDataSet")

    dxd_default_p2 <- estimateDispersions( dxd_default, BPPARAM=MulticoreParam(2) )
    dxd_default_p2 <- testForDEU( dxd_default_p2, BPPARAM=MulticoreParam(2) )
    expect_s4_class(dxd_default, "DEXSeqDataSet")

    res_default <- DEXSeqResults( dxd_default )
    res_default_p2 <- DEXSeqResults( dxd_default_p2 )

    expect_identical(res_default$pvalue, res_default_p2$pvalue)
    ## res_glmgampoi <- DEXSeqResults(dxd_glmgampoi, independentFiltering=FALSE)
    ## res_default <- DEXSeqResults(dxd_default, independentFiltering=FALSE)
    ## table( glmgampoi=res_glmgampoi$padj < 0.1, def=res_default$padj < 0.1 )

    dxd_glmgampoi_p2 <- testForDEU( dxd_glmgampoi, fitType="glmGamPoi", BPPARAM=MulticoreParam(3) )
    res_glmgampoi <- DEXSeqResults( dxd_glmgampoi )
    
    res_glmgampoi_p2 <- DEXSeqResults( dxd_glmgampoi_p2 )
    expect_identical( res_glmgampoi$pvalue, res_glmgampoi_p2$pvalue )

} )

setClass("DEXSeqDataSet",
    contains = "DESeqDataSet",
        representation = representation( modelFrameBM = "data.frame") )

DEXSeqDataSet <- function( countData, sampleData, design= ~ sample + exon + condition:exon , featureID, groupID, featureRanges=NULL, transcripts=NULL, alternativeCountData=NULL)
{
    ### Checking inputs ###
    if( !(is( countData, "matrix" ) | is( countData, "data.frame" )) )
      stop( "Unexpected input: the parameter 'countData' must be either a matrix or a data.frame", 
            call.=FALSE )
    countData <- as.matrix( countData )
    if( !( is( featureID, "character" ) | is( featureID, "factor" ) ) )
      stop( "Unexpected input: the parameter 'featureID' must be either a character or a factor", 
            call.=FALSE )
    if( !( is( groupID, "character" ) | is( groupID, "factor" ) ) )
      stop( "Unexpected input: the parameter 'groupID' must be either a character or a factor", 
            call.=FALSE )
    if( !is( sampleData, "data.frame" ) )
      stop( "Unexpected input: the parameter 'sampleData' must be a data.frame", 
            call.=FALSE )
    rowNumbers <- nrow( countData )
    if( length(groupID) != rowNumbers )
      stop( "Unexpected length of 'groupID' parameter, it must be the same as the number of rows of countData", 
            call.=FALSE )
    if( length(featureID) != rowNumbers )
      stop( "Unexpected length of 'featureID' parameter, it must be the same as the number of rows of countData", 
            call.=FALSE )
    if( nrow(sampleData) != ncol(countData) )
      stop( "Unexpected number of rows of the 'sampleData' parameter, it must be the same as the number of columns of countData", 
            call.=FALSE )
    
    modelFrame <- cbind(
        sample = rownames(sampleData), sampleData )
    modelFrame <- rbind( cbind(modelFrame, exon = "this"),
                        cbind(modelFrame, exon = "others"))
    rownames(modelFrame) <- NULL
    colData <- DataFrame( modelFrame )
    
    if( !"exon" %in% all.vars( design ) )
        stop("The design formula does not specify an interaction contrast with the variable 'exon'", 
             call.=FALSE )

    allVars <- all.vars(design)
    if( any(!allVars %in% colnames( colData )) ){
        notPresent <- allVars[!allVars %in% colnames( colData ) ]
        notPresent <- paste(notPresent, collapse=",")
        stop(sprintf("The variables '%s', present in the design formula must be columns of 'sampleData'", 
                     notPresent ),
             call.=FALSE )
    }

    if( any( grepl(" |:", groupID ) | grepl(" |:", featureID) ) ) {
        warning("empty spaces or ':' characters were found either in your groupIDs or in your featureIDs, these will be removed from the identifiers")
        groupID <- gsub(" |:", "", groupID)
        featureID <- gsub(" |:", "", featureID)
    }

    rownames( countData ) <- paste( groupID, featureID, sep=":" )
    forCycle <- split( seq_len(nrow( countData )), as.character( groupID ) )

    if( is.null(alternativeCountData) ){
        others <- lapply( forCycle, function(i){
        sct <- countData[i, , drop = FALSE]
        rs <- t( vapply( seq_len(nrow(sct)), function(r) colSums(sct[-r, , drop = FALSE]), numeric(ncol(countData) ) ) )
        rownames(rs) <- rownames(sct)
        rs })
        others <- do.call(rbind, others)
    }else{
        stopifnot( identical(dim(countData), dim(alternativeCountData)) )
        stopifnot( identical( colnames(countData), colnames(alternativeCountData)))
        others <- alternativeCountData
        rownames( others ) <- paste( groupID, featureID, sep=":" )

    }
    
    stopifnot( all( rownames(countData) %in% rownames(others) ) )
    others <- others[rownames(countData),]
    nCountData <- cbind( countData, others )
    colnames(nCountData) <- NULL

    if( !is.null(featureRanges) ){
        stopifnot(is(featureRanges, "GRanges") ||
                  is(featureRanges, "GRangesList"))
        se <- SummarizedExperiment( nCountData, colData=colData, rowRanges=featureRanges )
    }else{
        se <- SummarizedExperiment( nCountData, colData=colData )
    }

    names(assays(se))[1] = "counts"
    mcols( se )$featureID <- featureID
    mcols( se )$groupID <- groupID
    mcols( se )$exonBaseMean <- rowMeans( countData )
    mcols( se )$exonBaseVar <- rowVars( countData )

    if( !is.null(transcripts) ){
        mcols(se)$transcripts <- transcripts
    }

    rownames(se) <- paste( groupID, featureID, sep=":")
    rse <- as( se, "RangedSummarizedExperiment" )
    mcols(rse) <- mcols(se)

    dds <- DESeqDataSet( rse, design, ignoreRank=TRUE )

    modelFrame <- makeBigModelFrame(dds)

    dxd <- new( "DEXSeqDataSet", dds, modelFrameBM=modelFrame )
    return(dxd)
}

makeBigModelFrame <- function(object){
    groupID <- mcols(object)$groupID
    featureID <- mcols(object)$featureID
    sampleData <- as.data.frame(colData(object)[colData(object)$exon == "this",])
    numExonsPerGene <- table(groupID)
    maxGene <- names(which.max(numExonsPerGene))
    rows <- mcols(object)$groupID %in%  maxGene
    numExons <- sum( rows )
    exonCol <-
        rep(factor(featureID[rows]), nrow(sampleData))
    modelFrame <- data.frame(
        sample=rep( sampleData$sample, each=numExons),
        exon = exonCol )
    varNames <- colnames(sampleData)[!colnames(sampleData) %in% c("sample", "exon")]
    for( i in varNames ){
        modelFrame[[i]] <- rep( sampleData[[i]], each=numExons )
    }
    modelFrame$dispersion <- NA
    if( is.null(object$sizeFactor )){
        modelFrame$sizeFactor <- NA
    }
    modelFrame$count <- NA
    modelFrame
}

setValidity( "DEXSeqDataSet", function( object ) {
    stopifnot(
        c("sample", "exon", "dispersion", "sizeFactor", "count")
        %in% colnames( object@modelFrameBM ) )
    stopifnot( all(object@modelFrameBM$sample %in% colData(object)$sample))
    stopifnot( all( c("sample", "exon") %in% colnames(colData(object)) ) )
    TRUE
} )

setClass("DEXSeqResults",
         contains = "DFrame",
         representation = representation( modelFrameBM = "data.frame", sampleData="DataFrame", dispersionFunction = "function") )


setValidity( "DEXSeqResults", function( object ){
    stopifnot( "sample" %in% colnames( object@sampleData ) )
    stopifnot( colnames(object$countData) == as.character(object@sampleData$sample) )
    TRUE
})

###########################
#### ACCESSOR FUNCTIONS####
###########################


featureCounts <- function( object, normalized=FALSE ){
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    validObject(object)
    res <- counts(object, normalized=normalized)[,colData(object)$exon == "this"]
    colnames( res ) <- sampleAnnotation(object)$sample
    res
}

featureIDs <- function(object){
    if( is( object, "DEXSeqDataSet") ){
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
        if (!.hasSlot(object, "rowRanges"))
            object <- updateObject(object)
        validObject(object)
        mcols(object)$featureID
    }else if( is(object, "DEXSeqResults") ){
        object$featureID
    }
}

`featureIDs<-` <- function( object, value ) {
     stopifnot( is( object, "DEXSeqDataSet" ) )
     # Temporary hack for backward compatibility with "old" DEXSeqDataSet
     # objects. Remove once all serialized DEXSeqDataSet objects around have
     # been updated.
     if (!.hasSlot(object, "rowRanges"))
         object <- updateObject(object)
     mcols(object)$featureID <- value
     rownames(object) <- paste( mcols(object)$groupID, mcols(object)$featureID, sep=":" )
     validObject(object)
     object
}

exonIDs <- function(object){
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    validObject(object)
    featureIDs(object)
}

`exonIDs<-` <- function( object, value ) {
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    object <- `featureIDs<-`( object, value )
    object
}

groupIDs <- function( object ){
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    validObject( object )
    mcols( object )$groupID
}

`groupIDs<-` <- function( object, value ) {
     stopifnot( is( object, "DEXSeqDataSet" ) )
     # Temporary hack for backward compatibility with "old" DEXSeqDataSet
     # objects. Remove once all serialized DEXSeqDataSet objects around have
     # been updated.
     if (!.hasSlot(object, "rowRanges"))
         object <- updateObject(object)
     mcols( object )$groupID <- value
     rownames(object) <- paste( mcols(object)$groupID, mcols(object)$featureID, sep=":" )
     validObject( object )
     object
}

geneIDs <- function( object ){
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    validObject( object )
    groupIDs( object )
}

`geneIDs<-` <- function( object, value ) {
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
    # been updated.
    if (!.hasSlot(object, "rowRanges"))
        object <- updateObject(object)
    object <- `groupIDs<-`(object, value)
    object
}


sampleAnnotation <- function( object ){
    # Temporary hack for backward compatibility with "old" DEXSeqDataSet
    # objects. Remove once all serialized DEXSeqDataSet objects around have
                                        # been updated.
    if( is( object, "DEXSeqDataSet")){
        if (!.hasSlot(object, "rowRanges"))
            object <- updateObject(object)
        validObject( object )
        colData( object )[colData( object )$exon == "this",!colnames(colData( object )) %in% "exon"]
    }else if( is(object, "DEXSeqResults") ){
        object@sampleData
    }
}

#################
###FROM HTSEQ####
#################


DEXSeqDataSetFromHTSeq <- function( countfiles, sampleData, design= ~ sample + exon + condition:exon, flattenedfile=NULL )
{
    if( !all( sapply(countfiles, class) == 'character' ) ){
        stop("The countfiles parameter must be a character vector")
    }
    lf <- lapply( countfiles, function(x)
        read.table( x, header=FALSE,stringsAsFactors=FALSE ) )
    if( !all( sapply( lf[-1], function(x) all( x$V1 == lf[1]$V1 ) ) ) )
        stop( "Count files have differing gene ID column." )
    dcounts <- sapply( lf, `[[`, "V2" )
    rownames(dcounts) <- lf[[1]][,1]
    dcounts <- dcounts[ substr(rownames(dcounts),1,1)!="_", ]
    rownames(dcounts) <- sub(":", ":E", rownames(dcounts))
    colnames(dcounts) <- countfiles
    splitted <- strsplit(rownames(dcounts), ":")
    exons <- sapply(splitted, "[[", 2)
    genesrle <- sapply( splitted, "[[", 1)
    if(!is.null(flattenedfile)){
        aggregates<-read.delim(flattenedfile, stringsAsFactors=FALSE, header=FALSE)
        colnames(aggregates)<-c("chr", "source", "class", "start", "end", "ex", "strand", "ex2", "attr")
        aggregates$strand <- gsub( "\\.", "*", aggregates$strand )
        aggregates<-aggregates[which(aggregates$class =="exonic_part"),]
        aggregates$attr <- gsub("\"|=|;", "", aggregates$attr)
        aggregates$gene_id <- sub(".*gene_id\\s(\\S+).*", "\\1", aggregates$attr)
        transcripts <- gsub(".*transcripts\\s(\\S+).*", "\\1", aggregates$attr)
        transcripts <- strsplit(transcripts, "\\+")
        exonids <- gsub(".*exonic_part_number\\s(\\S+).*", "\\1", aggregates$attr)
        exoninfo<-GRanges(
            as.character(aggregates$chr),
            IRanges(start=aggregates$start, end=aggregates$end),
            strand=aggregates$strand)
        names( exoninfo ) <- paste( aggregates$gene_id, exonids, sep=":E" )
        names(transcripts) <- rownames(exoninfo)
        if (!all( rownames(dcounts) %in% names(exoninfo) )){
            stop("Count files do not correspond to the flattened annotation file")
        }
        matching <- match(rownames(dcounts), names(exoninfo))
        stopifnot( all( names( exoninfo[matching] ) == rownames(dcounts) ) )
        stopifnot( all( names( transcripts[matching] ) == rownames(dcounts) ) )

        dxd <- DEXSeqDataSet( dcounts, sampleData, design, exons, genesrle,
                             exoninfo[matching], transcripts[matching] )
        return(dxd)
    }else{
        dxd <- DEXSeqDataSet( dcounts, sampleData, design, exons, genesrle)
        return(dxd)
    }
}

DEXSeqDataSetFromSE <- function( SE, design= ~ sample + exon + condition:exon ){
    if( !all( c("gene_id", "tx_name") %in% colnames( mcols( SE  ) ) ) ){
      stop("make sure your SummarizedExperiment object contain the columns gene_id and tx_name")
    }
    SE <- SE[order(mcols(SE)$gene_id),]
    groupID <- as.character( mcols(SE)$gene_id )
    ln <- table( groupID )
    mcols(SE)$exonic_part <- unlist( lapply( ln, function(x){1:x}) )
    stopifnot( all( rep(names(ln), ln) == groupID ) )
    featureID <- sprintf("E%3.3d",  mcols(SE)$exonic_part )
    transcripts <- as.list( mcols(SE)$tx_name )
    sampleData <- as.data.frame( colData(SE) )
    design <- design
    mcols(SE) <- NULL
    featureRanges <- rowRanges(SE)
    countData <- assay(SE)
    dxd <- DEXSeqDataSet( countData,
        sampleData,
        design,
        featureID,
        groupID,
        featureRanges=featureRanges,
        transcripts=transcripts )
    dxd
}

context( "Create DEXSeqDataSet" )
test_that( "Funcion to create DEXSeqDataSet has the correct inputs", {
  data(pasillaDEXSeqDataSet, package="pasilla")
  mat <- featureCounts( dxd )
  pDat <- as.data.frame( colData(dxd) )
  pDat <- droplevels(pDat[pDat$exon == "this",])
  pDat$exon <- NULL
  dxdSub <- DEXSeqDataSet( mat, pDat, featureID = featureIDs( dxd ), groupID=geneIDs(dxd) )
  expect_equal( counts(dxd), counts(dxdSub) )
  expect_s4_class( dxdSub, "DEXSeqDataSet" )
  expect_error(
      DEXSeqDataSet( as.vector(mat), pDat, featureID = featureIDs( dxd ), groupID=geneIDs(dxd) ),
      "Unexpected input: the parameter 'countData' must be either a matrix or a data.frame" )
  expect_error(
      DEXSeqDataSet( mat, pDat, featureID = seq_along( featureIDs( dxd ) ), groupID=geneIDs(dxd) ),
      "Unexpected input: the parameter 'featureID' must be either a character or a factor" )
  expect_error(
      DEXSeqDataSet( mat, pDat, featureID = featureIDs( dxd ), groupID=seq_along( geneIDs(dxd) ) ),
      "Unexpected input: the parameter 'groupID' must be either a character or a factor" )
  expect_error(
      DEXSeqDataSet( mat, NULL, featureID = featureIDs( dxd ), groupID=geneIDs(dxd) ),
      "Unexpected input: the parameter 'sampleData' must be a data.frame" )
  expect_error(
      DEXSeqDataSet( mat, pDat, featureID = featureIDs( dxd ), groupID=geneIDs(dxd)[seq_len(nrow(mat)-1)] ),
      "Unexpected length of 'groupID' parameter, it must be the same as the number of rows of countData" )
  expect_error(
    DEXSeqDataSet( mat, pDat, featureID = featureIDs( dxd )[seq_len(nrow(mat)-1)], groupID=geneIDs(dxd) ),
    "Unexpected length of 'featureID' parameter, it must be the same as the number of rows of countData" )
  expect_error(
    DEXSeqDataSet( mat, pDat[seq_len(ncol(mat)-1),], featureID = featureIDs( dxd ), groupID=geneIDs(dxd) ),
    "Unexpected number of rows of the 'sampleData' parameter, it must be the same as the number of columns of countData" )
  expect_error(
    DEXSeqDataSet( mat, design=~1, pDat, featureID = featureIDs( dxd ), groupID=geneIDs(dxd) ),
    "The design formula does not specify an interaction contrast with the variable 'exon'" )
  expect_error(
    DEXSeqDataSet( mat, pDat[,1,drop=FALSE], featureID = featureIDs( dxd ), groupID=geneIDs(dxd) ),
    "The variables 'condition', present in the design formula must be columns of 'sampleData'" )
} )

---
Title: "Inferring differential exon usage in RNA-Seq data with the DEXSeq package"
author:
- name: Alejandro Reyes
  affiliation: Dana-Farber Cancer Institute, Boston, USA
- name: Simon Anders
  affiliation: Heidelberg University, Heidelberg, USA
- name: Wolfgang Huber
  affiliation: European Molecular Biology Laboratory, Heidelberg, Germany
output:
  BiocStyle::html_document:
    toc_float: true
package: DEXSeq
bibliography: DEXSeq.bib
abstract: |
  The Bioconductor package `r BiocStyle::Biocpkg("DEXSeq")` implements a method 
  to test for
  differential exon usage in comparative RNA-Seq experiments. By *differential 
  exon usage* (DEU), we mean changes in the relative usage of exons caused by the
  experimental condition. The relative usage of an exon is defined as
  $\frac{\text{number of transcripts from the gene that contain this exon}}
  {\text{number of all transcripts from the gene}}$. The statistical method used by
  `r BiocStyle::Biocpkg("DEXSeq")` was introduced in our 
  paper [@DEXSeqPaper]. The basic concept 
  can be summarized as follows. For each exon (or part of an exon) and each 
  sample, we count how many reads map to this exon and how many reads map to 
  any of the other exons of the same gene. We consider the ratio of these two 
  counts, and how it changes across conditions, to infer changes in the relative 
  exon usage. In the case of an inner exon, a change in relative exon usage 
  is typically due to a change in the rate with which this exon is spliced 
  into transcripts (alternative splicing). Note, however, that DEU is a more
  general concept than alternative splicing, since it also includes 
  changes in the usage of alternative transcript start sites
  and polyadenylation sites, which can cause differential usage of 
  exons at the 5' and 3' boundary of transcripts.
  DEXSeq package version: `r packageVersion("DEXSeq")`
vignette: |
  %\VignetteIndexEntry{Inferring differential exon usage in RNA-Seq data with the DEXSeq package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r echo=FALSE, include=FALSE}
knitr::opts_chunk$set(tidy = FALSE,
                      cache = TRUE,
                      dev = "png",
                      message = FALSE,
                      error = FALSE,
                      warning = TRUE)
```

# Overview

Similar as with differential gene expression, we need to make sure that observed differences of exon usage values between conditions are statistically significant, i.e., are sufficiently unlikely to be just due to random fluctuations such as those seen even between samples from the same condition, i.e., between replicates. To this end, `r BiocStyle::Biocpkg("DEXSeq")` assesses the strength of these fluctuations (quantified by the so-called *dispersion*) by comparing replicates before comparing the averages between the sample groups.

The preceding description is somewhat simplified (and perhaps over-simplified), and we recommend that users consult the publication for a more complete description, as well as Appendix \@ref(methodChanges) of this vignette, which describes how the current implementation of `r BiocStyle::Biocpkg("DEXSeq")` differs from the original approach described in the paper. Nevertheless, two important aspects should be mentioned already here: First, `r BiocStyle::Biocpkg("DEXSeq")` does not actually work on the exon usage ratios, but on the counts in the numerator and denominator, to be able to make use of the information that is contained in the magnitude of count values. For example, 3000 reads versus 1000 reads is the same ratio as 3 reads versus 1 read, but the latter is a far less reliable estimate of the underlying true value, because of statistical sampling. Second, `r BiocStyle::Biocpkg("DEXSeq")` is not limited to simple two-group comparisons; rather, it uses so-called generalized linear models
(*GLMs*) to permit ANOVA-like analysis of potentially complex experimental designs.

# Preparations

## Example data

To demonstrate the use of `r BiocStyle::Biocpkg("DEXSeq")`, we use the *pasilla* dataset, an RNA-Seq dataset generated by [@Brooks2010]. They investigated the effect of siRNA knock-down of the gene *pasilla* on the transcriptome of fly *S2-DRSC* cells. The RNA-binding protein *pasilla* protein is thought to be involved in the regulation of splicing. (Its mammalian orthologs, NOVA1 and NOVA2, are well-studied examples of splicing factors.) Brooks et al. prepared seven cell cultures, treated three with siRNA to knock-down *pasilla* and left four as untreated controls, and performed RNA-Seq on all samples. They deposited the raw sequencing reads with the NCBI Gene Expression Omnibus (GEO) under the accession number [GSE18508](http://www.ncbi.nlm.nih.gov/projects/geo/query/acc.cgi?acc=GSE18508).

## Executability of the code

Usually, Bioconductor vignettes contain automatically executable code, i.e., you can follow the vignette by directly running the code shown, using functionality and data provided with the package. However, it would not be practical to include the voluminous raw data of the pasilla experiment here. Therefore, the code in the alignment section is not automatically executable. You may download the raw data yourself from GEO, as well as the required extra tools, and follow the work flow shown here and in the *pasilla* vignette [@pasillaVignette]. From Section \@ref(standard-analysis-workflow) on, code is directly executable, as usual. Therefore, we recommend that you just read this section, and try following our analysis in R only from the next section onwards. Once you work with your own data, you will want to come back and adapt the workflow shown here to your data.

## Alignment

The first step of the analysis is to align the reads to a reference genome. If you are using classic alignment methods (i.e. not pseudo-alignment approaches) it is important to align them to the genome, not to the transcriptome, and to use a splice-aware aligner (i.e., a short-read alignment tool that can deal with reads that span across introns) such as TopHat2 [@TopHat2], GSNAP [@GSNAP], or STAR [@STAR]. The explanation of the analysis workflow presented here starts with the aligned reads in the SAM format. If you are unfamiliar with the process of aligning reads to obtain SAM files, you can find a summary how we proceeded in preparing the pasilla data in the vignette for the `r BiocStyle::Biocpkg("pasilla")` data package [@pasillaVignette] and a more extensive explanation, using the same  data set, in our protocol article on differential expression calling [@DEProt].

## Preparing the annotation {#preparing-the-annotation-bioc}

In a transcript annotations such as a GTF file, many exons appear multiple times, once for each transcript that contains them. We need to "collapse" or "flatten" this information to define *exon counting bins*, i.e., a list of intervals, each corresponding to one exon or part of an exon. Counting bins for parts of exons arise when an exonic region appears with different boundaries in different transcripts. See Figure 1 of the `r BiocStyle::Biocpkg("DEXSeq")` paper (@DEXSeqPaper) for an illustration. 

Here we use the function `exonicParts()` of the package `r BiocStyle::Biocpkg("GenomicFeatures")` to define exonic counting bins. First, we download a annotation file from *ENSEMBL* in *GTF* format, create an *txdb* object and then run the `exonicParts()` function.

```{r, warning=FALSE}

library(GenomicFeatures)  # for the exonicParts() function
library(txdbmaker)        # for the makeTxDbFromGFF() function
download.file(
    "https://ftp.ensembl.org/pub/release-62/gtf/drosophila_melanogaster/Drosophila_melanogaster.BDGP5.25.62.gtf.gz",
    destfile="Drosophila_melanogaster.BDGP5.25.62.gtf.gz")
txdb = makeTxDbFromGFF("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

file.remove("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

flattenedAnnotation = exonicParts( txdb, linked.to.single.gene.only=TRUE )
names(flattenedAnnotation) =
    sprintf("%s:E%0.3d", flattenedAnnotation$gene_id, flattenedAnnotation$exonic_part)

```

Note that the code above first converted the annotation *GTF* file into a *transcriptDb* object, 
but transcripts could also start from *txdb* objects available as Bioconductor packages or from
objects available through `r BiocStyle::Biocpkg("AnnotationHub")`. 

## Counting reads

For each BAM file, we count the number of reads that overlap with each of the exon counting bin defined in the 
previous section. The read counting step is performed by the **summarizeOverlaps()** function of the 
`r BiocStyle::Biocpkg("GenomicAlignments")` package. To demonstrate this step, we will use a subset of the 
pasilla BAM files that are available through the `r BiocStyle::Biocpkg("pasillaBamSubset")` package.

```{r}

library(pasillaBamSubset)
library(Rsamtools)
library(GenomicAlignments)
library(GenomeInfoDb)
bamFiles = c( untreated1_chr4(), untreated3_chr4() )
bamFiles = BamFileList( bamFiles )
seqlevelsStyle(flattenedAnnotation) = "UCSC"
se = summarizeOverlaps(
    flattenedAnnotation, BamFileList(bamFiles), singleEnd=FALSE,
    fragments=TRUE, ignore.strand=TRUE )

```

All `singleEnd=`, `fragments=`, `ignore.strand=` and `strandMode=` are important parameters 
to tune and are dataset specific. Two alternative ways to do the read counting process are
the **HTSeq** python scripts provided with `r BiocStyle::Biocpkg("DEXSeq")` and the 
`r BiocStyle::Biocpkg("Rsubread")` package. Both of these approaches are described in 
the appendix section of this vignette. 

## Building a _**DEXSeqDataSet**_

We can use the function `DEXSeqDataSetFromSE()` to build an _**DEXSeqDataSet**_ object. In a
well-designed experiment, we would specify the conditions of the experiment in the `colData()`
slot of the object. Since we have only two samples in our example, we just exemplify this step 
by specify the sample names as conditions.

```{r buildExonCountSet}
library(DEXSeq)
colData(se)$condition = factor(c("untreated1", "untreated3"))
colData(se)$libType = factor(c("single-end", "paired-end"))
dxd = DEXSeqDataSetFromSE( se, design= ~ sample + exon + condition:exon )
```

# Standard analysis workflow

## Loading and inspecting the example data

To demonstrate the rest of the work flow we will use a _**DEXSeqDataSet**_ that contains a subset of the data
with only a few genes. This example object is available through the `r BiocStyle::Biocpkg("pasilla")` package.

```{r startVignette}
data(pasillaDEXSeqDataSet, package="pasilla")
```

The _**DEXSeqDataSet**_ class is derived from the _**DESeqDataSet**_. As such, it contains the usual accessor functions for the column data, row data, and some specific ones. The core data in an _**DEXSeqDataSet**_ object are the counts per exon. Each row of the _**DEXSeqDataSet**_ contains in each column the count data from a given exon ('this') as well as the count data from the sum of the other exons belonging to the same gene ('others'). This annotation, as well as all the information regarding each column of the _**DEXSeqDataSet**_, is specified in the `colData()`.

```{r}
colData(dxd)
```

We can access the first 5 rows from the count data by doing, 

```{r}
head( counts(dxd), 5 )
```

Note that the number of columns is 14, the first seven (we have seven samples)  corresponding to the number of reads mapping to out exonic regions and the last seven correspond to the sum of the counts mapping to the rest of the exons from the same gene on each sample.

```{r}
split( seq_len(ncol(dxd)), colData(dxd)$exon )
```

We can also access only the first five rows from the count belonging to the exonic regions ('this') (without showing the sum of counts from the rest of the exons from the same gene) by doing,

```{r}
head( featureCounts(dxd), 5 )
```

In both cases, the rows are labelled with gene IDs (here Flybase IDs), followed by a colon and the counting bin number. Since a counting bin corresponds to an exon or part of an exon, this ID is called the **feature ID** or **exon ID** within `r BiocStyle::Biocpkg("DEXSeq")`. The table content indicates the number of reads that have been mapped to each counting bin in the respective sample.

To see details on the counting bins, we also print the first 3 lines of the feature data annotation:

```{r}
head( rowRanges(dxd), 3 )
```

So far, this table contains information on the annotation data, such as gene and exon IDs, genomic coordinates of the exon, and the list of transcripts that contain an exon. 

The accessor function `annotationData()` shows the design table with the sample annotation (which was passed as the second argument to `DEXSeqDataSetFromHTSeq()`):

```{r}
sampleAnnotation( dxd )
```

In the following sections, we will update the object by calling a number of analysis functions, always using the idiom `dxd = someFunction( dxd )`, which takes the _**dxd**_ object, fills in the results of the performed computation and writes the returned
and updated object back into the variable _**dxd**_.

## Normalisation

`r BiocStyle::Biocpkg("DEXSeq")` uses the same method as `r BiocStyle::Biocpkg("DESeq")` and `r BiocStyle::Biocpkg("DESeq2")`, which is provided in the function `estimateSizeFactors()`.

```{r}
dxd = estimateSizeFactors( dxd )
```

## Dispersion estimation

To test for differential exon usage, we need to estimate the variability of the data. This is necessary to be able to distinguish technical and biological variation (noise) from real effects on exon usage due to the different conditions. The information on the strength of the noise is inferred from the biological replicates in the data set and characterized by the so-called *dispersion*. In RNA-Seq experiments the number of replicates is typically too small to reliably estimate variance or dispersion parameters individually exon by exon, and therefore, variance information is shared across exons and genes, in an intensity-dependent manner.

In this section, we discuss simple one-way designs: In this setting, samples with the same experimental condition, as indicated in the `condition` factor of the design table (see above), are considered as replicates -- and therefore, the design table needs to contain a column with the name `condition`. In Section \@ref(complexdesigns), we discuss how to treat more complicated experimental designs which are not accommodated by a single condition factor.

To estimate the dispersion estimates, `r BiocStyle::Biocpkg("DEXSeq")` uses the approach of the package `r BiocStyle::Biocpkg("DESeq2")`. Internally, the functions from `r BiocStyle::Biocpkg("DEXSeq")` are called, adapting the parameters of the functions for the specific case of the `r BiocStyle::Biocpkg("DEXSeq")` model. Briefly, per-exon dispersions are calculated using a Cox-Reid adjusted profile likelihood estimation, then a dispersion-mean relation is fitted to this individual dispersion values andfinally, the fitted values are taken as a prior in order to shrink the per-exon estimates towards the fitted
values. See the `r BiocStyle::Biocpkg("DESeq2")` paper for the rational behind the shrinkage approach (@deseq2).

```{r}
dxd = estimateDispersions( dxd )
```

As a shrinkage diagnostic, the _**DEXSeqDataSet**_ inherits the method `plotDispEsts()` that plots the per-exon dispersion estimates versus the mean normalised count, the resulting fitted valuesand the *a posteriori* (shrinked) dispersion estimates (Figure \@ref(fig:fitdiagnostics)).

```{r fitdiagnostics, fig.small=TRUE, fig.cap="Fit Diagnostics. The initial per-exon dispersion estimates (shown by black points), the fitted mean-dispersion values function (red line), and the shrinked values in blue"}
plotDispEsts( dxd )
```

# Testing for differential exon usage

Having the dispersion estimates and the size factors, we can now test for differential exon usage.
For each gene, `r BiocStyle::Biocpkg("DEXSeq")` fits a generalized linear model with the formula
`~sample + exon + condition:exon` and compare it to the smaller model (the null model) `~ sample + exon`. 

In these formulae (which use the standard notation for linear model formulae in **R**; consult a text book
on **R** if you are unfamiliar with it), `sample` is a factor with different levels for each sample,
`condition` is the factor of experimental conditions that we defined when constructing the
_**DEXSeqDataSet**_ object at the beginning of the analysis, and `exon` is a factor with
two levels, `this` and `others`, that were specified when we generated our _**DEXSeqDataSet**_
object. The two models described by these formulae are fit for each counting bin, where the data 
supplied to the fit comprise **two** read count values for each sample, corresponding to the 
two levels of the `exon` factor: the number of reads mapping to the bin in question 
(level `this`), and the sum of the read counts from all other bins of the same gene 
(level `others`). Note that this approach differs from the approach described in 
the paper (@DEXSeqPaper) and used in older versions of `r BiocStyle::Biocpkg("DEXSeq")`; 
see Appendix \@ref(methodChanges) 
for further discussion.

We have an interaction term `condition:exon`, but denote no main effect for `condition`.
Note, however, that all observations from the same sample are also from the same condition, i.e., the
`condition` main effects are absorbed in the `sample` main effects, because the `sample` 
factor is nested within the `condition` factor. 

The deviances of both fits are compared using a $\chi^2$-distribution, providing a $p$ value. Based on this $p$-value, we can decide whether the null model is sufficient to explain the data, or whether it may be rejected in favour of the alternative model, which contains an interaction coefficient for `condition:exon`. The latter means that the fraction of the gene's reads that fall onto the exon under the test differs significantly between the experimental conditions.

The function `testForDEU()` performs these tests for each exon in each gene.

```{r testForDEU1}
dxd = testForDEU( dxd )
```

The resulting _**DEXSeqDataSet**_ object contains slots with information
regarding the test.  

For some uses, we may also want to estimate relative exon usage fold changes. To this end, 
we call `estimateExonFoldChanges()`. Exon usage fold changes are 
calculated based on the coefficients of a GLM fit that uses the formula

`count ~ condition + exon + condition:exon`

where `condition` can be replaced with any of the column names of `colData()` (see man pages for details). The resulting coefficients allow the estimation of changes on the usage of exons across different conditions. Note that the differences on exon usage are distinguished from gene expression differences across conditions. For example, consider the case of a gene that is differentially expressed between conditions and has one exon that is differentially used between conditions. From the coefficients of the fitted model, it is possible to distinguish overall gene expression effects, that alter the counts from all the exons, from exon usage effects, that are captured by the interaction term `condition:exon` and that affect the each exon individually.

```{r estFC}
dxd = estimateExonFoldChanges( dxd, fitExpToVar="condition")
```

So far in the pipeline, the intermediate and final results have been stored in the meta data of a _**DEXSeqDataSet**_ object, they can be accessed using the function `mcols()`. In order to summarize the results without showing the values from intermediate steps, we call the function `DEXSeqResults()`. The result is a _**DEXSeqResults**_ object, which is a subclass of a _**DataFrame**_ object. 

```{r results1}
dxr1 = DEXSeqResults( dxd )
dxr1
```

The description of each of the column of the object _**DEXSeqResults**_ can be found in the metadata columns.

```{r results2}
mcols(dxr1)$description
```

From this object, we can ask how many exonic regions are significant with a false discovery rate of 10%:

```{r tallyExons}
table ( dxr1$padj < 0.1 )
```

We may also ask how many genes are affected

```{r tallyGenes}
table ( tapply( dxr1$padj < 0.1, dxr1$groupID, any ) )
```

Remember that our example data set contains only a selection of genes. We have chosen these to contain interesting cases; so the fact that such a large fraction of genes is significantly affected here is not typical.

To see how the power to detect differential exon usage depends on the number of reads that
map to an exon, a so-called MA plot is useful, which plots the logarithm of fold change versus average
normalized count per exon and marks by red colour the exons which are considered significant; here,
the exons with an adjusted $p$ values of less than $0.1$ (Figure \@ref(fig:MvsA). There is of course nothing special about the number $0.1$, and you can specify other thresholds in the call to `plotMA()`.

```{r MvsA, fig.small=TRUE, fig.cap="MA plot. Mean expression versus $log{2}$ fold change plot. Significant hits at an $FDR=0.1$ are coloured in red."}
plotMA( dxr1, cex=0.8 )
```

# Additional technical or experimental variables {#complexdesigns}

In the previous section we performed a simple analysis of differential exon usage, in which each sample was assigned
to one of two experimental conditions. If your experiment is of this type, you can use the work flow shown above. All
you have to make sure is that you indicate which sample belongs to which experimental condition when you construct
the _**DEXSeqDataSet**_ object. Do so by means of a column called `condition` in the sample table.

If you have a more complex experimental design, you can provide different or additional columns in the sample table. You
then have to indicate the design by providing explicit formulae for the test.

In the `r BiocStyle::Biocpkg("pasilla")` dataset, some samples were sequenced in single-end and others in paired-end mode. Possibly,
this influenced counts and should hence be accounted for. We therefore use this as an example for a complex design.

When we constructed the _**DEXSeqDataSet**_, we provided in the `colData()` an additional column called `type`, which 
has been stored in the object:

```{r design}
sampleAnnotation(dxd)
```

We specify two design formulae, which indicate that the `type` _**factor**_ should be treated as a blocking factor:

```{r formulas2}
formulaFullModel    =  ~ sample + exon + type:exon + condition:exon
formulaReducedModel =  ~ sample + exon + type:exon 
```

Compare these formulae with the default formulae given in previous sections. We have added, in both the full model and the reduced model, the term `type:exon`. Therefore, any dependence of exon
usage on library type will be absorbed by this term and accounted for equally in the full
and a reduced model, and the likelihood ratio test comparing them will only detect differences
in exon usage that can be attributed to `condition`, independent of `type`.

Next, we estimate the dispersions. This time, we need to inform the `estimateDispersions()` function
about our design by providing the full model's formula, which should be used instead of the default 
formula.

```{r estDisps_again}
dxd = estimateDispersions( dxd, formula = formulaFullModel )
```

The test function now needs to be informed about both formulae

```{r test_again}
dxd = testForDEU( dxd, 
	reducedModel = formulaReducedModel, 
        fullModel = formulaFullModel )
```

Finally, we get a summary table, as before.

```{r res_again}
dxr2 = DEXSeqResults( dxd )
```

How many significant DEU cases have we got this time?

```{r table2}
table( dxr2$padj < 0.1 )
```

We can now compare with the previous result:

```{r table3}
table( before = dxr1$padj < 0.1, now = dxr2$padj < 0.1 )
```

Accounting for the library type has allowed us to find six more hits, which confirms that accounting for the covariate improves power.

# Visualization

The `plotDEXSeq()` provides a means  to visualize the results of an analysis.

```{r plot1, fig.height=8, fig.width=12, fig.cap="Fitted expression. The plot represents the expression estimates from a call to `testForDEU()`. Shown in red is the exon that showed significant differential exon usage."}
plotDEXSeq( dxr2, "FBgn0010909", legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

```{r checkClaim}
wh = (dxr2$groupID=="FBgn0010909")
stopifnot(sum(dxr2$padj[wh] < formals(plotDEXSeq)$FDR)==1)
```

The result is shown in Figure \@ref(fig:plot1). This plot shows the fitted expression values of each of the exons of gene *FBgn0010909*, for each of the two conditions, treated and untreated. The function `plotDEXSeq()` expects at least two arguments, the _**DEXSeqDataSet**_ object and the gene ID. The option `legend=TRUE` causes a legend to be included. The three remaining arguments in the code chunk above are ordinary plotting parameters which are simply handed over to _**R**_ standard
plotting functions. They are not strictly needed and included here to improve appearance of
the plot. See the help page for `par()` for details.

Optionally, one can also visualize the transcript models (Figure \@ref(fig:plot2)), which can be
useful for putting differential exon usage results into the context of isoform regulation.

```{r plot2, fig.height=8, fig.width=12, fig.cap="Transcripts. As in the previous plots, but including the annotated transcript models"}
plotDEXSeq( dxr2, "FBgn0010909", displayTranscripts=TRUE, legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

Other useful options are to look at the count values from the individual samples, rather than at the
model effect estimates. For this display (option `norCounts=TRUE`), the counts are normalized by 
dividing them by the size factors (Figure \@ref(fig:plot3))

```{r plot3, fig.height=8, fig.width=12, fig.cap="Normalized counts. As in the previous plots, with normalized count values of each exon in each of the samples."}
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, norCounts=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

As explained in Section \@ref(overview), `r BiocStyle::Biocpkg("DEXSeq")` is designed to find
changes in relative exon usage, i.e., changes in the expression of individual exons that are
not simply the consequence of overall up- or down-regulation of the gene. To visualize such changes, it is sometimes advantageous to remove overall changes in expression from the
plots. Use the option `splicing=TRUE` for this purpose.

```{r plot4, fig.height=8, fig.width=12, fig.cap="Fitted splicing. As in the previous plots, but after subtraction of overall changes in gene expression."}
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, splicing=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

To generate an easily browsable, detailed overview over all analysis results, the package provides an HTML report generator, implemented in the function `DEXSeqHTML()`. This function uses the package `r BiocStyle::CRANpkg("hwriter")` [@hwriter] to create a result table with links to plots for the
significant results, allowing a more detailed exploration of the results. 

```{r DEXSeqHTML, cache=TRUE, eval=FALSE}
DEXSeqHTML( dxr2, FDR=0.1, color=c("#FF000080", "#0000FF80") )
```

# Parallelization and large number of samples

`r BiocStyle::Biocpkg("DEXSeq")` analyses can be computationally heavy, especially with data sets that 
comprise a large number of samples, or with genomes containing genes with large numbers of exons.  While 
some steps of the analysis work on the whole data set, the computational load can be parallelized for 
some steps. We use the package `r BiocStyle::Biocpkg("BiocParallel")`, and implemented the `BPPARAM=` 
parameter of the functions `estimateDispersions()`, `testForDEU()` and `estimateExonFoldChanges()`:

```{r para1,cache=TRUE,results='hide', eval=FALSE}
BPPARAM = BiocParallel::MulticoreParam(4)
dxd = estimateSizeFactors( dxd )
dxd = estimateDispersions( dxd, BPPARAM=BPPARAM)
dxd = testForDEU( dxd, BPPARAM=BPPARAM)
dxd = estimateExonFoldChanges(dxd, BPPARAM=BPPARAM)
```

For running analysis with a large number of samples (e.g. more than 20), we recommend configuring 
`BatchJobsParam()` classes in order to distribute the calculations across a computer cluster and 
significantly reduce running times. Users might also consider reducing the number of tests by 
filtering for lowly expressed isoforms or exonic regions with low counts. Appart from reducing 
running times, this filtering step also leads to more accurate results [@soneson2015]. 

Nevertheless, `r BiocStyle::Biocpkg("DEXSeq")` was designed to work with small sample numbers,
and users might have to consider alternatives for large numbers of samples such as the 
diffSplice function from `r BiocStyle::Biocpkg("limma")`, `r BiocStyle::Biocpkg("satuRn")` 
or non-parametric approaches for exon usage testing.

# Perform a standard differential exon usage analysis in one command

In the previous sections, we went through the analysis step by step. Once you are sufficiently 
confident about the work flow for your data, its invocation can be streamlined by the wrapper 
function `DEXseq()`, which runs the analysis shown above through a single function call. 

In the simplest case, construct the _**DEXSeqDataSet**_ as shown before, then run `DEXSeq()` 
passing the _**DEXSeqDataSet**_ as only argument, this function will output 
a _**DEXSeqResults**_ object.

```{r alldeu}
dxr = DEXSeq(dxd)
class(dxr)
```


# Appendix

## Controlling FDR at the gene level

`r BiocStyle::Biocpkg("DEXSeq")` returns one $p$-value for each exonic region. Using 
these p-values, a rejection threshold is established according to a multiple testing correction method. Consider a setting with $M$ exonic regions, where  $p_{k}$ is the $p$-value for the $k$ exonic region and 
$\theta \in {p_{1}, ..., p_{M}} \subset [0, 1]$ is a rejection threshold, leading to 
$V=\left|\left\{k: p_k\le\theta\right\}\right|$ rejections (i.e. number of exonic regions being 
DEU). The Benjamini-Hochberg method establishes that FDR is controlled at the level $q^{*}$ where

$q^* = \frac{M \theta}{\left|\left\{k: p_k\le\theta\right\}\right|}$.

Note that $\theta$ in the denominator of this expression is the probability of rejecting a single 
hypothesis if it is true. $q^*$ allows to estimate the number of exons that are differentially
used at a specific FDR. 

However, sometimes it is informative to know what is the FDR at the gene level, i.e. knowing the
numer of genes with at least one differentially used exon while keeping control of FDR. For this 
scenario, let $p_{i,l}$ be the $p$-value from the DEU test for exon $l$ in gene $i$, let $n_i$ 
the number of exons in gene $i$, and $M$ the number of genes. A gene $i$ has at least one DEU 
exon if $\exists l$ such that $p_{i,l}\le\theta$. Then, FDR is controlled at the level

$\frac{\sum_{i=1}^{M} 1-(1-\theta)^{n_i}}{\left|\left\{i: \exists l:p_{il}\le\theta\right\}\right|}$.

The implementation of the formula above is provided by the function `perGeneQValue()`. For
the `r BiocStyle::Biocpkg("pasilla")` example, the code below calculates the number of genes (at a FDR of 10%) with at least one differentially used exon.

```{r pergeneqval}
numbOfGenes = sum( perGeneQValue(dxr) < 0.1)
numbOfGenes
```

## Preprocessing with python

### Preparing the annotation file with python

The preprocessing steps of the analysis are done using two Python scripts that we provide with `r BiocStyle::Biocpkg("DEXSeq")`.

You do not need to know how to use Python; however you have to install the Python package **HTSeq**, following the explanations given on the [**HTSeq** web page](http://www-huber.embl.de/users/anders/HTSeq/doc/install.html).

Once you have installed **HTSeq**, you can use the two Python scripts, *dexseq_prepare_annotation.py* and *dexseq_count.py*, that come with the `r BiocStyle::Biocpkg("DEXSeq")` package. If you have trouble finding them, start R and ask for the installation directory with:

```{r systemFile}
pythonScriptsDir = system.file( "python_scripts", package="DEXSeq" )
list.files(pythonScriptsDir)
system.file( "python_scripts", package="DEXSeq", mustWork=TRUE )
```

The displayed path should contain the two files. If it does not, try to re-install `r BiocStyle::Biocpkg("DEXSeq")` 
(as usual, with the function `BiocManager::install()`). An alternative work flow, which replaces the two Python-based steps with R based code, is also available in the Appendix section of this vignette.

The Python scripts expect a GTF file with gene models for your species. We have tested our tools chiefly with GTF files from Ensembl and hence recommend to use these, as files from other providers sometimes do not adhere fully to the GTF standard and cause the preprocessing to fail. Ensembl GTF files can be found in the "FTP Download" sections of the Ensembl web sites (i.e., Ensembl, EnsemblPlants, EnsemblFungi, etc.). Make sure that your GTF file uses a coordinate system that matches the reference genome that you have used for aligning your reads. The safest way to ensure this is to download the reference genome from Ensembl, too. If you cannot use an Ensembl GTF file, see Appendix \@ref(requirements-on-gtf-files) for advice on converting GFF files from other sources to make them suitable as input for the *dexseq_prepare_annotation.py* script.

In a GTF file, many exons appear multiple times, once for each transcript that contains them. We need to "collapse" this information to define *exon counting bins*, i.e., a list of intervals, each corresponding to one exon or part of an exon. Counting bins for parts of exons arise when an exonic region appears with different boundaries in different transcripts. See Figure 1 of the `r BiocStyle::Biocpkg("DEXSeq")` paper (@DEXSeqPaper) for an illustration. The Python script *dexseq_prepare_annotation.py*
takes an Ensembl GTF file and translates it into a GFF file with collapsed exon counting bins.

Make sure that your current working directory contains the GTF file and call the script (from the command line shell, not from within R) with

```
python /path/to/library/DEXSeq/python_scripts/dexseq_prepare_annotation.py Drosophila_melanogaster.BDGP5.72.gtf Dmel.BDGP5.25.62.DEXSeq.chr.gff
```

In this command, which should be entered as a single line, replace */path/to.../python_scripts* with the correct path to the Python scripts, which you have found with the call to `system.file()` shown above. *Drosophila_melanogaster.BDGP5.72.gtf* is the Ensembl GTF file (here the one for fruit fly, already de-compressed) and *Dmel.BDGP5.25.62.DEXSeq.chr.gff* is the name of the output file.

In the process of forming the counting bins, the script might come across overlapping genes. If two genes on the same strand are found with an exon of the first gene overlapping with an exon of the second gene, the script's default behaviour is to combine the genes into a single "aggregate gene" which is subsequently referred to with the IDs of the individual genes, joined by a plus ('+') sign. If you do not like this behaviour, you can disable aggregation with the option `-r no`. Without aggregation, exons that overlap with other exons from different genes are simply skipped.

### Counting reads with python

Read counting is done with the script *python_count.py*:

```
python /path/to/library/DEXSeq/python_scripts/dexseq_count.py Dmel.BDGP5.25.62.DEXSeq.chr.gff untreated1.sam untreated1fb.txt
```

This command (again, to be entered as a single line) expects two files in the current working directory, namely the GFF file produced in the previous step (here *Dmel.BDGP5.25.62.DEXSeq.chr.gff*) and a SAM file with the aligned reads from a sample (here the file *untreated1.sam* with the aligned reads from the first control sample). The command generates an output file, here called *untreated1fb.txt*, with one line for each exon counting bin defined in the flattened GFF file. The lines contain the exon counting bin IDs (which are composed of gene IDs and exon bin numbers), followed by a integer number which indicates the number of reads that were aligned such that they overlap with the counting bin.

Use the script multiple times to produce a count file from each of your SAM files.

There are a number of crucial points to pay attention to when using the *python_count.py* script:

**Paired-end data: ** If your data is from a paired-end sequencing run, you need to add the option `-p yes` to 
the command to call the script. (As usual, options have to be placed before the file names, surrounded by spaces.) In addition, the SAM file needs to be sorted, either by read name or by position. Most aligners produce sorted SAM files; if your SAM file is not sorted, use `samtools sort -n` to sort by read name (or `samtools sort`) to sort by position. (See @DEProt, if you need further explanations on how to sort SAM files.) Use the option `-r pos` or `-r name` to indicate whether your paired-end data is sorted by alignment position or by read name.

**Strandedness:** By default, the counting script assumes your library to be *strand-specific*, i.e., reads are aligned to the same strand as the gene they originate from. If you have used a library preparation protocol that does not preserve strand information (i.e., reads from a given gene can appear equally likely on either strand), you need to inform the script by specifying the option `-s no`. If your library preparation protocol reverses the strand (i.e., reads appear on the strand opposite to their gene of origin), use `-s reverse`. In case of paired-end data, the default (`-s yes`) means that the read from the first sequence pass is on the same strand as the gene and the read from the second pass on the opposite strand ("forward-reverse" or "fr" order in the parlance of the Bowtie/TopHat manual) and the options `-s reverse` specifies the opposite case.

**SAM and BAM files:** By default, the script expects its input to be in plain-text SAM format.
However, it can also read BAM files, i.e., files in the the compressed binary variant of the SAM format.
If you wish to do so, use the option `-f bam`. This works only if you have installed the Python
package [*pysam*](https://code.google.com/p/pysam/).

**Alignment quality:** The scripts takes a further option, `-a` to specify the minimum alignment quality (as given in the fifth column of the SAM file). All reads with a lower quality than specified (with default `-a 10`) are skipped.

**Help pages:** Calling either script without arguments displays a help page with an overview of all options and arguments.

### Reading the data from the python ouputs into R

The remainder of the analysis is now done in **R**. We will use the output of the python scripts for the *pasilla* experiment, that can be found in the package `r BiocStyle::Rpackage("pasilla")`. Open an **R** session and type:

```{r loadDEXSeq}
inDir = system.file("extdata", package="pasilla")
countFiles = list.files(inDir, pattern="fb.txt$", full.names=TRUE)
basename(countFiles)
flattenedFile = list.files(inDir, pattern="gff$", full.names=TRUE)
basename(flattenedFile)
```

Now, we need to prepare a sample table. This table should contain one row for each library, and columns for all relevant information such as name of the file with the read counts, experimental conditions, technical information and further covariates. To keep this vignette simple, we construct the table on the fly.

```{r sampleTable}
sampleTable = data.frame(
   row.names = c( "treated1", "treated2", "treated3", 
      "untreated1", "untreated2", "untreated3", "untreated4" ),
   condition = c("knockdown", "knockdown", "knockdown",  
      "control", "control", "control", "control" ),
   libType = c( "single-end", "paired-end", "paired-end", 
      "single-end", "single-end", "paired-end", "paired-end" ) )
```

While this is a simple way to prepare the table, it may be less error-prone and more prudent to used an existing table that had already been prepared when the experiments were done, save it in CSV format and use the **R** function `read.csv()` to load it.

In any case, it is vital to check the table carefully for correctness.

```{r check}
sampleTable
```

Our table contains the sample names as row names and the two covariates that vary between samples: first the experimental condition (factor _**condition**_ with levels **control** and **treatment**) and the library type (factor **libType**), which we included because the samples in this particular experiment were sequenced partly in single-end runs and partly in paired-end runs. 
For now, we will ignore this latter piece of information, and postpone the discussion of how to include such additional covariates to Section \@ref(complexdesigns). If you have only a single covariate and want to perform a simple analysis, the column with this covariate should be named **condition**.

Now, we construct an _**DEXSeqDataSet**_ object from this data. This object holds all the input data and will be passed along the stages of the subsequent analysis. We construct the object with the function `DEXSeqDataSetFromHTSeq()`, 
as follows:

```{r message=FALSE}
library( "DEXSeq" )

dxd = DEXSeqDataSetFromHTSeq(
   countFiles,
   sampleData=sampleTable,
   design= ~ sample + exon + condition:exon,
   flattenedfile=flattenedFile )
```

The function takes four arguments. First, a vector with names of count files, i.e., of files that have been generated with the *dexseq_count.py* script. The function will read these files and arrange the count data in a matrix, which is stored in the _**DEXSeqDataSet**_ object `dxd`. The second argument is our sample table, with one row for each of the files listed in the first argument. This information is simply stored as is in the object's meta-data slot (see below). The third argument is a formula of the form `~ sample + exon + condition:exon` that specifies the contrast with of a variable from the sample table columns and the **'exon'** variable. Using this formula, we are interested in differences in exon usage due to the "condition" variable changes. Later in this vignette, we will how to add additional variables for complex designs.  The fourth argument is a file name, now of the flattened GFF file that was generated with *dexseq_prepare_annotation.py* and used as input to *dexseq_count.py* when creating the count file.

## Preprocessing using featureCounts

As an alternative to *HTSeq*, one could use the function `featureCounts()` to count the number of reads overlapping with each exonic region. The output files from `featureCounts()` can then be used as input to DEXSeq. In [this](https://github.com/vivekbhr/Subread_to_DEXSeq) *Github* repository, Vivek Bhardwaj provides scripts and documentation that integrate `featureCounts()` with `r BiocStyle::Biocpkg("DEXSeq")`. He also provides a function to import the count files into _**R**_ and generate a _**DEXSeqDataSet**_ object. 

## Further accessors

The function `geneIDs()` returns the gene ID column of the feature data as a character
vector, and the function `exonIDs()` return the exon ID column as a _**factor**_.

```{r acc}
head( geneIDs(dxd) )
head( exonIDs(dxd) )
```

These functions are useful for subsetting an _**DEXSeqDataSet**_ object.

## Overlap operations

The methods `subsetByOverlaps()` and `findOverlaps()` have been implemented for the _**DEXSeqResults**_ object, the `query=` argument must be a _**DEXSeqResults**_ object. 

```{r grmethods}
interestingRegion = GRanges( "chr2L", 
   IRanges(start=3872658, end=3875302) )
subsetByOverlaps( x=dxr, ranges=interestingRegion )
findOverlaps( query=dxr, subject=interestingRegion )
```

This functions could be useful for further downstream analysis.

## Methodological changes since publication of the paper {#methodChanges}

In our paper [@DEXSeqPaper], we suggested to fit for each exon a model that includes separately 
the counts for all the gene's exons. However, this turned out to be computationally inefficient
for genes with many exons, because the many exons required large model matrices, which are computationally expensive to deal with. We have therefore modified the approach: when fitting
a model for an exon, we now sum up the counts from all the other exon and use only the total, rather than the individual counts in the model. Now, computation time per exon is independent of the number of other exons in the gene, which improved `r BiocStyle::Biocpkg("DEXSeq")`'s scalability. While the $p$ values returned by the two approaches are not exactly equal, the differences were very minor in the tests that we performed.

Deviating from the paper's notation, we now use the index $i$ to indicate a specific counting bin, 
with $i$ running through all counting bins of all genes. The samples are indexed with $j$, as in the paper. We write $K_{ij0}$ for the count or reads mapped to counting bin $i$ in sample $j$ and $K_{ij1}$ for the sum of the read counts from all other counting bins in the same gene. Hence, when we write $K_{ijl}$, the third index $l$ indicates whether we mean the read count for bin $i$ ($l=0$) or the sum of counts for all other bins of the same gene ($l=1$). As before, we fit a GLM of the negative binomial (NB) family
$ K_{ijl} \sim \operatorname{NB}(\text{mean}=s_j\mu_{ijl},\text{dispersion}=\alpha_i)$ and we write
$\log_2 \mu_{ijl} = \beta^\text{S}_{ij} + l \beta^\text{E}_{i} + \beta^\text{EC}_{i\rho_j}$.

This model is fit separately for each counting bin $i$. The coefficient $\beta^\text{S}_{ij}$ accounts for the sample-specific contribution (factor `sample`), the term $\beta^\text{E}_{i}$
is only included if $l=1$ and hence estimates the logarithm of the ratio $K_{ij1}/K_{ij0}$ between the counts for all other exons and the counts for the tested exon. As this coefficient is estimated from data from all samples, it can be considered as a measure of "average exon usage". In the R model formula, it is represented by the term `exon` with its two levels `this` ($l=0$) and `others` ($l=1$). Finally, the last term, $\beta^\text{EC}_{i,\rho_j}$, captures the interaction `condition:exon`, i.e., the change in exon usage if sample $j$ is from experimental condition group $\rho(j)$. Here, the first condition, $\rho=0$, is absorbed in the sample coefficients, i.e., $\beta^\text{EC}_{i0}$ is fixed to zero and does not appear in the model matrix.

For the dispersion estimation, one dispersion value $\alpha_i$ is estimated with Cox-Reid-adjusted maximum likelihood using the full model given above. A mean-variance relation is fitted using the individual dispersion values. Finally, the individual values are shrinked towards the fitted values. For more details about this shrinkage approach look at the `r BiocStyle::Biocpkg("DESeq2")` vignette and/or its manuscript [@deseq2]. For the likelihood ratio test, this full model 
is fit and compared with the fit of the reduced model, which lacks the interaction term $\beta^\text{EC}_{i\rho_j}$. As described in Section \@ref{complexdesigns}, alternative model formulae can be specified.

## Requirements on GTF files

In the initial preprocessing step described in Section \@ref{preparing-the-annotation}, the Python script *dexseq_prepare_annotation.py* is used to convert a GTF file with gene models into a GFF file with collapsed gene models. We recommend to use GTF files downloaded from Ensembl as input for this script, as files from other sources may deviate from the format expected by the script. Hence, if you need to use a GTF or GFF file from another source, you may need to convert it to the expected format. To help with this task, we here give details on how the *dexseq_prepare_annotation.py* script interprets a GFF file.

* The script only looks at `exon` lines, i.e., at lines which contain the term `exon` in the 
third ("type") column. All other lines are ignored.
* Of the data in these lines, the information about chromosome, start, end, and strand (1st, 4th, 5th, and 7th column) are used, and, from the last column, the attributes `gene_id` and `transcript_id`. The rest is ignored.
* The `gene_id` attribute is used to see which exons belong to the same gene. It must be called `gene_id` (and not `Parent` as in GFF3 files, or `GeneID` as in some older GFF files), and it must give the same identifier to all exons from the same gene, even if they are from different transcripts of this gene. (This last requirement is not met
by GTF files generated by the Table Browser function of the UCSC Genome Browser.)
* The `transcript_id` attribute is used to build the *transcripts* attribute in the flattened GFF file, which indicates which transcripts contain the described counting bin. This information is needed only to draw the transcript model at the bottom of the plots when the `displayTranscripts=` option to `plotDEXSeq()` is used.

Therefore, converting a GFF file to make it suitable as input to *dexseq_prepare_annotation.py* amounts to making sure that the exon lines have type `exon` and that the atributes giving gene ID (or gene symbol) and transcript ID are called `gene_id` and `transcript_id`, with this exact spelling. Remember to also take care that the chromosome names match those in your SAM files, and that the coordinates refer to the reference assembly that you used when aligning your reads.

# Session Information

The session information records the versions of all the packages used in the generation of the present document.

```{r sessionInfo}
sessionInfo()
```

# References

---
title: "Inferring differential exon usage in RNA-Seq data with the DEXSeq package"
author:
- name: Alejandro Reyes
  affiliation: Dana-Farber Cancer Institute, Boston, USA
- name: Simon Anders
  affiliation: Heidelberg University, Heidelberg, USA
- name: Wolfgang Huber
  affiliation: European Molecular Biology Laboratory, Heidelberg, Germany
output:
  BiocStyle::html_document:
    toc_float: true
package: DEXSeq
bibliography: DEXSeq.bib
abstract: |
  The Bioconductor package `r BiocStyle::Biocpkg("DEXSeq")` implements a method 
  to test for
  differential exon usage in comparative RNA-Seq experiments. By *differential 
  exon usage* (DEU), we mean changes in the relative usage of exons caused by the
  experimental condition. The relative usage of an exon is defined as
  $\frac{\text{number of transcripts from the gene that contain this exon}}
  {\text{number of all transcripts from the gene}}$. The statistical method used by
  `r BiocStyle::Biocpkg("DEXSeq")` was introduced in our 
  paper [@DEXSeqPaper]. The basic concept 
  can be summarized as follows. For each exon (or part of an exon) and each 
  sample, we count how many reads map to this exon and how many reads map to 
  any of the other exons of the same gene. We consider the ratio of these two 
  counts, and how it changes across conditions, to infer changes in the relative 
  exon usage. In the case of an inner exon, a change in relative exon usage 
  is typically due to a change in the rate with which this exon is spliced 
  into transcripts (alternative splicing). Note, however, that DEU is a more
  general concept than alternative splicing, since it also includes 
  changes in the usage of alternative transcript start sites
  and polyadenylation sites, which can cause differential usage of 
  exons at the 5' and 3' boundary of transcripts.
  DEXSeq package version: `r packageVersion("DEXSeq")`
vignette: |
  %\VignetteIndexEntry{Inferring differential exon usage in RNA-Seq data with the DEXSeq package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r echo=FALSE, include=FALSE}
knitr::opts_chunk$set(tidy = FALSE,
                      cache = TRUE,
                      dev = "png",
                      message = FALSE,
                      error = FALSE,
                      warning = TRUE)
```

# Overview

Similar as with differential gene expression, we need to make sure that observed differences of exon usage values between conditions are statistically significant, i.e., are sufficiently unlikely to be just due to random fluctuations such as those seen even between samples from the same condition, i.e., between replicates. To this end, `r BiocStyle::Biocpkg("DEXSeq")` assesses the strength of these fluctuations (quantified by the so-called *dispersion*) by comparing replicates before comparing the averages between the sample groups.

The preceding description is somewhat simplified (and perhaps over-simplified), and we recommend that users consult the publication for a more complete description, as well as Appendix \@ref(methodChanges) of this vignette, which describes how the current implementation of `r BiocStyle::Biocpkg("DEXSeq")` differs from the original approach described in the paper. Nevertheless, two important aspects should be mentioned already here: First, `r BiocStyle::Biocpkg("DEXSeq")` does not actually work on the exon usage ratios, but on the counts in the numerator and denominator, to be able to make use of the information that is contained in the magnitude of count values. For example, 3000 reads versus 1000 reads is the same ratio as 3 reads versus 1 read, but the latter is a far less reliable estimate of the underlying true value, because of statistical sampling. Second, `r BiocStyle::Biocpkg("DEXSeq")` is not limited to simple two-group comparisons; rather, it uses so-called generalized linear models
(*GLMs*) to permit ANOVA-like analysis of potentially complex experimental designs.

# Preparations

## Example data

To demonstrate the use of `r BiocStyle::Biocpkg("DEXSeq")`, we use the *pasilla* dataset, an RNA-Seq dataset generated by [@Brooks2010]. They investigated the effect of siRNA knock-down of the gene *pasilla* on the transcriptome of fly *S2-DRSC* cells. The RNA-binding protein *pasilla* protein is thought to be involved in the regulation of splicing. (Its mammalian orthologs, NOVA1 and NOVA2, are well-studied examples of splicing factors.) Brooks et al. prepared seven cell cultures, treated three with siRNA to knock-down *pasilla* and left four as untreated controls, and performed RNA-Seq on all samples. They deposited the raw sequencing reads with the NCBI Gene Expression Omnibus (GEO) under the accession number [GSE18508](http://www.ncbi.nlm.nih.gov/projects/geo/query/acc.cgi?acc=GSE18508).

## Executability of the code

Usually, Bioconductor vignettes contain automatically executable code, i.e., you can follow the vignette by directly running the code shown, using functionality and data provided with the package. However, it would not be practical to include the voluminous raw data of the pasilla experiment here. Therefore, the code in the alignment section is not automatically executable. You may download the raw data yourself from GEO, as well as the required extra tools, and follow the work flow shown here and in the *pasilla* vignette [@pasillaVignette]. From Section \@ref(standard-analysis-workflow) on, code is directly executable, as usual. Therefore, we recommend that you just read this section, and try following our analysis in R only from the next section onwards. Once you work with your own data, you will want to come back and adapt the workflow shown here to your data.

## Alignment

The first step of the analysis is to align the reads to a reference genome. If you are using classic alignment methods (i.e. not pseudo-alignment approaches) it is important to align them to the genome, not to the transcriptome, and to use a splice-aware aligner (i.e., a short-read alignment tool that can deal with reads that span across introns) such as TopHat2 [@TopHat2], GSNAP [@GSNAP], or STAR [@STAR]. The explanation of the analysis workflow presented here starts with the aligned reads in the SAM format. If you are unfamiliar with the process of aligning reads to obtain SAM files, you can find a summary how we proceeded in preparing the pasilla data in the vignette for the `r BiocStyle::Biocpkg("pasilla")` data package [@pasillaVignette] and a more extensive explanation, using the same  data set, in our protocol article on differential expression calling [@DEProt].

## Preparing the annotation {#preparing-the-annotation-bioc}

In a transcript annotations such as a GTF file, many exons appear multiple times, once for each transcript that contains them. We need to "collapse" or "flatten" this information to define *exon counting bins*, i.e., a list of intervals, each corresponding to one exon or part of an exon. Counting bins for parts of exons arise when an exonic region appears with different boundaries in different transcripts. See Figure 1 of the `r BiocStyle::Biocpkg("DEXSeq")` paper (@DEXSeqPaper) for an illustration. 

Here we use the function `exonicParts()` of the package `r BiocStyle::Biocpkg("GenomicFeatures")` to define exonic counting bins. First, we download a annotation file from *ENSEMBL* in *GTF* format, create an *txdb* object and then run the `exonicParts()` function.

```{r, warning=FALSE}

library(GenomicFeatures)  # for the exonicParts() function
library(txdbmaker)        # for the makeTxDbFromGFF() function
download.file(
    "https://ftp.ensembl.org/pub/release-62/gtf/drosophila_melanogaster/Drosophila_melanogaster.BDGP5.25.62.gtf.gz",
    destfile="Drosophila_melanogaster.BDGP5.25.62.gtf.gz")
txdb = makeTxDbFromGFF("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

file.remove("Drosophila_melanogaster.BDGP5.25.62.gtf.gz")

flattenedAnnotation = exonicParts( txdb, linked.to.single.gene.only=TRUE )
names(flattenedAnnotation) =
    sprintf("%s:E%0.3d", flattenedAnnotation$gene_id, flattenedAnnotation$exonic_part)

```

Note that the code above first converted the annotation *GTF* file into a *transcriptDb* object, 
but transcripts could also start from *txdb* objects available as Bioconductor packages or from
objects available through `r BiocStyle::Biocpkg("AnnotationHub")`. 

## Counting reads

For each BAM file, we count the number of reads that overlap with each of the exon counting bin defined in the 
previous section. The read counting step is performed by the **summarizeOverlaps()** function of the 
`r BiocStyle::Biocpkg("GenomicAlignments")` package. To demonstrate this step, we will use a subset of the 
pasilla BAM files that are available through the `r BiocStyle::Biocpkg("pasillaBamSubset")` package.

```{r}

library(pasillaBamSubset)
library(Rsamtools)
library(GenomicAlignments)
bamFiles = c( untreated1_chr4(), untreated3_chr4() )
bamFiles = BamFileList( bamFiles )
seqlevelsStyle(flattenedAnnotation) = "UCSC"
se = summarizeOverlaps(
    flattenedAnnotation, BamFileList(bamFiles), singleEnd=FALSE,
    fragments=TRUE, ignore.strand=TRUE )

```

All `singleEnd=`, `fragments=`, `ignore.strand=` and `strandMode=` are important parameters 
to tune and are dataset specific. Two alternative ways to do the read counting process are
the **HTSeq** python scripts provided with `r BiocStyle::Biocpkg("DEXSeq")` and the 
`r BiocStyle::Biocpkg("Rsubread")` package. Both of these approaches are described in 
the appendix section of this vignette. 

## Building a _**DEXSeqDataSet**_

We can use the function `DEXSeqDataSetFromSE()` to build an _**DEXSeqDataSet**_ object. In a
well-designed experiment, we would specify the conditions of the experiment in the `colData()`
slot of the object. Since we have only two samples in our example, we just exemplify this step 
by specify the sample names as conditions.

```{r buildExonCountSet, eval=FALSE}
library(DEXSeq)
colData(se)$condition = factor(c("untreated1", "untreated3"))
colData(se)$libType = factor(c("single-end", "paired-end"))
dxd = DEXSeqDataSetFromSE( se, design= ~ sample + exon + condition:exon )
```

# Standard analysis workflow

## Loading and inspecting the example data

To demonstrate the rest of the work flow we will use a _**DEXSeqDataSet**_ that contains a subset of the data
with only a few genes. This example object is available through the `r BiocStyle::Biocpkg("pasilla")` package.

```{r startVignette}
data(pasillaDEXSeqDataSet, package="pasilla")
```

The _**DEXSeqDataSet**_ class is derived from the _**DESeqDataSet**_. As such, it contains the usual accessor functions for the column data, row data, and some specific ones. The core data in an _**DEXSeqDataSet**_ object are the counts per exon. Each row of the _**DEXSeqDataSet**_ contains in each column the count data from a given exon ('this') as well as the count data from the sum of the other exons belonging to the same gene ('others'). This annotation, as well as all the information regarding each column of the _**DEXSeqDataSet**_, is specified in the `colData()`.

```{r}
colData(dxd)
```

We can access the first 5 rows from the count data by doing, 

```{r}
head( counts(dxd), 5 )
```

Note that the number of columns is 14, the first seven (we have seven samples)  corresponding to the number of reads mapping to out exonic regions and the last seven correspond to the sum of the counts mapping to the rest of the exons from the same gene on each sample.

```{r}
split( seq_len(ncol(dxd)), colData(dxd)$exon )
```

We can also access only the first five rows from the count belonging to the exonic regions ('this') (without showing the sum of counts from the rest of the exons from the same gene) by doing,

```{r}
head( featureCounts(dxd), 5 )
```

In both cases, the rows are labelled with gene IDs (here Flybase IDs), followed by a colon and the counting bin number. Since a counting bin corresponds to an exon or part of an exon, this ID is called the **feature ID** or **exon ID** within `r BiocStyle::Biocpkg("DEXSeq")`. The table content indicates the number of reads that have been mapped to each counting bin in the respective sample.

To see details on the counting bins, we also print the first 3 lines of the feature data annotation:

```{r}
head( rowRanges(dxd), 3 )
```

So far, this table contains information on the annotation data, such as gene and exon IDs, genomic coordinates of the exon, and the list of transcripts that contain an exon. 

The accessor function `annotationData()` shows the design table with the sample annotation (which was passed as the second argument to `DEXSeqDataSetFromHTSeq()`):

```{r}
sampleAnnotation( dxd )
```

In the following sections, we will update the object by calling a number of analysis functions, always using the idiom `dxd = someFunction( dxd )`, which takes the _**dxd**_ object, fills in the results of the performed computation and writes the returned
and updated object back into the variable _**dxd**_.

## Normalisation

`r BiocStyle::Biocpkg("DEXSeq")` uses the same method as `r BiocStyle::Biocpkg("DESeq")` and `r BiocStyle::Biocpkg("DESeq2")`, which is provided in the function `estimateSizeFactors()`.

```{r}
dxd = estimateSizeFactors( dxd )
```

## Dispersion estimation

To test for differential exon usage, we need to estimate the variability of the data. This is necessary to be able to distinguish technical and biological variation (noise) from real effects on exon usage due to the different conditions. The information on the strength of the noise is inferred from the biological replicates in the data set and characterized by the so-called *dispersion*. In RNA-Seq experiments the number of replicates is typically too small to reliably estimate variance or dispersion parameters individually exon by exon, and therefore, variance information is shared across exons and genes, in an intensity-dependent manner.

In this section, we discuss simple one-way designs: In this setting, samples with the same experimental condition, as indicated in the `condition` factor of the design table (see above), are considered as replicates -- and therefore, the design table needs to contain a column with the name `condition`. In Section \@ref(complexdesigns), we discuss how to treat more complicated experimental designs which are not accommodated by a single condition factor.

To estimate the dispersion estimates, `r BiocStyle::Biocpkg("DEXSeq")` uses the approach of the package `r BiocStyle::Biocpkg("DESeq2")`. Internally, the functions from `r BiocStyle::Biocpkg("DEXSeq")` are called, adapting the parameters of the functions for the specific case of the `r BiocStyle::Biocpkg("DEXSeq")` model. Briefly, per-exon dispersions are calculated using a Cox-Reid adjusted profile likelihood estimation, then a dispersion-mean relation is fitted to this individual dispersion values andfinally, the fitted values are taken as a prior in order to shrink the per-exon estimates towards the fitted
values. See the `r BiocStyle::Biocpkg("DESeq2")` paper for the rational behind the shrinkage approach (@deseq2).

```{r}
dxd = estimateDispersions( dxd )
```

As a shrinkage diagnostic, the _**DEXSeqDataSet**_ inherits the method `plotDispEsts()` that plots the per-exon dispersion estimates versus the mean normalised count, the resulting fitted valuesand the *a posteriori* (shrinked) dispersion estimates (Figure \@ref(fig:fitdiagnostics)).

```{r fitdiagnostics, fig.small=TRUE, fig.cap="Fit Diagnostics. The initial per-exon dispersion estimates (shown by black points), the fitted mean-dispersion values function (red line), and the shrinked values in blue"}
plotDispEsts( dxd )
```

# Testing for differential exon usage

Having the dispersion estimates and the size factors, we can now test for differential exon usage.
For each gene, `r BiocStyle::Biocpkg("DEXSeq")` fits a generalized linear model with the formula
`~sample + exon + condition:exon` and compare it to the smaller model (the null model) `~ sample + exon`. 

In these formulae (which use the standard notation for linear model formulae in **R**; consult a text book
on **R** if you are unfamiliar with it), `sample` is a factor with different levels for each sample,
`condition` is the factor of experimental conditions that we defined when constructing the
_**DEXSeqDataSet**_ object at the beginning of the analysis, and `exon` is a factor with
two levels, `this` and `others`, that were specified when we generated our _**DEXSeqDataSet**_
object. The two models described by these formulae are fit for each counting bin, where the data 
supplied to the fit comprise **two** read count values for each sample, corresponding to the 
two levels of the `exon` factor: the number of reads mapping to the bin in question 
(level `this`), and the sum of the read counts from all other bins of the same gene 
(level `others`). Note that this approach differs from the approach described in 
the paper (@DEXSeqPaper) and used in older versions of `r BiocStyle::Biocpkg("DEXSeq")`; 
see Appendix \@ref(methodChanges) 
for further discussion.

We have an interaction term `condition:exon`, but denote no main effect for `condition`.
Note, however, that all observations from the same sample are also from the same condition, i.e., the
`condition` main effects are absorbed in the `sample` main effects, because the `sample` 
factor is nested within the `condition` factor. 

The deviances of both fits are compared using a $\chi^2$-distribution, providing a $p$ value. Based on this $p$-value, we can decide whether the null model is sufficient to explain the data, or whether it may be rejected in favour of the alternative model, which contains an interaction coefficient for `condition:exon`. The latter means that the fraction of the gene's reads that fall onto the exon under the test differs significantly between the experimental conditions.

The function `testForDEU()` performs these tests for each exon in each gene.

```{r testForDEU1}
dxd = testForDEU( dxd )
```

The resulting _**DEXSeqDataSet**_ object contains slots with information
regarding the test.  

For some uses, we may also want to estimate relative exon usage fold changes. To this end, 
we call `estimateExonFoldChanges()`. Exon usage fold changes are 
calculated based on the coefficients of a GLM fit that uses the formula

`count ~ condition + exon + condition:exon`

where `condition` can be replaced with any of the column names of `colData()` (see man pages for details). The resulting coefficients allow the estimation of changes on the usage of exons across different conditions. Note that the differences on exon usage are distinguished from gene expression differences across conditions. For example, consider the case of a gene that is differentially expressed between conditions and has one exon that is differentially used between conditions. From the coefficients of the fitted model, it is possible to distinguish overall gene expression effects, that alter the counts from all the exons, from exon usage effects, that are captured by the interaction term `condition:exon` and that affect the each exon individually.

```{r estFC}
dxd = estimateExonFoldChanges( dxd, fitExpToVar="condition")
```

So far in the pipeline, the intermediate and final results have been stored in the meta data of a _**DEXSeqDataSet**_ object, they can be accessed using the function `mcols()`. In order to summarize the results without showing the values from intermediate steps, we call the function `DEXSeqResults()`. The result is a _**DEXSeqResults**_ object, which is a subclass of a _**DataFrame**_ object. 

```{r results1}
dxr1 = DEXSeqResults( dxd )
dxr1
```

The description of each of the column of the object _**DEXSeqResults**_ can be found in the metadata columns.

```{r results2}
mcols(dxr1)$description
```

From this object, we can ask how many exonic regions are significant with a false discovery rate of 10%:

```{r tallyExons}
table ( dxr1$padj < 0.1 )
```

We may also ask how many genes are affected

```{r tallyGenes}
table ( tapply( dxr1$padj < 0.1, dxr1$groupID, any ) )
```

Remember that our example data set contains only a selection of genes. We have chosen these to contain interesting cases; so the fact that such a large fraction of genes is significantly affected here is not typical.

To see how the power to detect differential exon usage depends on the number of reads that
map to an exon, a so-called MA plot is useful, which plots the logarithm of fold change versus average
normalized count per exon and marks by red colour the exons which are considered significant; here,
the exons with an adjusted $p$ values of less than $0.1$ (Figure \@ref(fig:MvsA). There is of course nothing special about the number $0.1$, and you can specify other thresholds in the call to `plotMA()`.

```{r MvsA, fig.small=TRUE, fig.cap="MA plot. Mean expression versus $log{2}$ fold change plot. Significant hits at an $FDR=0.1$ are coloured in red."}
plotMA( dxr1, cex=0.8 )
```

# Additional technical or experimental variables {#complexdesigns}

In the previous section we performed a simple analysis of differential exon usage, in which each sample was assigned
to one of two experimental conditions. If your experiment is of this type, you can use the work flow shown above. All
you have to make sure is that you indicate which sample belongs to which experimental condition when you construct
the _**DEXSeqDataSet**_ object. Do so by means of a column called `condition` in the sample table.

If you have a more complex experimental design, you can provide different or additional columns in the sample table. You
then have to indicate the design by providing explicit formulae for the test.

In the `r BiocStyle::Biocpkg("pasilla")` dataset, some samples were sequenced in single-end and others in paired-end mode. Possibly,
this influenced counts and should hence be accounted for. We therefore use this as an example for a complex design.

When we constructed the _**DEXSeqDataSet**_, we provided in the `colData()` an additional column called `type`, which 
has been stored in the object:

```{r design}
sampleAnnotation(dxd)
```

We specify two design formulae, which indicate that the `type` _**factor**_ should be treated as a blocking factor:

```{r formulas2}
formulaFullModel    =  ~ sample + exon + type:exon + condition:exon
formulaReducedModel =  ~ sample + exon + type:exon 
```

Compare these formulae with the default formulae given in previous sections. We have added, in both the full model and the reduced model, the term `type:exon`. Therefore, any dependence of exon
usage on library type will be absorbed by this term and accounted for equally in the full
and a reduced model, and the likelihood ratio test comparing them will only detect differences
in exon usage that can be attributed to `condition`, independent of `type`.

Next, we estimate the dispersions. This time, we need to inform the `estimateDispersions()` function
about our design by providing the full model's formula, which should be used instead of the default 
formula.

```{r estDisps_again}
dxd = estimateDispersions( dxd, formula = formulaFullModel )
```

The test function now needs to be informed about both formulae

```{r test_again}
dxd = testForDEU( dxd, 
	reducedModel = formulaReducedModel, 
        fullModel = formulaFullModel )
```

Finally, we get a summary table, as before.

```{r res_again}
dxr2 = DEXSeqResults( dxd )
```

How many significant DEU cases have we got this time?

```{r table2}
table( dxr2$padj < 0.1 )
```

We can now compare with the previous result:

```{r table3}
table( before = dxr1$padj < 0.1, now = dxr2$padj < 0.1 )
```

Accounting for the library type has allowed us to find six more hits, which confirms that accounting for the covariate improves power.

# Visualization

The `plotDEXSeq()` provides a means  to visualize the results of an analysis.

```{r plot1, fig.height=8, fig.width=12, fig.cap="Fitted expression. The plot represents the expression estimates from a call to `testForDEU()`. Shown in red is the exon that showed significant differential exon usage."}
plotDEXSeq( dxr2, "FBgn0010909", legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

```{r checkClaim}
wh = (dxr2$groupID=="FBgn0010909")
stopifnot(sum(dxr2$padj[wh] < formals(plotDEXSeq)$FDR)==1)
```

The result is shown in Figure \@ref(fig:plot1). This plot shows the fitted expression values of each of the exons of gene *FBgn0010909*, for each of the two conditions, treated and untreated. The function `plotDEXSeq()` expects at least two arguments, the _**DEXSeqDataSet**_ object and the gene ID. The option `legend=TRUE` causes a legend to be included. The three remaining arguments in the code chunk above are ordinary plotting parameters which are simply handed over to _**R**_ standard
plotting functions. They are not strictly needed and included here to improve appearance of
the plot. See the help page for `par()` for details.

Optionally, one can also visualize the transcript models (Figure \@ref(fig:plot2)), which can be
useful for putting differential exon usage results into the context of isoform regulation.

```{r plot2, fig.height=8, fig.width=12, fig.cap="Transcripts. As in the previous plots, but including the annotated transcript models"}
plotDEXSeq( dxr2, "FBgn0010909", displayTranscripts=TRUE, legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

Other useful options are to look at the count values from the individual samples, rather than at the
model effect estimates. For this display (option `norCounts=TRUE`), the counts are normalized by 
dividing them by the size factors (Figure \@ref(fig:plot3))

```{r plot3, fig.height=8, fig.width=12, fig.cap="Normalized counts. As in the previous plots, with normalized count values of each exon in each of the samples."}
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, norCounts=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

As explained in Section \@ref(overview), `r BiocStyle::Biocpkg("DEXSeq")` is designed to find
changes in relative exon usage, i.e., changes in the expression of individual exons that are
not simply the consequence of overall up- or down-regulation of the gene. To visualize such changes, it is sometimes advantageous to remove overall changes in expression from the
plots. Use the option `splicing=TRUE` for this purpose.

```{r plot4, fig.height=8, fig.width=12, fig.cap="Fitted splicing. As in the previous plots, but after subtraction of overall changes in gene expression."}
plotDEXSeq( dxr2, "FBgn0010909", expression=FALSE, splicing=TRUE,
   legend=TRUE, cex.axis=1.2, cex=1.3, lwd=2 )
```

To generate an easily browsable, detailed overview over all analysis results, the package provides an HTML report generator, implemented in the function `DEXSeqHTML()`. This function uses the package `r BiocStyle::CRANpkg("hwriter")` [@hwriter] to create a result table with links to plots for the
significant results, allowing a more detailed exploration of the results. 

```{r DEXSeqHTML, cache=TRUE, eval=FALSE}
DEXSeqHTML( dxr2, FDR=0.1, color=c("#FF000080", "#0000FF80") )
```

# Parallelization and large number of samples

`r BiocStyle::Biocpkg("DEXSeq")` analyses can be computationally heavy, especially with data sets that 
comprise a large number of samples, or with genomes containing genes with large numbers of exons.  While 
some steps of the analysis work on the whole data set, the computational load can be parallelized for 
some steps. We use the package `r BiocStyle::Biocpkg("BiocParallel")`, and implemented the `BPPARAM=` 
parameter of the functions `estimateDispersions()`, `testForDEU()` and `estimateExonFoldChanges()`:

```{r para1,cache=TRUE,results='hide', eval=FALSE}
BPPARAM = MultiCoreParam(4)
dxd = estimateSizeFactors( dxd )
dxd = estimateDispersions( dxd, BPPARAM=BPPARAM)
dxd = testForDEU( dxd, BPPARAM=BPPARAM)
dxd = estimateExonFoldChanges(dxd, BPPARAM=BPPARAM)
```

For running analysis with a large number of samples (e.g. more than 20), we recommend configuring 
`BatchJobsParam()` classes in order to distribute the calculations across a computer cluster and 
significantly reduce running times. Users might also consider reducing the number of tests by 
filtering for lowly expressed isoforms or exonic regions with low counts. Appart from reducing 
running times, this filtering step also leads to more accurate results [@soneson2015]. 

Nevertheless, `r BiocStyle::Biocpkg("DEXSeq")` was designed to work with small sample numbers,
and users might have to consider alternatives for large numbers of samples such as the 
diffSplice function from `r BiocStyle::Biocpkg("limma")`, `r BiocStyle::Biocpkg("satuRn")` 
or non-parametric approaches for exon usage testing.

# Perform a standard differential exon usage analysis in one command

In the previous sections, we went through the analysis step by step. Once you are sufficiently 
confident about the work flow for your data, its invocation can be streamlined by the wrapper 
function `DEXseq()`, which runs the analysis shown above through a single function call. 

In the simplest case, construct the _**DEXSeqDataSet**_ as shown before, then run `DEXSeq()` 
passing the _**DEXSeqDataSet**_ as only argument, this function will output 
a _**DEXSeqResults**_ object.

```{r alldeu}
dxr = DEXSeq(dxd)
class(dxr)
```


# Appendix

## Controlling FDR at the gene level

`r BiocStyle::Biocpkg("DEXSeq")` returns one $p$-value for each exonic region. Using 
these p-values, a rejection threshold is established according to a multiple testing correction method. Consider a setting with $M$ exonic regions, where  $p_{k}$ is the $p$-value for the $k$ exonic region and 
$\theta \in {p_{1}, ..., p_{M}} \subset [0, 1]$ is a rejection threshold, leading to 
$V=\left|\left\{k: p_k\le\theta\right\}\right|$ rejections (i.e. number of exonic regions being 
DEU). The Benjamini-Hochberg method establishes that FDR is controlled at the level $q^{*}$ where

$q^* = \frac{M \theta}{\left|\left\{k: p_k\le\theta\right\}\right|}$.

Note that $\theta$ in the denominator of this expression is the probability of rejecting a single 
hypothesis if it is true. $q^*$ allows to estimate the number of exons that are differentially
used at a specific FDR. 

However, sometimes it is informative to know what is the FDR at the gene level, i.e. knowing the
numer of genes with at least one differentially used exon while keeping control of FDR. For this 
scenario, let $p_{i,l}$ be the $p$-value from the DEU test for exon $l$ in gene $i$, let $n_i$ 
the number of exons in gene $i$, and $M$ the number of genes. A gene $i$ has at least one DEU 
exon if $\exists l$ such that $p_{i,l}\le\theta$. Then, FDR is controlled at the level

$\frac{\sum_{i=1}^{M} 1-(1-\theta)^{n_i}}{\left|\left\{i: \exists l:p_{il}\le\theta\right\}\right|}$.

The implementation of the formula above is provided by the function `perGeneQValue()`. For
the `r BiocStyle::Biocpkg("pasilla")` example, the code below calculates the number of genes (at a FDR of 10%) with at least one differentially used exon.

```{r pergeneqval}
numbOfGenes = sum( perGeneQValue(dxr) < 0.1)
numbOfGenes
```

## Preprocessing with python

### Preparing the annotation file with python

The preprocessing steps of the analysis are done using two Python scripts that we provide with `r BiocStyle::Biocpkg("DEXSeq")`.

You do not need to know how to use Python; however you have to install the Python package **HTSeq**, following the explanations given on the [**HTSeq** web page](http://www-huber.embl.de/users/anders/HTSeq/doc/install.html).

Once you have installed **HTSeq**, you can use the two Python scripts, *dexseq_prepare_annotation.py* and *dexseq_count.py*, that come with the `r BiocStyle::Biocpkg("DEXSeq")` package. If you have trouble finding them, start R and ask for the installation directory with:

```{r systemFile}
pythonScriptsDir = system.file( "python_scripts", package="DEXSeq" )
list.files(pythonScriptsDir)
system.file( "python_scripts", package="DEXSeq", mustWork=TRUE )
```

The displayed path should contain the two files. If it does not, try to re-install `r BiocStyle::Biocpkg("DEXSeq")` 
(as usual, with the function `BiocManager::install()`). An alternative work flow, which replaces the two Python-based steps with R based code, is also available in the Appendix section of this vignette.

The Python scripts expect a GTF file with gene models for your species. We have tested our tools chiefly with GTF files from Ensembl and hence recommend to use these, as files from other providers sometimes do not adhere fully to the GTF standard and cause the preprocessing to fail. Ensembl GTF files can be found in the "FTP Download" sections of the Ensembl web sites (i.e., Ensembl, EnsemblPlants, EnsemblFungi, etc.). Make sure that your GTF file uses a coordinate system that matches the reference genome that you have used for aligning your reads. The safest way to ensure this is to download the reference genome from Ensembl, too. If you cannot use an Ensembl GTF file, see Appendix \@ref(requirements-on-gtf-files) for advice on converting GFF files from other sources to make them suitable as input for the *dexseq_prepare_annotation.py* script.

In a GTF file, many exons appear multiple times, once for each transcript that contains them. We need to "collapse" this information to define *exon counting bins*, i.e., a list of intervals, each corresponding to one exon or part of an exon. Counting bins for parts of exons arise when an exonic region appears with different boundaries in different transcripts. See Figure 1 of the `r BiocStyle::Biocpkg("DEXSeq")` paper (@DEXSeqPaper) for an illustration. The Python script *dexseq_prepare_annotation.py*
takes an Ensembl GTF file and translates it into a GFF file with collapsed exon counting bins.

Make sure that your current working directory contains the GTF file and call the script (from the command line shell, not from within R) with

```
python /path/to/library/DEXSeq/python_scripts/dexseq_prepare_annotation.py Drosophila_melanogaster.BDGP5.72.gtf Dmel.BDGP5.25.62.DEXSeq.chr.gff
```

In this command, which should be entered as a single line, replace */path/to.../python_scripts* with the correct path to the Python scripts, which you have found with the call to `system.file()` shown above. *Drosophila_melanogaster.BDGP5.72.gtf* is the Ensembl GTF file (here the one for fruit fly, already de-compressed) and *Dmel.BDGP5.25.62.DEXSeq.chr.gff* is the name of the output file.

In the process of forming the counting bins, the script might come across overlapping genes. If two genes on the same strand are found with an exon of the first gene overlapping with an exon of the second gene, the script's default behaviour is to combine the genes into a single "aggregate gene" which is subsequently referred to with the IDs of the individual genes, joined by a plus ('+') sign. If you do not like this behaviour, you can disable aggregation with the option `-r no`. Without aggregation, exons that overlap with other exons from different genes are simply skipped.

### Counting reads with python

Read counting is done with the script *python_count.py*:

```
python /path/to/library/DEXSeq/python_scripts/dexseq_count.py Dmel.BDGP5.25.62.DEXSeq.chr.gff untreated1.sam untreated1fb.txt
```

This command (again, to be entered as a single line) expects two files in the current working directory, namely the GFF file produced in the previous step (here *Dmel.BDGP5.25.62.DEXSeq.chr.gff*) and a SAM file with the aligned reads from a sample (here the file *untreated1.sam* with the aligned reads from the first control sample). The command generates an output file, here called *untreated1fb.txt*, with one line for each exon counting bin defined in the flattened GFF file. The lines contain the exon counting bin IDs (which are composed of gene IDs and exon bin numbers), followed by a integer number which indicates the number of reads that were aligned such that they overlap with the counting bin.

Use the script multiple times to produce a count file from each of your SAM files.

There are a number of crucial points to pay attention to when using the *python_count.py* script:

**Paired-end data: ** If your data is from a paired-end sequencing run, you need to add the option `-p yes` to 
the command to call the script. (As usual, options have to be placed before the file names, surrounded by spaces.) In addition, the SAM file needs to be sorted, either by read name or by position. Most aligners produce sorted SAM files; if your SAM file is not sorted, use `samtools sort -n` to sort by read name (or `samtools sort`) to sort by position. (See @DEProt, if you need further explanations on how to sort SAM files.) Use the option `-r pos` or `-r name` to indicate whether your paired-end data is sorted by alignment position or by read name.

**Strandedness:** By default, the counting script assumes your library to be *strand-specific*, i.e., reads are aligned to the same strand as the gene they originate from. If you have used a library preparation protocol that does not preserve strand information (i.e., reads from a given gene can appear equally likely on either strand), you need to inform the script by specifying the option `-s no`. If your library preparation protocol reverses the strand (i.e., reads appear on the strand opposite to their gene of origin), use `-s reverse`. In case of paired-end data, the default (`-s yes`) means that the read from the first sequence pass is on the same strand as the gene and the read from the second pass on the opposite strand ("forward-reverse" or "fr" order in the parlance of the Bowtie/TopHat manual) and the options `-s reverse` specifies the opposite case.

**SAM and BAM files:** By default, the script expects its input to be in plain-text SAM format.
However, it can also read BAM files, i.e., files in the the compressed binary variant of the SAM format.
If you wish to do so, use the option `-f bam`. This works only if you have installed the Python
package [*pysam*](https://code.google.com/p/pysam/).

**Alignment quality:** The scripts takes a further option, `-a` to specify the minimum alignment quality (as given in the fifth column of the SAM file). All reads with a lower quality than specified (with default `-a 10`) are skipped.

**Help pages:** Calling either script without arguments displays a help page with an overview of all options and arguments.

### Reading the data from the python ouputs into R

The remainder of the analysis is now done in **R**. We will use the output of the python scripts for the *pasilla* experiment, that can be found in the package `r BiocStyle::Rpackage("pasilla")`. Open an **R** session and type:

```{r loadDEXSeq}
inDir = system.file("extdata", package="pasilla")
countFiles = list.files(inDir, pattern="fb.txt$", full.names=TRUE)
basename(countFiles)
flattenedFile = list.files(inDir, pattern="gff$", full.names=TRUE)
basename(flattenedFile)
```

Now, we need to prepare a sample table. This table should contain one row for each library, and columns for all relevant information such as name of the file with the read counts, experimental conditions, technical information and further covariates. To keep this vignette simple, we construct the table on the fly.

```{r sampleTable}
sampleTable = data.frame(
   row.names = c( "treated1", "treated2", "treated3", 
      "untreated1", "untreated2", "untreated3", "untreated4" ),
   condition = c("knockdown", "knockdown", "knockdown",  
      "control", "control", "control", "control" ),
   libType = c( "single-end", "paired-end", "paired-end", 
      "single-end", "single-end", "paired-end", "paired-end" ) )
```

While this is a simple way to prepare the table, it may be less error-prone and more prudent to used an existing table that had already been prepared when the experiments were done, save it in CSV format and use the **R** function `read.csv()` to load it.

In any case, it is vital to check the table carefully for correctness.

```{r check}
sampleTable
```

Our table contains the sample names as row names and the two covariates that vary between samples: first the experimental condition (factor _**condition**_ with levels **control** and **treatment**) and the library type (factor **libType**), which we included because the samples in this particular experiment were sequenced partly in single-end runs and partly in paired-end runs. 
For now, we will ignore this latter piece of information, and postpone the discussion of how to include such additional covariates to Section \@ref(complexdesigns). If you have only a single covariate and want to perform a simple analysis, the column with this covariate should be named **condition**.

Now, we construct an _**DEXSeqDataSet**_ object from this data. This object holds all the input data and will be passed along the stages of the subsequent analysis. We construct the object with the function `DEXSeqDataSetFromHTSeq()`, 
as follows:

```{r message=FALSE}
library( "DEXSeq" )

dxd = DEXSeqDataSetFromHTSeq(
   countFiles,
   sampleData=sampleTable,
   design= ~ sample + exon + condition:exon,
   flattenedfile=flattenedFile )
```

The function takes four arguments. First, a vector with names of count files, i.e., of files that have been generated with the *dexseq_count.py* script. The function will read these files and arrange the count data in a matrix, which is stored in the _**DEXSeqDataSet**_ object `dxd`. The second argument is our sample table, with one row for each of the files listed in the first argument. This information is simply stored as is in the object's meta-data slot (see below). The third argument is a formula of the form `~ sample + exon + condition:exon` that specifies the contrast with of a variable from the sample table columns and the **'exon'** variable. Using this formula, we are interested in differences in exon usage due to the "condition" variable changes. Later in this vignette, we will how to add additional variables for complex designs.  The fourth argument is a file name, now of the flattened GFF file that was generated with *dexseq_prepare_annotation.py* and used as input to *dexseq_count.py* when creating the count file.

## Preprocessing using featureCounts

As an alternative to *HTSeq*, one could use the function `featureCounts()` to count the number of reads overlapping with each exonic region. The output files from `featureCounts()` can then be used as input to DEXSeq. In [this](https://github.com/vivekbhr/Subread_to_DEXSeq) *Github* repository, Vivek Bhardwaj provides scripts and documentation that integrate `featureCounts()` with `r BiocStyle::Biocpkg("DEXSeq")`. He also provides a function to import the count files into _**R**_ and generate a _**DEXSeqDataSet**_ object. 

## Further accessors

The function `geneIDs()` returns the gene ID column of the feature data as a character
vector, and the function `exonIDs()` return the exon ID column as a _**factor**_.

```{r acc}
head( geneIDs(dxd) )
head( exonIDs(dxd) )
```

These functions are useful for subsetting an _**DEXSeqDataSet**_ object.

## Overlap operations

The methods `subsetByOverlaps()` and `findOverlaps()` have been implemented for the _**DEXSeqResults**_ object, the `query=` argument must be a _**DEXSeqResults**_ object. 

```{r grmethods}
interestingRegion = GRanges( "chr2L", 
   IRanges(start=3872658, end=3875302) )
subsetByOverlaps( x=dxr, ranges=interestingRegion )
findOverlaps( query=dxr, subject=interestingRegion )
```

This functions could be useful for further downstream analysis.

## Methodological changes since publication of the paper {#methodChanges}

In our paper [@DEXSeqPaper], we suggested to fit for each exon a model that includes separately 
the counts for all the gene's exons. However, this turned out to be computationally inefficient
for genes with many exons, because the many exons required large model matrices, which are computationally expensive to deal with. We have therefore modified the approach: when fitting
a model for an exon, we now sum up the counts from all the other exon and use only the total, rather than the individual counts in the model. Now, computation time per exon is independent of the number of other exons in the gene, which improved `r BiocStyle::Biocpkg("DEXSeq")`'s scalability. While the $p$ values returned by the two approaches are not exactly equal, the differences were very minor in the tests that we performed.

Deviating from the paper's notation, we now use the index $i$ to indicate a specific counting bin, 
with $i$ running through all counting bins of all genes. The samples are indexed with $j$, as in the paper. We write $K_{ij0}$ for the count or reads mapped to counting bin $i$ in sample $j$ and $K_{ij1}$ for the sum of the read counts from all other counting bins in the same gene. Hence, when we write $K_{ijl}$, the third index $l$ indicates whether we mean the read count for bin $i$ ($l=0$) or the sum of counts for all other bins of the same gene ($l=1$). As before, we fit a GLM of the negative binomial (NB) family
$ K_{ijl} \sim \operatorname{NB}(\text{mean}=s_j\mu_{ijl},\text{dispersion}=\alpha_i)$ and we write
$\log_2 \mu_{ijl} = \beta^\text{S}_{ij} + l \beta^\text{E}_{i} + \beta^\text{EC}_{i\rho_j}$.

This model is fit separately for each counting bin $i$. The coefficient $\beta^\text{S}_{ij}$ accounts for the sample-specific contribution (factor `sample`), the term $\beta^\text{E}_{i}$
is only included if $l=1$ and hence estimates the logarithm of the ratio $K_{ij1}/K_{ij0}$ between the counts for all other exons and the counts for the tested exon. As this coefficient is estimated from data from all samples, it can be considered as a measure of "average exon usage". In the R model formula, it is represented by the term `exon` with its two levels `this` ($l=0$) and `others` ($l=1$). Finally, the last term, $\beta^\text{EC}_{i,\rho_j}$, captures the interaction `condition:exon`, i.e., the change in exon usage if sample $j$ is from experimental condition group $\rho(j)$. Here, the first condition, $\rho=0$, is absorbed in the sample coefficients, i.e., $\beta^\text{EC}_{i0}$ is fixed to zero and does not appear in the model matrix.

For the dispersion estimation, one dispersion value $\alpha_i$ is estimated with Cox-Reid-adjusted maximum likelihood using the full model given above. A mean-variance relation is fitted using the individual dispersion values. Finally, the individual values are shrinked towards the fitted values. For more details about this shrinkage approach look at the `r BiocStyle::Biocpkg("DESeq2")` vignette and/or its manuscript [@deseq2]. For the likelihood ratio test, this full model 
is fit and compared with the fit of the reduced model, which lacks the interaction term $\beta^\text{EC}_{i\rho_j}$. As described in Section \@ref{complexdesigns}, alternative model formulae can be specified.

## Requirements on GTF files

In the initial preprocessing step described in Section \@ref{preparing-the-annotation}, the Python script *dexseq_prepare_annotation.py* is used to convert a GTF file with gene models into a GFF file with collapsed gene models. We recommend to use GTF files downloaded from Ensembl as input for this script, as files from other sources may deviate from the format expected by the script. Hence, if you need to use a GTF or GFF file from another source, you may need to convert it to the expected format. To help with this task, we here give details on how the *dexseq_prepare_annotation.py* script interprets a GFF file.

* The script only looks at `exon` lines, i.e., at lines which contain the term `exon` in the 
third ("type") column. All other lines are ignored.
* Of the data in these lines, the information about chromosome, start, end, and strand (1st, 4th, 5th, and 7th column) are used, and, from the last column, the attributes `gene_id` and `transcript_id`. The rest is ignored.
* The `gene_id` attribute is used to see which exons belong to the same gene. It must be called `gene_id` (and not `Parent` as in GFF3 files, or `GeneID` as in some older GFF files), and it must give the same identifier to all exons from the same gene, even if they are from different transcripts of this gene. (This last requirement is not met
by GTF files generated by the Table Browser function of the UCSC Genome Browser.)
* The `transcript_id` attribute is used to build the *transcripts* attribute in the flattened GFF file, which indicates which transcripts contain the described counting bin. This information is needed only to draw the transcript model at the bottom of the plots when the `displayTranscripts=` option to `plotDEXSeq()` is used.

Therefore, converting a GFF file to make it suitable as input to *dexseq_prepare_annotation.py* amounts to making sure that the exon lines have type `exon` and that the atributes giving gene ID (or gene symbol) and transcript ID are called `gene_id` and `transcript_id`, with this exact spelling. Remember to also take care that the chromosome names match those in your SAM files, and that the coordinates refer to the reference assembly that you used when aligning your reads.

# Session Information

The session information records the versions of all the packages used in the generation of the present document.

```{r sessionInfo}
sessionInfo()
```

# References

\name{DEXSeqHTML}
\alias{DEXSeqHTML}
\title{DEXSeq HTML report writer}
\description{
  This function generates an HTML report from the results
  of a \code{DEXSeqResults} object. Gives an simple report
  to explore differential exon usage results.
}
\usage{
  DEXSeqHTML(object, genes=NULL, path="DEXSeqReport", file="testForDEU.html", 
    fitExpToVar="condition", FDR=0.1, color=NULL, color.samples=NULL,
    mart="", filter="", attributes="", extraCols=NULL, BPPARAM=MulticoreParam(workers=1))
}
\arguments{
  \item{object}{
    An DEXSeqResults object
  }
  \item{genes}{
    A character vector of gene identifiers 
    to be included in the report. If NULL, 
    the genes included in the report will 
    be the significant hits at the given 
    false discovery rate. See "FDR" below.
  }
  \item{path}{
    A path in the system where to write the 
    report.
  }
  \item{file}{
    The name of the html file.
  }
  \item{fitExpToVar}{
    A variable contained in the design of the ecs; the counts 
    will be fitted to this variable to get the plotting values.
  }
  \item{FDR}{
    A false discovery rate
  }
  \item{color}{
    A vector of colors, one for each of the levels of the 
    values of "fitExpToVar".
  }
  \item{color.samples}{
    A vector of colors for each of the samples. If NULL, the colors 
    of each sample will be asigned according to its corresponding 
    condition. Useful to visualize complex experimental designs.
   }
   \item{mart}{
    object of class Mart, created with the useMart function, 
    with dataset specified
   }
   \item{filter}{ 
     Filters (ONLY ONE) that should be used in the query.  A
     possible list of filters can be retrieved using the function
     listFilters. Please note that the value of this filter will
     always be the geneIDs in the DEXSeqResults object.
   }
   \item{attributes}{
     Attributes you want to retrieve.  A possible list of
     attributes can be retrieved using the biomaRt function
     listAttributes.
   }
   \item{extraCols}{
     A data frame with one or more columns to add to the report.
     For example, additional information about the genes. The data frame
     should be indexed by the gene names of the ExonCountSet object,
     e.g. the rownames of the data frame should correspond to the gene 
     names.
   }
  \item{BPPARAM}{ 
     A 'BiocParallelParam' instance. 
     See \code{?bplapply} for details.
  }
}
\value{
  This function will write an HTML report in the directory specified by 'path'. 
  There, it will create an html file with the initial report page and a 
  directory called "files" in which SVG files with the plots and other html files are
  placed. Different plots with different labels are generated for each gene:
   - counts: the raw data, for each sample
   - fitted expression: the fitted coefficients per compared condition (e.g.: treated, untreated)
   - fitted splicing: as 'expression', but after removing overall gene-level differential expression: this is the view most relevant for the interpretation of DEXSeq results, which are about changes in relative exon usage (i.e.: relative to overall gene expression) 
}
\examples{
  \dontrun{
    data(pasillaDEXSeqDataSet, package="pasilla")
    dxr <- DEXSeq( dxd )
    DEXSeqHTML( object=dxr )
  }
}

\name{plotMA-methods}
\docType{methods}
\alias{plotMA}
\alias{plotMA-methods}
\alias{plotMA,DEXSeqDataSet-method}
\alias{plotMA,DEXSeqResults-method}
\title{Generate an MA plot}
\description{
  This function generates an MA plot.
}
\usage{

  \S4method{plotMA}{DEXSeqDataSet}( object, alpha=0.1, ylim=c(-2, 2), foldChangeColumn=NULL, ... )

  \S4method{plotMA}{DEXSeqResults}( object, alpha=0.1, ylim=c(-2, 2), foldChangeColumn=NULL, ... )

}
\arguments{
  \item{object}{
     Either a \code{DEXSeqDataSeq} or a \code{DEXSeqResults}.
  }
  \item{alpha}{
     the false discovery rate, i.e., threshold to the adjusted p values, to be used 
     to colour the dots
  }
  \item{ylim}{
     y-limits of the plot
  }
  \item{foldChangeColumn}{
     Name of the column containing the fold changes to plot. If NULL,
     the first fold change column from the object will be taken.
  }
  \item{...}{
    Further parameters to be passed through to \code{\link{plot}}.
  }
}

\examples{
 \dontrun{
    data(pasillaDEXSeqDataSet, package="pasilla")
    dxd <- estimateSizeFactors( dxd )
    dxd <- estimateDispersions( dxd )
    dxd <- testForDEU( dxd )
    dxd <- estimateExonFoldChanges( dxd )
    dxr <- DEXSeqResults( dxd )

    plotMA( dxr )

    plotMA( dxd )
    
 }
}

\name{DEXSeq}
\alias{DEXSeq}
\title{Performs the differential exon usage test in a single command}
\description{
  This function is a wrapper that calls the necessary functions
  to perform a differential exon usage test in a single command.
}
\usage{
  DEXSeq(object, 
    fullModel=design(object),
    reducedModel = ~ sample + exon, 
    BPPARAM=MulticoreParam(workers=1), 
    fitExpToVar="condition", quiet=TRUE )
}
\arguments{
  \item{object}{
    An DEXSeqDataSet object.
  }
  \item{fullModel}{
     The full model formula
  }
  \item{reducedModel}{
     Null model formula.
  }
  \item{BPPARAM}{
     A "BiocParallelParam" instance. 
     See \code{?bplapply} for details.
  }
  \item{fitExpToVar}{
    A variable name contained in the sample data. The expression 
    values will be fitted to this variable using the the formula 
    " ~ sample + fitExpToVar * exon".
  }
  \item{quiet}{
    Whether to print messages at each step
  }
}
\value{
  A \code{DEXSeqResults} object.
}
\examples{
  \dontrun{
      data(pasillaDEXSeqDataSet, package="pasilla")
      dxr <- DEXSeq( dxd )
  }
}

\docType{methods}
\name{estimateSizeFactors}
\alias{estimateSizeFactors}
\alias{estimateSizeFactors,DEXSeqDataSet-method}
\title{Estimate the size factors for a DEXSeqDataSet}
\usage{
  \S4method{estimateSizeFactors}{DEXSeqDataSet}(object,locfunc=median,geoMeans)
}
\arguments{
  \item{object}{a DEXSeqDataSet}

  \item{locfunc}{a function to compute a location for a
  sample. By default, the median is used. However,
  especially for low counts, the
  \code{\link[genefilter]{shorth}} function from the
  genefilter package may give better results.}

  \item{geoMeans}{by default this is not provided and the
  geometric means of the counts are calculated within the
  function. A vector of geometric means from another count
  matrix can be provided for a "frozen" size factor
  calculation}
}
\value{
  The DEXSeqDataSet passed as parameters, with the size
  factors filled in.
}
\description{
  Estimate the size factors for a DEXSeqDataSet
}
\details{
  This function calls the method estimateSizeFactors
  for the DESeqDataSet object, and adapts it to the
  specific use to the DEXSeqDataSet.
}
\references{
  \itemize{ \item Simon Anders, Wolfgang Huber: Differential
  expression analysis for sequence count data. Genome Biology
  11 (2010) R106,
  \url{http://dx.doi.org/10.1186/gb-2010-11-10-r106} }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\docType{methods}
\name{estimateDispersions}
\alias{estimateDispersions}
\alias{estimateDispersions,DEXSeqDataSet-method}
\title{Estimate the dispersions for a DEXSeqDataSet}
\usage{
\S4method{estimateDispersions}{DEXSeqDataSet}(
  object,
  fitType = c("parametric", "local", "mean", "glmGamPoi"),
  maxit = 100,
  niter = 10,
  quiet = FALSE,
  formula = design(object),
  BPPARAM = SerialParam()
)
}
\arguments{
\item{object}{A DEXSeqDataSet.}

\item{fitType}{Either "parametric", "local", "mean" or "glmGamPoi"
for the type of fitting of dispersions to the mean
intensity. See ?estimateDispersions,DESeqDataSet-method for details.
If "glmGamPoi" is selected, the GLM fitter from "glmGamPoi" is also
used for estimating the dispersion estimates.}

\item{maxit}{Control parameter: maximum number of iterations to allow for convergence}

\item{niter}{Number of times to iterate between estimation of means and estimation of dispersion.}

\item{quiet}{Whether to print messages at each step.}

\item{formula}{Formula used to fit the dispersion estimates.}

\item{BPPARAM}{A "BiocParallelParam" instance. See \code{?bplapply} for details.}
}
\value{
A DEXSeqDataSet with the dispersion information filled in as metadata columns.
}
\description{
This function obtains dispersion estimates for negative binomial distributed data for the specific case for DEXSeq.
}
\details{
See ?estimateDispersions,DESeqDataSet-method for details.
}
\examples{
data(pasillaDEXSeqDataSet, package="pasilla")
dxd <- estimateSizeFactors( dxd )
dxd <- estimateDispersions( dxd )

}

\name{DEXSeqDataSet-methods}
\docType{methods}
\alias{[,DEXSeqDataSet-method}
\alias{[,DEXSeqDataSet,ANY-method}
\alias{[,DEXSeqDataSet,ANY,ANY-method}
\alias{[,DEXSeqDataSet,ANY,ANY,ANY-method}
\alias{colData<-,DEXSeqDataSet,DataFrame-method}
\alias{$<-,DEXSeqDataSet-method}
\title{Subset and replacements of DEXSeqDataSet slots}
\description{
  Subset and replacements of DEXSeqDataSet slots.
}
\usage{
    \S4method{[}{DEXSeqDataSet}(x, i, j, ..., drop=TRUE)
    \S4method{$}{DEXSeqDataSet}(x, name) <- value
}
\arguments{
    \item{x}{
        A DEXSeqDataSet object.
    }
    \item{i}{
        Indices specifying elements to extract.
    }
    \item{j}{
        Indices specifying elements to extract.
    }
    \item{name}{ A symbol representing the name of a column of
       \code{colData}. }
    \item{value}{An object of a class specified in the S4 method signature.}
    \item{...}{
        Other arguments passed to lower functions
    }
    \item{drop}{
        Ignored for subsetting DEXSeqDataSet objects.
    }
}

\name{DEXSeq-defunct}
\alias{DEXSeq-defunct}
\title{
  This functions are deprecated and will become defunct.
}
\description{
  The ExonCountSet object has been deprecated and substituted by
  the DEXSeqDataSet object.  Therefore, all the functions and methods
  for the ExonCountSet object were deprecated. These functions have 
  been subsituted by new functions and methods designed for DEXSeqDataSet objects.
}
\details{
  The replacements of defunct functions are summarized in the
  following items.
  \itemize{
    \item{newExonCountSet: \code{DEXseqDataSet}}
    \item{DEUresultTable: \code{DEXseqResults}}
    \item{subsetByGenes: }
    \item{geneCountTable: }
    \item{estimateExonDispersionsForModelFrame_BM: estimateDispersions-DEXSeqDataSet}
    \item{estimateDispersions_BM: estimateDispersions-DEXSeqDataSet}
    \item{testGeneForDEU_BM: \code{testForDEU}}
    \item{testForDEU_BM: \code{testForDEU}}
    \item{doCompleteDEUAnalysis: \code{DEXSeq}}
    \item{makeCompleteDEUAnalysis_BM: \code{DEXSeq}}
    \item{read.HTSeqCounts: \code{DEXSeqDataSetFromHTSeq}}
    \item{countTableForGene: }
    \item{fitDispersionFunction: estimateDispersions-DEXSeqDataSet}
    \item{estimatelog2FoldChange: \code{estimateExonFoldChanges}}
    \item{modelFrameForGene: }
    \item{buildExonCountSet: \code{DEXSeqDataSetFromSE}}
    \item{constructModelFrame: }
    \item{getCountVector: }
    \item{estimateExonDispersion: estimateDispersions-DEXSeqDataSet}
    \item{testExonForDEU: \code{testForDEU}}
    \item{doCompleteDEUAnalysis: \code{DEXSeq}}
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{testForDEU}
\alias{testForDEU}
\title{Test for Differential Exon Usage}
\usage{
testForDEU(
  object,
  fullModel = design(object),
  reducedModel = ~sample + exon,
  BPPARAM = SerialParam(),
  fitType = c("DESeq2", "glmGamPoi")
)
}
\arguments{
\item{object}{A DEXSeqDataSet object.}

\item{fullModel}{The full model formula.}

\item{reducedModel}{Null model formula.}

\item{BPPARAM}{A "BiocParallelParam" instance. See \code{?bplapply} for details.}

\item{fitType}{Specifies what internal engine to use for fitting the GLMs. Options are "DESeq2" or "glmGamPoi".}
}
\value{
A \code{DEXSeqDataSet} with slots filled with information about the test.
}
\description{
This will perform a likelihood ratio test for differential
exon usage. Internally, it calls the DESeq2 function \code{nbinomLRT}.
}
\details{
The information of the variables of the formulas
should be present in the \code{colData} of the
\code{DEXSeqDataSet} object.
}
\examples{

data(pasillaDEXSeqDataSet, package="pasilla")
dxd <- estimateSizeFactors( dxd )
dxd <- estimateDispersions( dxd )
dxd <- testForDEU( dxd )
}

\docType{class}
\name{DEXSeqResults-class}
\alias{DEXSeqResults}
\alias{DEXSeqResults-class}
\title{DEXSeqResults object}
\usage{
  DEXSeqResults( object, independentFiltering=TRUE, filter)
}
\arguments{
    \item{object}{
        A \code{DEXSeqDataSet} object.
    }
    \item{independentFiltering}{
        Logical indicating whether independent filtering should be 
    	applied automatically. 
	}
    \item{filter}{
        A vector of filter statistics over which the independent
        filtering will be optimized. The default is the normalized
        exon means.
	}						   
}
\value{
  A DEXSeqResults object.
}
\description{
  The \code{DEXSeqResults} object is a subclass 
  of a \code{DataFrame}. It collects relevant 
  information from a \code{DEXSeqDataSet} object with
  the results generated from testing for differences
  in exon usage.
}
\examples{
  data(pasillaDEXSeqDataSet, package="pasilla")
  dxd <- estimateSizeFactors( dxd )
  dxd <- estimateDispersions( dxd )
  dxd <- testForDEU( dxd )
  dxr <- DEXSeqResults( dxd )
}

\name{findOverlaps}
\docType{methods}
\alias{subsetByOverlaps}
\alias{subsetByOverlaps-methods}
\alias{subsetByOverlaps,DEXSeqResults,GenomicRanges-method}
\alias{findOverlaps}
\alias{findOverlaps-methods}
\alias{findOverlaps,DEXSeqResults,GenomicRanges-method}
\title{Methods from GRanges overlaps for the DEXSeq object}
\description{
  This function generates an MA plot.
}
\usage{

  \S4method{subsetByOverlaps}{DEXSeqResults,GenomicRanges}( x, ranges, maxgap = -1L, 
    minoverlap = 0L, type = c("any", "start", "end", "within", "equal"),
    ignore.strand = FALSE )

  \S4method{findOverlaps}{DEXSeqResults,GenomicRanges}( query, subject, maxgap = -1L, 
    minoverlap = 0L, type = c("any", "start", "end", "within", "equal"),
    ignore.strand = FALSE )

}
\arguments{
  \item{query, x}{
     Either a \code{DEXSeqResults} object.
  }
  \item{subject, ranges}{
      A GRanges or GRangesList object.
  }
  \item{maxgap, minoverlap, type}{
    See \code{findOverlaps} in the IRanges package for
    a description of these arguments.
  }
  \item{ignore.strand}{
    See \code{findOverlaps} in the GenomicRanges package for
    a description of this.
  }

}

\examples{
  data(pasillaDEXSeqDataSet, package="pasilla")
  dxd <- estimateSizeFactors( dxd )
  dxd <- estimateDispersions( dxd )
  dxd <- testForDEU( dxd )
  dxr <- DEXSeqResults( dxd )

  interestingRegion = GRanges("chr2L", IRanges(start=3872658, end=3875302))

  subsetByOverlaps( x=dxr, ranges=interestingRegion )
  findOverlaps( query=dxr, subject=interestingRegion )

}

\docType{class}
\name{DEXSeqDataSet-class}
\alias{DEXSeqDataSet}
\alias{DEXSeqDataSet-class}
\alias{DEXSeqDataSetFromHTSeq}
\alias{DEXSeqDataSetFromSE}
\title{DEXSeqDataSet object and constructors}
\usage{

  DEXSeqDataSet( countData, sampleData, 
    design= ~ sample + exon + condition:exon, 
    featureID, groupID, featureRanges=NULL, 
    transcripts=NULL, alternativeCountData=NULL)

  DEXSeqDataSetFromHTSeq( 
    countfiles, sampleData, 
    design= ~ sample + exon + condition:exon, 
    flattenedfile=NULL )

  DEXSeqDataSetFromSE( SE, 
    design= ~ sample + exon + condition:exon )

}
\arguments{
  \item{countData}{
    A matrix of count data of non-negative integer values. 
    The rows correspond to counts for each exon counting bin, 
    the columns correspond to samples. Note that biological 
    replicates should each get their own column, while the 
    counts of technical replicates (i.e., several sequencing 
    runs/lanes from the same sample) should be summed up 
    into a single column
  }
  \item{alternativeCountData}{
    DEXSeq can be also used for test for differences in exon inclusion
    based on the number of reads supporting the inclusion of an exon and
    the number of reads supporting the exclusion of an exon. A matrix of
    count data of non-negative integer values.The rows correspond to exonic
    regions and the columns correspond to samples. This matrix should contain
    the number of exon-exon junction reads that skip each exon in each sample.
    If NULL, then the sum of the other exons belonging to the same gene is
    considered for testing (i.e. the normal DEXSeq approach).
  }
  \item{countfiles}{
    A character vector containing the path to the files that
    were originated with the script 'dexseq_count.py'.
  }
  \item{sampleData}{
    A \code{data.frame} with the annotation (e.g. treatments, or tissue 
    types, or phenotypes, or the like).  The number of rows in 
    the data frame must to be equal to the number of columns of 
    the countData matrix, assigning the annotation of each sample.
  }
  \item{design}{
    A formula which specifies the design of the experiment. It must
    specify an interaction term between a variable from the sampleData
    columns with the 'exon' variable. By default, the design 
    will be '~ sample + exon + condition:exon'. This formula indicates
    the contrast between 'condition' and exon', i.e. differences in exon 
    usage due to changes in the 'condition' variable.  See the vignette 
    for more examples of other designs.
  }
  \item{featureID}{
    A character vector of counting regions identifiers ordered according 
    to the rows in countData.  The identifiers names can be repeated
    between groups but not within groups. 
  }
  \item{groupID}{
    A vector of group identifiers ordered according to its
    respective row in countData. It must reflect the sets
    of counting regions belonging to the same group, for 
    example, exon bins in belonging to the same gene should
    have the same group identifier. 
  }
  \item{featureRanges}{
    Optional. \code{GRanges} or \code{GRangesList} describing
    the genomic coordinates of each of the rows of countData.
  }
  \item{transcripts}{
    Optional. A \code{list} of the same length as the number of
    rows in countData. Each element of the list should contain
    a character vector of transcript identifiers indicating the
    transcript identifiers overlapping with the exonic regions.
  }
  \item{flattenedfile}{
    A character vector containing the path to the flattened annotation 
    file that was originated with the script 'dexseq_prepare_annotation.py'.
  }
  \item{SE}{
    A \code{SummarizedExperiments} object, originated using the function
    \code{SummarizeOverlaps}. See examples for more details.
  }
}

\value{
  A DEXSeqDataSet object.
}

\seealso{
  \code{DEXSeqDataSetFromHTSeq}
  \code{DEXSeqDataSetFromSE}
}

\description{
  The \code{DEXSeqDataSet} is a subclass of \code{DESeqDataSet},
  specifically designed to adapt the \code{DESeqDataSet} to test
  for differences in exon usage. 
}
\examples{

\dontrun{
  #######################################
  ### From the output of the          ### 
  ### acconpaning python scripts      ### 
  #######################################

  inDir = system.file("extdata", package="pasilla", mustWork=TRUE)
  flattenedfile = file.path(inDir, "Dmel.BDGP5.25.62.DEXSeq.chr.gff")
  sampleData = data.frame(
      condition = c( rep("treated", 3), rep("untreated", 4) ),
      type = c("single", "paired", "paired", "single", "single", "paired", "paired") )
     
  countFiles <- list.files(inDir, pattern="fb.txt")
  rownames( sampleData ) <- countFiles

  DEXSeqDataSetFromHTSeq(
    countfiles=file.path( inDir, countFiles ),
    sampleData = sampleData,
    design = ~ sample + exon + type:exon + condition:exon,
    flattenedfile=flattenedfile )


  #######################################
  ### From GRanges derived objects    ###
  #######################################

  library(GenomicRanges)
  library(GenomicFeatures)
  library(txdbmaker)
  library(GenomicAlignments)

  hse <- makeTxDbFromBiomart(biomart="ensembl",
                             dataset="hsapiens_gene_ensembl",
                             host="grch37.ensembl.org")

  bamDir <- system.file( 
    "extdata", package="parathyroidSE", mustWork=TRUE )
  fls <- list.files( bamDir, pattern="bam$", full=TRUE )
  
  bamlst <- BamFileList( 
    fls, index=character(), 
    yieldSize=100000, obeyQname=TRUE )

  exonicParts <- exonicParts( hse, linked.to.single.gene.only = TRUE )

  SE <- summarizeOverlaps( exonicParts, bamlst, 
    mode="Union", singleEnd=FALSE,
    ignore.strand=TRUE, inter.feature=FALSE, fragments=TRUE )

  colData(SE)$condition <- c("A", "A", "B")

  DEXSeqDataSetFromSE( SE, 
    design= ~ sample + exon + condition:exon )
   

  #######################################
  ### From elementary data structures ###
  #######################################
  countData <- matrix( rpois(10000, 100), nrow=1000 )
  sampleData <- data.frame(
      condition=rep( c("untreated", "treated"), each=5 ) )
  design <- formula( ~ sample + exon + condition:exon )
  groupID <- rep(
      paste0("gene", 1:10),
      each= 100 )
  featureID <- rep(
      paste0("exon", 1:100),
      times= 10 )
  DEXSeqDataSet( countData, sampleData, design,
              featureID, groupID )

}

}

\name{perGeneQValue}
\alias{perGeneQValue}
\title{Summarize per-exon p-values into per-gene q-values.}
\description{
  The use case for this function is the following analysis: given
  per-exon p-values for null hypothesis H0, we can determine the number
  of genes in which at least for one exon H0 is rejected. What is the
  associated false disovery rate?
}
\usage{
perGeneQValue(object, p = "pvalue", method = perGeneQValueExact)
}
\arguments{
  \item{object}{
    An \code{DEXSeqResults} object.
  }
  \item{p}{A character string indicating the name of the columns in
    \code{DEXSeqResults} from which to take the per-exon p-values.}
  \item{method}{Use the default value. This is for debugging only.}
  }
\details{Details}
\value{
  A named numeric vector, values are per-gene q-values, names are gene.
}
\examples{

  data(pasillaDEXSeqDataSet, package="pasilla")
  dxd <- estimateSizeFactors( dxd )
  dxd <- estimateDispersions( dxd )
  dxd <- testForDEU( dxd )
  dxr <- DEXSeqResults( dxd )

  perGeneQValue(dxr)


}

\docType{methods}
\name{plotDispEsts}
\alias{plotDispEsts}
\alias{plotDispEsts,DEXSeqDataSet-method}
\title{Plot dispersion estimates}
\usage{
\S4method{plotDispEsts}{DEXSeqDataSet}(object, ymin,
  genecol = "black", fitcol = "red", finalcol = "dodgerblue",
  legend=TRUE, xlab, ylab, log = "xy", cex = 0.45, ...)
}
\arguments{
  \item{object}{a DESeqDataSet}

  \item{ymin}{the lower bound for points on the plot,
  points beyond this are drawn as triangles at ymin}

  \item{genecol}{the color for gene-wise dispersion
  estimates}

  \item{fitcol}{the color of the fitted estimates}

  \item{finalcol}{the color of the final estimates used for
  testing}

  \item{legend}{logical, whether to draw a legend}

  \item{xlab}{xlab}

  \item{ylab}{ylab}

  \item{log}{log}

  \item{cex}{cex}

  \item{...}{further arguments to \code{plot}}
}
\description{
A simple helper function that plots the per-gene dispersion
estimates together with the fitted mean-dispersion
relationship. Internally, it is a wrapper for the 
pltoDispEsts method from DESeq2.
}

\docType{methods}
\name{counts}
\alias{counts}
\alias{counts,DEXSeqResults-method}
\title{Accessors for the 'counts' slot of a DEXSeqResults object.}
\usage{
\S4method{counts}{DEXSeqResults}(object,normalized=FALSE)
}
\arguments{
  \item{object}{a \code{DEXSeqResults} object.}

  \item{normalized}{logical indicating whether or not to
  divide the counts by the size factors or normalization
  factors before returning (normalization factors always
  preempt size factors)}
}
\value{
  an integer matrix
}
\description{
The counts slot holds the count data as a matrix of
non-negative integer count values, one row for each
observational unit (gene or the like), and one column for
each sample.
}
\examples{
  data(pasillaDEXSeqDataSet, package="pasilla")
  head( counts( dxd ))
}

\name{estimateExonFoldChanges}
\alias{estimateExonFoldChanges}
\title{Estimates exon usage coefficients from the fitted terms of the GLM.}
\description{
  This function calculates the exon usage coefficients and fold changes 
  (on log2 scale) between the different conditions.
}
\usage{
  estimateExonFoldChanges( object, 
     fitExpToVar = "condition", denominator = "",
     BPPARAM=SerialParam(), maxRowsMF=2400,
	 independentFiltering=FALSE, filter)

}
\arguments{
  \item{object}{
    An DEXSeqDataSet object.
  }
  \item{fitExpToVar}{
    A variable contained in \code{design(ecs)}. The expression 
    values will be fitted to this variable using the the formula 
    " ~ fitExpToVar * exon".
  }
  \item{denominator}{
    A value of the sample annotation (e.g. condition) to use as a denominator 
    in the log2 fold change. As a default, the function will take 
    the annotation of the first sample.
  }
  \item{BPPARAM}{
     A "BiocParallelParam" instance. 
     See \code{?bplapply} for details.
  }
  \item{maxRowsMF}{
     For the exon fold change estimation, the size of the model matrix 
     for the fitted glm increases with the number of samples and 
     the number of exons for a specific gene (see the DEXSeq paper
     for details). Since the glm fit for big models is very slow, 
     the maxRowsMF parameter allows to set a threshold on the number
     of rows from the model matrix (the number of rows of the model
     matrix will be number of samples times the number of exons of a
     gene). For all genes passing this threshold, the exon fold
     changes will be estimated by fitting a slightly different
     but equivalent model. The formula remains the same, but instead
     of fitting one model per gene that considers all its exons, it
     considers, for each exon, the counts from the specific exon and
     the sum of the rest of exons of the same gene.
  }
  \item{independentFiltering}{
    Logical indicating whether independent filtering should be
    applied automatically. For the exons that were discarded,
    fold changes won't be estimated.
  }
  \item{filter}{
    A vector of filter statistics over which the independent
    filtering will be optimized. The default is the normalized
    exon means.
  }
}
\details{
  Exon usage coefficients are calculated by fitting
  a GLM from the joint data of all exons of the same gene.
  The model frame can be found in the slot object@modelFrameBM
  of a \code{DEXSeqDataSet} object.  The model 
  '~ fitExpToVar * exon' is fitted.  
  The resulted coefficients are arranged and reformatted 
  in order to remove gene expression effects, leaving only exon 
  usage effects for each individual exon in each level of
  'fitExpToVar'. These values are used by the function
  \code{plotDEXSeq}. For specific details please check
  the DEXSeq paper.
}
\examples{
  data(pasillaDEXSeqDataSet, package="pasilla")
  dxd <- estimateSizeFactors( dxd )
  dxd <- estimateDispersions( dxd )
  dxd <- testForDEU( dxd )
  dxd <- estimateExonFoldChanges( dxd )
}

\name{featureCounts}
\alias{featureCounts}
\alias{featureIDs}
\alias{featureIDs<-}
\alias{exonIDs}
\alias{exonIDs<-}
\alias{groupIDs}
\alias{groupIDs<-}
\alias{geneIDs}
\alias{geneIDs<-}
\alias{sampleAnnotation}
\title{Accessor functions for DEXSeqDataSet details}
\description{
  Accessor functions of the DEXSeqDataSet object.
}
\usage{

  featureCounts(object, normalized=FALSE)

  featureIDs( object )
  featureIDs( object ) <- value

  exonIDs( object )
  exonIDs( object ) <- value

  groupIDs( object )
  groupIDs( object ) <- value

  geneIDs( object )
  geneIDs( object ) <- value

  sampleAnnotation( object )
  
}
\arguments{
  \item{object}{
    An DEXSeqDataSet object.
  }
  \item{value}{
    A character vector to replace previous values.
  }
  \item{normalized}{
    Logical indicating whether or not to
    divide the counts by the size factors or normalization
    factors before returning (normalization factors always
    preempt size factors)
  }
}
\value{
  'featureCounts' access the counts per exonic region or
  feature region names. 'featureIDs' and 'exonIDs' are accessor
  functions for the exon bin or features identifiers.
  'groupIDs' and 'geneIDs' are accessor functions for
  the character vector grouping the features, for example exonIDs
  from the same gene are grouped together by having the
  same value in the geneIDs. 'sampleAnnotation' is an accessor
  for the information from the samples.
}
\examples{
  data(pasillaDEXSeqDataSet, package="pasilla")
  head( featureCounts( dxd ) )
  head( featureIDs(dxd))
  head( exonIDs(dxd))
}

\name{plotDEXSeq}
\alias{plotDEXSeq}
\title{Visualization of the per gene DEXSeq results.}
\description{
  The function provides a plot to visualize read count data,
  expression and exon usage coefficients estimated from fitting
  a GLM model 'counts ~ fitExpToVar * exon'. The model frame used
  can be found in \code{object@modelFrameBM}. One GLM is fitted
  for each gene.
}
\usage{
  plotDEXSeq(object, geneID, FDR=0.1, fitExpToVar="condition", 
    norCounts=FALSE, expression=TRUE, splicing=FALSE, 
    displayTranscripts=FALSE, names=FALSE, legend=FALSE, 
    color=NULL, color.samples=NULL, transcriptDb=NULL,
    additionalAnnotation=NULL, maxRowsMF=2400, ...)
}
\arguments{
  \item{object}{
    A DEXSeqResults object. A DEXSeqDataSet object is also accepted, but
    only with the possibility of plotting the normalized read counts for
    each exon bin. 
  }
  \item{geneID}{
    Gene identifier to visualize.
  }
  \item{FDR}{
    A false discovery rate. 
  }
  \item{fitExpToVar}{
    A variable contained in the sample annotation of the \code{DEXSeqDataSet}, 
    the expression values will be fitted to this variable using the formula
    \code{count ~ fitExpToVar * exon}.  
  }
  \item{norCounts}{
    If TRUE, provides a plot of the counts normalized by the size factors.
  }
  \item{expression}{
    If TRUE, the function plots the fitted EXPRESSION estimates from the glm 
    regression.
  }
  \item{splicing}{
    If TRUE, the samples gene expression effects are averaged out, leaving
    only exon usage coefficients.
  }
  \item{displayTranscripts}{
    If TRUE, the transcripts are displayed in the plot.
  }
  \item{names}{
    If TRUE, the names of the transcripts are shown.
  }
  \item{legend}{
    If TRUE, a legend is displayed.
  }
  \item{color}{
    A vector of colors for each of the levels of the factor in the design of 
   the ExonCountSet object indicated by "fitExpToVar".
  }
  \item{color.samples}{
    A vector of colors for each of the samples. If NULL, the colors of each 
    sample will be assigned according to its corresponding level from "fitExpToVar". 
    This option is useful to visualize complex experimental designs.
  }
  \item{transcriptDb}{
    A TxDb object, as defined in the GenomicFeatures package. This parameter
    is optional. If present and if 'displayTranscripts=TRUE', the transcript
    UTRs will be distinguished from the coding regions.
  }
  \item{additionalAnnotation}{
    A GRangesList object specifying the genomic coordinates of
    annotation features that should be plotted. This parameter is
    useful, for example, when one wants to show protein domain
    annotations.
  }
  \item{maxRowsMF}{
    See \code{?estimateExonFoldChanges} for details.
 }

  \item{...}{
    Further graphical parameters (see \code{par}).
  }
}
\seealso{
  graphics, \code{segments}
}
\examples{
\dontrun{
  data(pasillaDEXSeqDataSet, package="pasilla")
  dxd <- estimateSizeFactors( dxd )
  dxd <- estimateDispersions( dxd )
  dxd <- testForDEU( dxd )
  dxr <- DEXSeqResults( dxd )

  plotDEXSeq( dxr, "FBgn0010909" )
}
}
